
;/////////////////////////////////////////////////////////////////////////////
;	Copyright (C) 2007 Enrique Ardanaz(ARDA), All Rights Reserved
;
;	This code returns L2 cache size.
;	It has only been tested under this filter; compatibily
;	is therefore not guaranteed.
;//............................................................................
;//............................................................................
;	This program is free software; you can redistribute it and/or modify
;	it under the terms of the GNU General Public License as published by
;	the Free Software Foundation.
;
;	This program is distributed in the hope that it will be useful,
;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;	GNU General Public License for more details.
;
;	You should have received a copy of the GNU General Public License
;	along with this program; if not, write to the Free Software
;	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;//............................................................................
;//............................................................................
;
;	The author can be contacted at:
;	ardanaz@excite.com
;
;/////////////////////////////////////////////////////////////////////////////

BITS 32
%macro cglobal 1 
	%ifdef PREFIX
		global _%1 
		%define %1 _%1
	%else
		global %1
	%endif
%endmacro

SECTION .text
;.....................................................
cglobal  getL2cache
;.....................................................
align 16
getL2cache:
		push ebx
		push edi

		mov				eax,080000000h
		cpuid

		cmp             eax,080000006h
        jb              classmethod

        mov             eax,080000006h
        cpuid

        shr				ecx,16			; discard lower 16 bits
        shl				ecx,10			; *1024

		pop	edi
		pop	ebx

		mov				eax,ecx			; needed for return
		mov				[L2Cache],ecx

		ret
align 16
classmethod:   
        mov     eax,2
        cpuid

        mov     esi,cache_sz
		mov		BYTE[counter],al

        shr     eax,8
        mov     [esi+0],eax
        mov     [esi+3],ebx
        mov     [esi+7],ecx
        mov     [esi+11],edx

        mov     edi,15

		dec     BYTE[counter]
		js		near nocache		; ??

		jz		near LookL2loop
;.....................................................
align 16
loopcache:
		add		esi,15
        mov     eax,2
        cpuid                       ;
        shr     eax,8               ;
        mov     [esi+0],eax
        mov     [esi+3],ebx         ;
        mov     [esi+7],ecx
        mov     [esi+11],edx
		add		edi,15
		dec     BYTE[counter]
		jnz		loopcache
;.....................................................
		mov   esi,cache_sz			;
align 16
LookL2loop:
        mov     dword eax,128

        cmp     BYTE [esi],0x41
        je      near done
        cmp     BYTE [esi],0x39
        je      near done
        cmp     BYTE [esi],0x3b
        je      near done
		cmp     BYTE [esi],0x79
        je      near done
        cmp     BYTE [esi],0x81
        je      near done
;.....................................................
        add		eax,64				;mov     dword eax,192

        cmp     BYTE [esi],0x3a
        je      near done
;.....................................................
        add		eax,64				;mov     dword eax,256

        cmp     BYTE [esi],0x42
        je      near done
        cmp     BYTE [esi],0x3c
        je      near done
        cmp     BYTE [esi],0x7a
        je      near done
        cmp     BYTE [esi],0x7e		;??	undocumented is it right??
        je      near done
        cmp     BYTE [esi],0x82
        je      near done
;.....................................................
        add		eax,128				;mov     dword eax,384

        cmp     BYTE [esi],0x3d
        je      near done
;.....................................................
        add		eax,128				;mov     dword eax,512

        cmp     BYTE [esi],0x43
        je      near done
        cmp     BYTE [esi],0x3e
        je      near done
        cmp     BYTE [esi],0x7b
        je      near done
        cmp     BYTE [esi],0x7f
        je      near done
        cmp     BYTE [esi],0x83
        je      near done
        cmp     BYTE [esi],0x86
        je      near done
;.....................................................
        add		eax,eax				;mov     dword eax,1024

        cmp     BYTE [esi],0x44
        je      near done
        cmp     BYTE [esi],0x78
        je      near done
        cmp     BYTE [esi],0x7c
        je      near done
        cmp     BYTE [esi],0x84
        je      near done
        cmp     BYTE [esi],0x87
        je      near done
;.....................................................
        add		eax,eax				;mov     dword eax,2048

        cmp     BYTE [esi],0x45
        je      near done
        cmp     BYTE [esi],0x7d
        je      near done
        cmp     BYTE [esi],0x85
        je      near done
;.....................................................
		add		eax,1024			;mov	dword eax,3072
		cmp		BYTE [esi],0x48		;??
		je		near done

		add		eax,1024			;mov	dword eax,4096
		cmp		BYTE [esi],0x49
		je		near done

		;here we have to test model and family
		;if Family F, Model 6 is Intel Xeon processor MP, with L3 cache, else L2 cache
		;if I don't check in certain cases it will consider L3 as L2 ?; does it matter?
		;as we test 0x49 at the end we probably already got L2 cache size before if Xeon?
		;else it is L2 of other processor.

		add		eax,2048			;mov dword eax,6144
		cmp		eax,0x4e
		je		near done

align 16
nextline:		
        inc     esi
        dec     edi
        jnz     LookL2loop			;if exits here L2 cache not detected

;exit with error
align 16
nocache:
		mov		eax,0xffffffff			;xor		eax,eax

		pop	edi
		pop	ebx

		mov     [L2Cache],eax
		mov		byte[alreadycache],1

        ret
;.....................................................
align 16
done:
		shl     eax,10				; *1024 (got bytes)

		pop	edi
		pop	ebx

		mov     [L2Cache],eax
		mov		byte[alreadycache],1

        ret
;-----------------------------------------------------
;-------------------------------------------------

SECTION .data

align 16
cache_sz		dd  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
				dd  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
				dd  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
				dd	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
align 16
alreadycache	db		0

align 16
counter			db		0

align 16
L2Cache			dd		0

