/*	*****************************************************************************************
	Three Flips plugins for Avisynth 2.5 Copyright (c) 2007 Enrique Ardanaz (ARDA)
	*****************************************************************************************
	Rotate180() Copyright (c) 2007 Enrique Ardanaz (ARDA)
	is an alternative version of the internal turn180().
	The objetive of this version was to test the perfomance of working over source (in place);
	when possible.
	Speed improvement can arrive till *3; depending on source size, colorspace and machine.
	It also works creating a new frame when necessary.
		
	In YUY2 colorspace for MMX machines I have adapted code from Dscaler 4.18
	Copyright (c) 2002 Rob Muller. FLT_Mirror.asm,v 1.1

	SSE2 and ISSE are my own versions. Copyright (c) Enrique Ardanaz (ARDA)

	*****************************************************************************************
	FHorizontal() Copyright (c) 2007 Enrique Ardanaz (ARDA)
	is an alternative version of the internal fliphorizontal().
	The objetive of this version was to test the perfomance of working over source(in place),
	when possible.
	Speed improvement can arrive till *3; depending on source size, colorspace and machine.
	It also works creating a new frame when necessary.

	In YUY2 colorspace for MMX machines I have adapted code from Dscaler 4.18 
	Copyright (c) 2002 Rob Muller. FLT_Mirror.asm,v 1.1

	SSE2 and ISSE are my own versions. Copyright (c) Enrique Ardanaz (ARDA)

	*****************************************************************************************
	FVertical() Copyright (c) 2007 Enrique Ardanaz (ARDA)
	is an alternative version of the internal flipvertical().
	The objetive of this version was to test the importance of working over source(in place),
	instead of using BitBlt.

	Speed improvement can arrive till *3; depending on source size, colorspace, and machine.
	It also works creating a new frame when necessary.

	*****************************************************************************************
	TODO:
	Some of plain assembler routines could probably be done with C++. Is it worth while?
	Fhorizontal SSE2 and ISSE codes for RGB24 colorspace??
	Rotate180 SSE2 and ISSE codes for RGB24 colorspace??
	Improve all plain asm for RGB24 colorspace.
	Open a branch when WC could increase perfomance. Done
	Tune prefetches.
	With some small changes they should work for any planar colorspace; need to be tested when 2.6
	Perfomance benchmarks to conclude if it is worth while so many specific cpu codes.
	There are a lot of duplicated code. Need to be cleaned.

	*****************************************************************************************
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	*****************************************************************************************
*/
/////////////////////////////////////////////////////////////////////////////////////////////
#include <windows.h>
#include "avisynth.h"
#include "getL2cache.h"
/////////////////////////////////////////////////////////////////////////////////////////////
class Rotate180 : public GenericVideoFilter
{
	public:
	Rotate180(PClip _child, IScriptEnvironment* env);
	~Rotate180();
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);

	private:
	bool use_SSE2;
	bool use_ISSE;
	bool use_MMX;
	bool DoInPlace;

	int  chsz2;
};
/////////////////////////////////////////////////////////////////////////////////////////////
Rotate180::Rotate180(PClip _child, IScriptEnvironment* env):
					GenericVideoFilter(_child)
{
//...........................................................................................
		use_SSE2 = (env->GetCPUFlags() & CPUF_SSE2) !=0;
		use_ISSE = (env->GetCPUFlags() & CPUF_INTEGER_SSE) !=0;
		use_MMX  = (env->GetCPUFlags() & CPUF_MMX) !=0;
//...........................................................................................
		if (vi.IsYUY2() && !use_MMX )
		env->ThrowError("Rotate180: YUY2 colorspace needs MMX cpu or higher");
//...........................................................................................
		if (vi.IsRGB32() && (vi.width<4) )use_SSE2 =false;	//????
//...........................................................................................
		chsz2=getL2cache();
		if (chsz2==0xffffffff)
		env->ThrowError("L2 cache size not detected; impossible to continue. Use Turn180.");
//...................unfinished.................................................................
		int bitppix= vi.BitsPerPixel();

		if (vi.IsYV12())
		{
			bitppix=1;
		}
		else if (!vi.IsPlanar())		//temporal
		{
			bitppix=bitppix>>3;
		}
		int pitchsize=vi.width*bitppix;
		if (pitchsize&15)pitchsize=((pitchsize>>4)+1)<<4;//dont know if it is worth while

		int framesize=((pitchsize)*vi.height);	//not always safe.

		int MaxInPlace=((chsz2<<2)+(chsz2<<1));		//chsz2*6. Still tunning

//...........................................................................................
		DoInPlace= ( framesize<MaxInPlace ) !=0;//need to be benchmarked and tunned
//...........................................................................................
}
Rotate180::~Rotate180() {}
/////////////////////////////////////////////////////////////////////////////////////////////
class FVertical : public GenericVideoFilter
{
	public:
	FVertical(PClip _child, IScriptEnvironment* env) ;
	~FVertical();
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);

	private:
	bool use_SSE2;
	bool use_ISSE;
	bool use_MMX;
	bool DoInPlace;

	int  chsz2;
};
/////////////////////////////////////////////////////////////////////////////////////////////
FVertical::FVertical(PClip _child, IScriptEnvironment* env):
					GenericVideoFilter(_child)
{
//...........................................................................................
		use_SSE2 = (env->GetCPUFlags() & CPUF_SSE2) !=0;
		use_ISSE = (env->GetCPUFlags() & CPUF_INTEGER_SSE) !=0;
		use_MMX  = (env->GetCPUFlags() & CPUF_MMX)  !=0;
//...........................................................................................
		chsz2=getL2cache();
		if (chsz2==0xffffffff)
		env->ThrowError("L2 cache size not detected; impossible to continue. Use FlipVertical.");
//...........................................................................................
		int bitppix= vi.BitsPerPixel();

		if (vi.IsYV12())
		{
			bitppix=1;
		}
		else if (!vi.IsPlanar())		//temporal
		{
			bitppix=bitppix>>3;
		}
		int pitchsize=vi.width*bitppix;
		if (pitchsize&15)pitchsize=((pitchsize>>4)+1)<<4;//dont know if it is worth while

		int framesize=((pitchsize)*vi.height);	//not always safe.

		int MaxInPlace=((chsz2<<2)+(chsz2<<1));		//chsz2*6 still tunning

//...........................................................................................
		DoInPlace= ( framesize<MaxInPlace ) !=0;//need to be benchmarked and tunned
//...........................................................................................
}

FVertical::~FVertical() {}
/////////////////////////////////////////////////////////////////////////////////////////////
class FHorizontal : public GenericVideoFilter
{
	public:
	FHorizontal(PClip _child, IScriptEnvironment* env) ;
	~FHorizontal();
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);

	private:
	bool use_SSE2;
	bool use_ISSE;
	bool use_MMX;
	bool DoInPlace;

	int  chsz2;
};
/////////////////////////////////////////////////////////////////////////////////////////////
FHorizontal::FHorizontal(PClip _child, IScriptEnvironment* env):
					GenericVideoFilter(_child)
{
//...........................................................................................
		use_SSE2 = (env->GetCPUFlags() & CPUF_SSE2) !=0;
		use_ISSE = (env->GetCPUFlags() & CPUF_INTEGER_SSE) !=0;
		use_MMX  = (env->GetCPUFlags() & CPUF_MMX) !=0;
//...........................................................................................
		if (vi.IsYUY2() && !use_MMX)
		env->ThrowError("FHorizontal: YUY2 needs MMX cpu or higher");
//...........................................................................................

		chsz2=getL2cache();
		if (chsz2==0xffffffff)
		env->ThrowError("L2 cache size not detected; impossible to continue. Use FlipHorizontal.");
//...........................................................................................
		int bitppix= vi.BitsPerPixel();

		if (vi.IsYV12())
		{
			bitppix=1;
		}
		else if (!vi.IsPlanar())		//temporal
		{
			bitppix=bitppix>>3;
		}
		int pitchsize=vi.width*bitppix;
		if (pitchsize&15)pitchsize=((pitchsize>>4)+1)<<4;//dont know if it is worth while

		int framesize=((pitchsize)*vi.height);	//not always safe.

		int MaxInPlace=((chsz2<<2)+(chsz2<<1));		//chsz2*6 still tunning

//...........................................................................................
		DoInPlace= ( framesize<MaxInPlace ) !=0;//need to be benchmarked and tunned
//...........................................................................................
}
FHorizontal::~FHorizontal() {}
/////////////////Copyright (c) 2007 Enrique Ardanaz (ARDA)////////////////////////////////////////
static void __declspec(naked) asm_rotate180_plain_row_1
			(BYTE* srcp,		//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr)			//20
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebx,[esp+16+12]		//height

			mov eax,[esp+16+20]		//last byte of last row
			mov ebp,ebx				//height

			mov edi,esi				//copy of ptr
			sar ebx,1

			and ebp,1				//seems right when odd lines (middle line)
			add edi,eax				//ptr to last byte of last row

			mov [esp+16+12],ebx
//...........................................................................................
align 16
y_loop:
			mov ecx,[esp+16+8]		//row
			mov ebx,ecx				//[esp+16+8]		//row

			sar ecx,2
			jz lower4
//...........................................................................................
			and ebx,3
align 16
x_loop:
			mov eax,[esi]
			mov edx,[edi-4]

			bswap eax
			bswap edx

			mov [edi-4],eax
			mov [esi],edx

			sub edi,4
			add esi,4

			sub ecx,1

			jne x_loop
//...........................................................................................
			test ebx,ebx
			jz nextline
align 16
lower4:
//...........................................................................................
			mov al,[esi]
			mov dl,[edi-1]

			sub ebx,1

			mov [edi-1],al
			mov [esi],dl

			lea edi,[edi-1]
			lea esi,[esi+1]

			jne lower4
//...........................................................................................
align 16
nextline:
			sub edi,[esp+16+16]					//modulo
			add esi,[esp+16+16]					//modulo

			sub dword ptr[esp+16+12],1
			jnz y_loop
//...........................................................................................
			test ebp,ebp
			jz end_loop
//...........................................................................................

			mov ecx,[esp+16+8]		//row
			mov ebx,[esp+16+8]		//row

//...........are we doing middle line ?....yes...............................................
			sar ecx,3
			jz lower8
//...........................................................................................
			and ebx,7
align 16
x_loop1:
			mov eax,[esi]
			mov edx,[edi-4]

			bswap eax
			bswap edx

			mov [edi-4],eax
			mov [esi],edx

			sub edi,4
			add esi,4

			sub ecx,1

			jne x_loop1
//...........................................................................................
			test ebx,ebx
			jz end_loop
//...........................................................................................
align 16
lower8:	
			mov al,[esi]
			mov dl,[edi-1]

			sub ebx,2

			mov [edi-1],al
			mov [esi],dl

			lea edi,[edi-1]
			lea esi,[esi+1]

			cmp ebx,2
			jge lower8
//...........................................................................................
align 16
end_loop:
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
/////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////////
static void __declspec(naked) asm_rotate180_isse_row_2
			(BYTE* srcp,		//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr)			//20
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebx,[esp+16+12]		//height
			mov eax,[esp+16+20]		//last byte of last row
			mov ebp,ebx				//height

			mov edi,esi				//copy of ptr
			sar ebx,1
			and ebp,1				//needed to test if there is a remainder middle line
			add edi,eax				//ptr to last byte of last row
			mov [esp+16+12],ebx		//height
//...........................................................................................
align 16
y_loop_0:	
			mov ecx,[esp+16+8]			//row

			mov ebx,ecx
			sar ecx,5
			jz x_loop_0

			and ebx,31
//...........................................................................................
align 16
ecxis8_0:
			movq mm0,[esi]
			movq mm2,[esi+8]
			movq mm4,[esi+16]
			movq mm6,[esi+24]

			pshufw	mm0,mm0,00011011b
			pshufw	mm2,mm2,00011011b
			pshufw	mm4,mm4,00011011b
			pshufw	mm6,mm6,00011011b

			movq    mm1,mm0
			movq    mm3,mm2
			movq	mm5,mm4
			movq	mm7,mm6

			psllw	mm0,8
			psrlw	mm1,8

			psllw	mm2,8
			psrlw	mm3,8

			psllw	mm4,8
			psrlw	mm5,8

			psllw	mm6,8
			psrlw	mm7,8

			por		mm0,mm1
			por		mm2,mm3
			por		mm4,mm5
			por		mm6,mm7
//...........................................................................................
			movq mm1,[edi-8]
			movq mm3,[edi-16]
			movq mm5,[edi-24]
			movq mm7,[edi-32]

			pshufw	mm1,mm1,00011011b
			pshufw	mm3,mm3,00011011b
			pshufw	mm5,mm5,00011011b
			pshufw	mm7,mm7,00011011b

			movq [edi-8],mm0
			movq [edi-16],mm2
			movq [edi-24],mm4
			movq [edi-32],mm6

			movq	mm0,mm1
			movq	mm2,mm3
			movq	mm4,mm5
			movq	mm6,mm7
			
			psllw	mm1,8
			psrlw	mm0,8

			psllw	mm3,8
			psrlw	mm2,8

			psllw	mm5,8
			psrlw	mm4,8

			psllw	mm7,8
			psrlw	mm6,8
			
			por		mm1,mm0
			por		mm3,mm2
			por		mm5,mm4
			por		mm7,mm6

			sub ecx,1

			movq [esi],mm1
			movq [esi+8],mm3
			movq [esi+16],mm5
			movq [esi+24],mm7

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne  ecxis8_0
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
			mov ecx,ebx

			sar ecx,3
			jz x_loop_0

			and ebx,7
//...........................................................................................
align 16
loop8:
			movq mm0,[esi]
			movq mm4,[edi-8]

			pshufw	mm0,mm0,00011011b
			pshufw	mm4,mm4,00011011b

			movq    mm1,mm0
			movq	mm5,mm4

			psllw	mm0,8
			psllw	mm4,8

			psrlw	mm1,8 
			psrlw	mm5,8

			por		mm0,mm1
			por		mm4,mm5

			movq [edi-8],mm0
			sub ecx,1
			movq [esi],mm4

			lea edi,[edi-8]
			lea esi,[esi+8]

			jnz loop8
//...........................................................................................
			test ebx,ebx
			jz nextline
align 16
x_loop_0:	
			mov al,[esi]
			mov dl,[edi-1]

			sub ebx,1

			mov [edi-1],al
			mov [esi],dl

			lea edi,[edi-1]
			lea esi,[esi+1]

			jne x_loop_0
align 16
nextline:
			sub edi,[esp+16+16]					//modulo
			add esi,[esp+16+16]					//modulo

			sub dword ptr[esp+16+12],1
			jne y_loop_0
//.......................falta height odd....................................................
			test ebp,ebp
			jz  end_loop
//...........................................................................................
			mov ecx,[esp+16+8]			//row

			mov ebx,ecx
			sar ecx,6
			jz lower64

			and ebx,63
align 16
loop64mirror:
			movq mm0,[esi]
			movq mm2,[esi+8]
			movq mm4,[esi+16]
			movq mm6,[esi+24]

			pshufw	mm0,mm0,00011011b
			pshufw	mm2,mm2,00011011b
			pshufw	mm4,mm4,00011011b
			pshufw	mm6,mm6,00011011b

			movq    mm1,mm0
			movq    mm3,mm2
			movq	mm5,mm4
			movq	mm7,mm6

			psllw	mm0,8
			psrlw	mm1,8

			psllw	mm2,8
			psrlw	mm3,8

			psllw	mm4,8
			psrlw	mm5,8

			psllw	mm6,8
			psrlw	mm7,8

			por		mm0,mm1
			por		mm2,mm3
			por		mm4,mm5
			por		mm6,mm7
//...........................................................................................
			movq mm1,[edi-8]
			movq mm3,[edi-16]
			movq mm5,[edi-24]
			movq mm7,[edi-32]

			pshufw	mm1,mm1,00011011b
			pshufw	mm3,mm3,00011011b
			pshufw	mm5,mm5,00011011b
			pshufw	mm7,mm7,00011011b

			movq [edi-8],mm0
			movq [edi-16],mm2
			movq [edi-24],mm4
			movq [edi-32],mm6

			movq	mm0,mm1
			movq	mm2,mm3
			movq	mm4,mm5
			movq	mm6,mm7
			
			psllw	mm1,8
			psrlw	mm0,8

			psllw	mm3,8
			psrlw	mm2,8

			psllw	mm5,8
			psrlw	mm4,8

			psllw	mm7,8
			psrlw	mm6,8
			
			por		mm1,mm0
			por		mm3,mm2
			por		mm5,mm4
			por		mm7,mm6

			sub ecx,1

			movq [esi],mm1
			movq [esi+8],mm3
			movq [esi+16],mm5
			movq [esi+24],mm7

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne  loop64mirror

			test ebx,ebx
			jz end_loop
//...........................................................................................			
			mov ecx,ebx
align 16
lower64:
			sar ecx,4
			jz lower16

			and ebx,15
align 16
looplower64:
			movq mm0,[esi]
			movq mm4,[edi-8]

			pshufw	mm0,mm0,00011011b
			pshufw	mm4,mm4,00011011b

			movq    mm1,mm0
			movq	mm5,mm4

			psllw	mm0,8
			psllw	mm4,8

			psrlw	mm1,8 
			psrlw	mm5,8

			por		mm0,mm1
			por		mm4,mm5

			movq [edi-8],mm0
			sub ecx,1
			movq [esi],mm4

			lea edi,[edi-8]
			lea esi,[esi+8]

			jnz looplower64

			test ebx,ebx
			jz end_loop
//....................................................			
			//mov ecx,ebx
align 16
lower16:
			mov al,[esi]
			mov dl,[edi-1]

			sub ebx,2

			mov [edi-1],al
			mov [esi],dl

			lea edi,[edi-1]
			lea esi,[esi+1]

			cmp ebx,2
			jge lower16
//...............here should add one byte when 2.6 for odds rows.............................
align 16
end_loop:
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////////
static void __declspec(naked) asm_rotate180_sse2_row_2
			(BYTE* srcp,		//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr)			//20
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebx,[esp+16+12]		//height
			mov eax,[esp+16+20]		//last byte of last row
			mov ebp,ebx				//height

			mov edi,esi				//copy of ptr
			sar ebx,1
			and ebp,1				//needed to test if there is a remainder middle line
			add edi,eax				//ptr to last byte of last row
			mov [esp+16+12],ebx		//height
align 16
testalign:
			mov ecx,[esp+16+8]		//ptr to src

			mov eax,esi
			or eax,edi

			test eax,15
			jnz y_loop_0
//...........................................................................................
align 16
loopaligned:
			mov ebx,ecx
			sar ecx,5
			jz preloop8

			and ebx,31//63
//...........................................................................................
align 16
loop64ecxal:
			movdqa xmm0,[esi]			//original B15B14B13B12 B11B10B09B08 B07B06B05B04 B03B02B01B00
										//			   DW3			DW2			DW1			DW0
										//         |.......HIGH WORDS.......||.......LOW WORDS.......|
			movdqa xmm2,[esi+16]

			pshufd	xmm0,xmm0,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW3
										//    |.......HIGH WORDS.......||.......LOW WORDS........|
			pshufd	xmm2,xmm2,00011011b// IDEM

										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm0,xmm0,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm0,xmm0,10110001B //GOT		B01B00 B03B02 B05B04 B07B06
										//high words..W7.. ..W6.. ..W5.. ..W4..
			pshuflw	xmm2,xmm2,10110001B
			pshufhw xmm2,xmm2,10110001B

			movdqa    xmm1,xmm0			//copy
			movdqa    xmm3,xmm2			//copy

			psllw	xmm0,8		//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm1,8		//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm0,xmm1	//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there are no QWSWAP and DQWSWAP instructions.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm2,8
			psrlw	xmm3,8
			por		xmm2,xmm3
//...........................................................................................
			movdqa xmm4,[edi-16]
			movdqa xmm6,[edi-32]

			pshufd	xmm4,xmm4,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW0
										//    |.......HIGH WORDS.......||.......LOW WORDS........|

			pshufd	xmm6,xmm6,00011011b// IDEM

										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm4,xmm4,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm4,xmm4,10110001B //GOT		B01B00 B03B02 B05B04 B07B06

			pshuflw	xmm6,xmm6,10110001B
			pshufhw xmm6,xmm6,10110001B

			movdqa    xmm5,xmm4			//copy
			movdqa    xmm7,xmm6			//copy

			psllw	xmm4,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm5,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm4,xmm5			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there is no an QWSWAP and DQWSWAP instruction.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm6,8
			psrlw	xmm7,8
			por		xmm6,xmm7
//...........................................................................................
			movdqa [edi-16],xmm0
			movdqa [edi-32],xmm2

			sub ecx,1

			movdqa [esi],xmm4
			movdqa [esi+16],xmm6

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne  loop64ecxal
//...........................................................................................
			jmp testremder
//...........................................................................................
align 16
y_loop_0:	test esi,15
			jz   preloop64ecx
align 16
test2align:
			test esi,3
			jz   test4align

			mov al,[esi]
			mov dl,[edi-1]

			sub ecx,1

			mov [edi-1],al
			mov [esi],dl

			lea edi,[edi-1]
			lea esi,[esi+1]

			jmp test2align
align 16
test4align: 
			test esi,15
			jz testbothptrs

			movq mm0,[esi]
			movq mm4,[edi-8]

			pshufw	mm0,mm0,00011011b
			pshufw	mm4,mm4,00011011b

			movq    mm1,mm0
			movq	mm5,mm4

			psllw	mm0,8
			psllw	mm4,8

			psrlw	mm1,8 
			psrlw	mm5,8

			por		mm0,mm1
			por		mm4,mm5

			movd [edi-4],mm0
			sub ecx,4
			movd [esi],mm4

			lea edi,[edi-4]
			lea esi,[esi+4]

			jmp test4align
align 16
testbothptrs:
			test edi,15
			jz	loopaligned
align 16
preloop64ecx:
			mov ebx,ecx
			sar ecx,5
			jz preloop8

			and ebx,31
//...........................................................................................
align 16
loop64ecx:
			movdqa xmm0,[esi]			//original B15B14B13B12 B11B10B09B08 B07B06B05B04 B03B02B01B00
										//			   DW3			DW2			DW1			DW0
										//         |.......HIGH WORDS.......||.......LOW WORDS.......|
			movdqa xmm2,[esi+16]

			pshufd	xmm0,xmm0,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW3
										//    |.......HIGH WORDS.......||.......LOW WORDS........|

			pshufd	xmm2,xmm2,00011011b// IDEM


										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm0,xmm0,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm0,xmm0,10110001B //GOT		B01B00 B03B02 B05B04 B07B06
										//high words..W7.. ..W6.. ..W5.. ..W4..
			pshuflw	xmm2,xmm2,10110001B
			pshufhw xmm2,xmm2,10110001B

			movdqa    xmm1,xmm0			//copy
			movdqa    xmm3,xmm2			//copies

			psllw	xmm0,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm1,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm0,xmm1			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there are no QWSWAP and DQWSWAP instructions.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm2,8
			psrlw	xmm3,8
			por		xmm2,xmm3
//...........................................................................................
			movdqu xmm4,[edi-16]
			movdqu xmm6,[edi-32]

			pshufd	xmm4,xmm4,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW0
										//    |.......HIGH WORDS.......||.......LOW WORDS........|

			pshufd	xmm6,xmm6,00011011b// IDEM


										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm4,xmm4,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm4,xmm4,10110001B //GOT		B01B00 B03B02 B05B04 B07B06

			pshuflw	xmm6,xmm6,10110001B
			pshufhw xmm6,xmm6,10110001B

			movdqa    xmm5,xmm4			//copy
			movdqa    xmm7,xmm6			//copy

			psllw	xmm4,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm5,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm4,xmm5			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there is no an QWSWAP and DQWSWAP instruction.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm6,8
			psrlw	xmm7,8
			por		xmm6,xmm7
//...........................................................................................
			movdqu [edi-16],xmm0
			movdqu [edi-32],xmm2

			sub ecx,1

			movdqa [esi],xmm4
			movdqa [esi+16],xmm6

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne  loop64ecx
//...........................................................................................
align 16
testremder:
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
preloop8:
			mov ecx,ebx
			sar ecx,3
			jz lower8rtate

			and ebx,7
//...........................................................................................
align 16
loop8:
			movq mm0,[esi]
			movq mm4,[edi-8]

			pshufw	mm0,mm0,00011011b
			pshufw	mm4,mm4,00011011b

			movq    mm1,mm0
			movq	mm5,mm4

			psllw	mm0,8
			psllw	mm4,8

			psrlw	mm1,8 
			psrlw	mm5,8

			por		mm0,mm1
			por		mm4,mm5

			movq [edi-8],mm0
			movq [esi],mm4

			sub ecx,1

			lea edi,[edi-8]
			lea esi,[esi+8]

			jnz loop8
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
lower8rtate:	
			mov al,[esi]
			mov dl,[edi-1]

			sub ebx,1

			mov [edi-1],al
			mov [esi],dl

			lea edi,[edi-1]
			lea esi,[esi+1]

			jne lower8rtate
align 16
nextline:
			sub edi,[esp+16+16]					//modulo
			add esi,[esp+16+16]					//modulo

			sub dword ptr[esp+16+12],1
			jne testalign
/////////////////////////////////////////////////////////////////////////////////////////////
			test ebp,ebp
			jz	 end_loop
/////////////////////////////////////////////////////////////////////////////////////////////
			mov ecx,[esp+16+8]		//row
			mov ebx,[esp+16+8]		//row
//...........are we doing middle line ?....yes...............................................
			mov eax,esi
			or eax,edi

			test eax,15
			jz loopalignedmirr
//...........................................................................................
			test esi,15
			jz   preloop64ecxmirr
align 16
test2alignmirr:
			test esi,3
			jz   test4alignmirr

			mov al,[esi]
			mov dl,[edi-1]

			sub ecx,2

			mov [edi-1],al
			mov [esi],dl

			lea edi,[edi-1]
			lea esi,[esi+1]

			jmp test2alignmirr
align 16
test4alignmirr: 
			test esi,15
			jz testbothptrsmirr

			movq mm0,[esi]
			movq mm4,[edi-8]

			pshufw	mm0,mm0,00011011b
			pshufw	mm4,mm4,00011011b

			movq    mm1,mm0
			movq	mm5,mm4

			psllw	mm0,8
			psllw	mm4,8

			psrlw	mm1,8 
			psrlw	mm5,8

			por		mm0,mm1
			por		mm4,mm5

			movd [edi-4],mm0
			sub ecx,8
			movd [esi],mm4

			lea edi,[edi-4]
			lea esi,[esi+4]

			jmp test4alignmirr
align 16
testbothptrsmirr:
			test edi,15
			jz	loopalignedmirr
align 16
preloop64ecxmirr:
			mov ebx,ecx
			sar ecx,6
			jz testmiddlinend

			and ebx,63
//...........................................................................................
align 16
loop64ecxmirr:
			movdqa xmm0,[esi]		//original B15B14B13B12 B11B10B09B08 B07B06B05B04 B03B02B01B00
									//			   DW3			DW2			DW1			DW0
									//         |.......HIGH WORDS.......||.......LOW WORDS.......|
			movdqa xmm2,[esi+16]

			pshufd	xmm0,xmm0,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW3
										//    |.......HIGH WORDS.......||.......LOW WORDS........|

			pshufd	xmm2,xmm2,00011011b// IDEM
										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm0,xmm0,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm0,xmm0,10110001B //GOT		B01B00 B03B02 B05B04 B07B06
										//high words..W7.. ..W6.. ..W5.. ..W4..
			pshuflw	xmm2,xmm2,10110001B
			pshufhw xmm2,xmm2,10110001B

			movdqa    xmm1,xmm0			//copy
			movdqa    xmm3,xmm2			//copies

			psllw	xmm0,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm1,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm0,xmm1			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there are no QWSWAP and DQWSWAP instructions.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm2,8
			psrlw	xmm3,8
			por		xmm2,xmm3
//...........................................................................................
			movdqu xmm4,[edi-16]
			movdqu xmm6,[edi-32]

			pshufd	xmm4,xmm4,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW0
										//    |.......HIGH WORDS.......||.......LOW WORDS........|

			pshufd	xmm6,xmm6,00011011b// IDEM


										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm4,xmm4,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm4,xmm4,10110001B //GOT		B01B00 B03B02 B05B04 B07B06

			pshuflw	xmm6,xmm6,10110001B
			pshufhw xmm6,xmm6,10110001B

			movdqa    xmm5,xmm4			//copy
			movdqa    xmm7,xmm6			//copy

			psllw	xmm4,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm5,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm4,xmm5			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there is no an QWSWAP and DQWSWAP instruction.
			//(shuffles and shifts)
//...........................................................................................
			psllw	xmm6,8
			psrlw	xmm7,8
			por		xmm6,xmm7
//...........................................................................................
			movdqu [edi-16],xmm0
			movdqu [edi-32],xmm2

			sub ecx,1

			movdqa [esi],xmm4
			movdqa [esi+16],xmm6

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne  loop64ecxmirr

			jmp testmiddlinend
align 16
loopalignedmirr:
			mov ebx,ecx
			sar ecx,6
			jz testmiddlinend

			and ebx,63
//...........................................................................................
align 16
loop64ecxalmirr:
			movdqa xmm0,[esi]			//original B15B14B13B12 B11B10B09B08 B07B06B05B04 B03B02B01B00
										//			   DW3			DW2			DW1			DW0
										//         |.......HIGH WORDS.......||.......LOW WORDS.......|
			movdqa xmm2,[esi+16]

			pshufd	xmm0,xmm0,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW3
										//    |.......HIGH WORDS.......||.......LOW WORDS........|
			pshufd	xmm2,xmm2,00011011b// IDEM

										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm0,xmm0,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm0,xmm0,10110001B //GOT		B01B00 B03B02 B05B04 B07B06
										//high words..W7.. ..W6.. ..W5.. ..W4..
			pshuflw	xmm2,xmm2,10110001B
			pshufhw xmm2,xmm2,10110001B

			movdqa    xmm1,xmm0			//copy
			movdqa    xmm3,xmm2			//copy

			psllw	xmm0,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm1,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm0,xmm1			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there are no QWSWAP and DQWSWAP instructions.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm2,8
			psrlw	xmm3,8
			por		xmm2,xmm3
//...........................................................................................
			movdqa xmm4,[edi-16]
			movdqa xmm6,[edi-32]

			pshufd	xmm4,xmm4,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW0
										//    |.......HIGH WORDS.......||.......LOW WORDS........|

			pshufd	xmm6,xmm6,00011011b// IDEM


										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm4,xmm4,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm4,xmm4,10110001B //GOT		B01B00 B03B02 B05B04 B07B06

			pshuflw	xmm6,xmm6,10110001B
			pshufhw xmm6,xmm6,10110001B

			movdqa    xmm5,xmm4			//copy
			movdqa    xmm7,xmm6			//copy

			psllw	xmm4,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm5,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm4,xmm5			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there is no an QWSWAP and DQWSWAP instruction.
			//(shuffles and shifts,	what have you thought?)
//...........................................................................................
			psllw	xmm6,8
			psrlw	xmm7,8
			por		xmm6,xmm7
//...........................................................................................
			movdqa [edi-16],xmm0
			movdqa [edi-32],xmm2

			sub ecx,1

			movdqa [esi],xmm4
			movdqa [esi+16],xmm6

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne  loop64ecxalmirr
align 16
testmiddlinend:			
			test ebx,ebx
			jz end_loop

			mov ecx,ebx
///////////////////////////////////////////////////////////////////////////////////////////
			sar ecx,3
			jz lower8
			and ebx,7
align 16
x_loop1:
			mov eax,[esi]
			mov edx,[edi-4]

			bswap eax
			bswap edx

			mov [edi-4],eax
			mov [esi],edx

			sub edi,4
			add esi,4

			sub ecx,1

			jne x_loop1
//...........................................................................................
			test ebx,ebx
			jz end_loop
//...........................................................................................
align 16
lower8:	
			mov al,[esi]
			mov dl,[edi-1]

			sub ebx,2

			mov [edi-1],al
			mov [esi],dl

			lea edi,[edi-1]
			lea esi,[esi+1]

			cmp ebx,2
			jge lower8
align 16
end_loop:
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////Copyright (c) 2007 Enrique Ardanaz (ARDA)////////////////////////////////////////
static void __declspec(naked) asm_rotate180_plain_row_1_dst
			(const BYTE* srcp,	//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr,			//20
			BYTE* dstp,			//24
			int dstmodulo,		//28
			int dstsizefr)		//32
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov edi,[esp+16+24]		//ptr to dst	
			mov eax,[esp+16+32]		//last byte of last row

			add edi,eax				//ptr to last byte of last row
//...........................................................................................
align 16
y_loop:
			mov ecx,[esp+16+8]		//row
			mov ebx,[esp+16+8]		//row

			sar ecx,3
			jz lower8
//...........................................................................................
			and ebx,7
align 16
x_loop:
			mov eax,[esi]
			mov edx,[esi+4]

			bswap eax
			bswap edx

			mov [edi-4],eax
			mov [edi-8],edx

			sub ecx,1

			lea esi,[esi+8]
			lea edi,[edi-8]

			jne x_loop
//...........................................................................................
			test ebx,ebx
			jz nextline
align 16
lower8:
			mov ecx,ebx
			sar ecx,2
			jz  lower4
//...........................................................................................
			and ebx,3
align 16
loop4:
			mov eax,[esi]

			bswap eax

			sub ecx,1

			mov [edi-4],eax

			lea esi,[esi+4]
			lea edi,[edi-4]

			jne loop4
align 16
lower4:
//...........................................................................................
			mov al,[esi]
			sub ebx,1

			mov [edi-1],al

			lea esi,[esi+1]
			lea edi,[edi-1]

			jne lower4
//...........................................................................................
align 16
nextline:
			add esi,[esp+16+16]					//modulo
			sub edi,[esp+16+28]					//modulo

			sub dword ptr [esp+16+12],1
			jnz y_loop

		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
/////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////////
static void __declspec(naked) asm_rotate180_isse_row_2_dst
			(const BYTE* srcp,	//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr,			//20
			BYTE* dstp,			//24
			int dstmodulo,		//28
			int dstsizefr)		//32
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov edi,[esp+16+24]		//ptr to dst	
			mov eax,[esp+16+32]		//last byte of last row

			add edi,eax				//ptr to last byte of last row
//...........................................................................................
align 16
y_loop_0:
			mov ecx,[esp+16+8]			//row
			mov ebx,ecx

			sar ecx,5
			jz pre_loop8

			and ebx,31//15
//...........................................................................................
align 16
ecxis8_0:
			movq mm0,[esi]
			movq mm2,[esi+8]
			movq mm4,[esi+16]
			movq mm6,[esi+24]

			pshufw	mm0,mm0,00011011b
			pshufw	mm2,mm2,00011011b
			pshufw	mm4,mm4,00011011b
			pshufw	mm6,mm6,00011011b

			movq    mm1,mm0
			movq    mm3,mm2
			movq	mm5,mm4
			movq	mm7,mm6

			psllw	mm0,8
			psrlw	mm1,8

			psllw	mm2,8
			psrlw	mm3,8

			psllw	mm4,8
			psrlw	mm5,8

			psllw	mm6,8
			psrlw	mm7,8

			por		mm0,mm1
			por		mm2,mm3
			por		mm4,mm5
			por		mm6,mm7
//...........................................................................................
			movq [edi-8],mm0
			movq [edi-16],mm2
			movq [edi-24],mm4
			movq [edi-32],mm6

			sub ecx,1

			lea esi,[esi+32]
			lea edi,[edi-32]

			jne  ecxis8_0
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
pre_loop8:
			mov ecx,ebx

			sar ecx,3
			jz x_loop_0

			and ebx,7
//...........................................................................................
align 16
loop8:
			movq mm0,[esi]
			pshufw	mm0,mm0,00011011b

			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1

			sub ecx,1

			movq [edi-8],mm0

			lea esi,[esi+8]
			lea edi,[edi-8]

			jnz loop8
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
x_loop_0:	
			mov al,[esi]

			sub ebx,1

			mov [edi-1],al

			lea edi,[edi-1]
			lea esi,[esi+1]

			jne x_loop_0
//...........................................................................................
align 16
nextline:
			add esi,[esp+16+16]					//modulo
			sub edi,[esp+16+28]					//modulo

			sub dword ptr [esp+16+12],1
			jne y_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////
static void __declspec(naked) asm_rotate180_sse2_row_2_dst
			(const BYTE* srcp,	//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr,			//20
			BYTE* dstp,			//24
			int dstmodulo,		//28
			int dstsizefr)		//32
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov edi,[esp+16+24]		//ptr to dst	
			mov eax,[esp+16+32]		//last byte of last row

			add edi,eax				//ptr to last byte of last row
//...........................................................................................
align 16
testalign:
			mov ecx,[esp+16+8]

			mov eax,esi
			or eax,edi

			test eax,15
			jnz y_loop_0
//...........................................................................................
align 16
loopaligned:
			mov ebx,ecx

			sar ecx,5//6
			jz testremder

			and ebx,31//63
//...........................................................................................
align 16
loop64ecxal:

			movdqa xmm0,[esi]			//original B15B14B13B12 B11B10B09B08 B07B06B05B04 B03B02B01B00
										//			   DW3			DW2			DW1			DW0
										//         |.......HIGH WORDS.......||.......LOW WORDS.......|
			movdqa xmm2,[esi+16]
			pshufd	xmm0,xmm0,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW3
										//    |.......HIGH WORDS.......||.......LOW WORDS........|
			pshufd	xmm2,xmm2,00011011b	// IDEM

										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm0,xmm0,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm0,xmm0,10110001B //GOT		B01B00 B03B02 B05B04 B07B06
										//high words..W7.. ..W6.. ..W5.. ..W4..
			pshuflw	xmm2,xmm2,10110001B
			pshufhw xmm2,xmm2,10110001B

			movdqa    xmm1,xmm0			//copies
			movdqa    xmm3,xmm2			//copy

			psllw	xmm0,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm1,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm0,xmm1			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there are no QWSWAP and DQWSWAP instructions.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm2,8
			psrlw	xmm3,8
			por		xmm2,xmm3
//...........................................................................................
			movdqa [edi-16],xmm0
			movdqa [edi-32],xmm2

			sub ecx,1

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne  loop64ecxal

			jmp testremder
//...........................................................................................
align 16
y_loop_0:	test esi,15
			jz   preloop64ecx
align 16
test2align:
			test esi,3
			jz   test4align

			mov al,[esi]

			sub ecx,1

			mov [edi-1],al

			lea edi,[edi-1]
			lea esi,[esi+1]

			jmp test2align
align 16
test4align: 
			test esi,15
			jz testbothptrs

			movq mm0,[esi]
			pshufw	mm0,mm0,00011011b
			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1
			movd [edi-4],mm0
			sub ecx,4

			lea esi,[esi+4]
			lea edi,[edi-4]

			jmp test4align
align 16
testbothptrs:
			test edi,15
			jz	loopaligned
align 16
preloop64ecx:
			mov ebx,ecx

			sar ecx,5//6
			jz testremder

			and ebx,31//63
//...........................................................................................
align 16
loop64ecx:

			movdqa xmm0,[esi]			//original B15B14B13B12 B11B10B09B08 B07B06B05B04 B03B02B01B00
										//			   DW3			DW2			DW1			DW0
										//         |.......HIGH WORDS.......||.......LOW WORDS.......|
			movdqa xmm2,[esi+16]

			pshufd	xmm0,xmm0,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW3
										//    |.......HIGH WORDS.......||.......LOW WORDS........|

			pshufd	xmm2,xmm2,00011011b// IDEM

										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm0,xmm0,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm0,xmm0,10110001B //GOT		B01B00 B03B02 B05B04 B07B06
										//high words..W7.. ..W6.. ..W5.. ..W4..
			pshuflw	xmm2,xmm2,10110001B
			pshufhw xmm2,xmm2,10110001B

			movdqa    xmm1,xmm0			//copy
			movdqa    xmm3,xmm2			//copies

			psllw	xmm0,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm1,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm0,xmm1			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there are no QWSWAP and DQWSWAP instructions.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm2,8
			psrlw	xmm3,8
			por		xmm2,xmm3
//...........................................................................................
			movdqu [edi-16],xmm0
			movdqu [edi-32],xmm2

			sub ecx,1

			lea esi,[esi+32]
			lea edi,[edi-32]

			jne  loop64ecx
//...........................................................................................
align 16
testremder:
			test ebx,ebx
			jz nextline
//...........................................................................................
			mov ecx,ebx
			sar ecx,3
			jz prex_loop_0

			and ebx,7
//...........................................................................................
align 16
loop8:
			movq mm0,[esi]

			pshufw	mm0,mm0,00011011b

			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1
			movq [edi-8],mm0
			sub ecx,1

			lea esi,[esi+8]
			lea edi,[edi-8]

			jnz loop8
//...........................................................................................
			test ebx,ebx
			jz nextline
//......................add a loop four bytes................................................
align 16
prex_loop_0:
//...........................................................................................
			mov ecx,ebx
			sar ecx,2
			jz x_loop_0
//...........................................................................................
			and ebx,3
//...........................................................................................
align 16
loop4:
			movq mm0,[esi]

			pshufw	mm0,mm0,00011011b

			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1
			movd [edi-4],mm0
			sub ecx,1

			lea esi,[esi+4]
			lea edi,[edi-4]

			jnz loop4
//...........................................................................................
			test ebx,ebx
			jz nextline
align 16
x_loop_0:	
			mov al,[esi]
			sub ebx,1
			mov [edi-1],al

			lea esi,[esi+1]
			lea edi,[edi-1]

			jne x_loop_0
//...........................................................................................
align 16
nextline:
			add esi,[esp+16+16]					//modulo
			sub edi,[esp+16+28]					//modulo

			sub dword ptr [esp+16+12],1
			jne testalign
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////////
static void __declspec(naked) asm_rotate180_isse_row_2_dst_nta
			(const BYTE* srcp,	//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr,			//20
			BYTE* dstp,			//24
			int dstmodulo,		//28
			int dstsizefr)		//32

{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ecx,[esp+16+8]		//row
			mov ebp,[esp+16+16]
			mov edi,[esp+16+24]		//ptr to dst	
			mov eax,[esp+16+32]		//last byte of last row
			add ebp,ecx
			add edi,eax				//ptr to last byte of last row
align 16
y_loop_0:	
			mov ecx,[esp+16+8]		//row
			mov ebx,ecx

			sar ecx,5
			//jz pre_loop8			// not needed

			and ebx,31
//...........................................................................................
align 16
ecxis8_0:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movq mm0,[esi]
			movq mm2,[esi+8]
			movq mm4,[esi+16]
			movq mm6,[esi+24]

			pshufw	mm0,mm0,00011011b
			pshufw	mm2,mm2,00011011b
			pshufw	mm4,mm4,00011011b
			pshufw	mm6,mm6,00011011b

			movq    mm1,mm0
			movq    mm3,mm2
			movq	mm5,mm4
			movq	mm7,mm6

			psllw	mm0,8
			psrlw	mm1,8

			psllw	mm2,8
			psrlw	mm3,8

			psllw	mm4,8
			psrlw	mm5,8

			psllw	mm6,8
			psrlw	mm7,8

			por		mm0,mm1
			por		mm2,mm3
			por		mm4,mm5
			por		mm6,mm7
//...........................................................................................
			movntq [edi-8],mm0
			movntq [edi-16],mm2
			movntq [edi-24],mm4
			movntq [edi-32],mm6

			lea esi,[esi+32]
			sub edi,32

			sub ecx,1
			jne  ecxis8_0
//...........................................................................................
			sfence
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
			mov ecx,ebx

			sar ecx,3
			jz x_loop_0

			and ebx,7
//...........................................................................................
align 16
loop8:
			movq	mm0,[esi]
			pshufw	mm0,mm0,00011011b
			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1
			movntq [edi-8],mm0

			lea esi,[esi+8]
			sub edi,8

			sub ecx,1
			jnz loop8

			sfence
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
x_loop_0:	
			mov al,[esi]
			sub ebx,1
			mov [edi-1],al

			lea edi,[edi-1]
			lea esi,[esi+1]

			jne x_loop_0
//...........................................................................................
align 16
nextline:
			add esi,[esp+16+16]					//modulo
			sub edi,[esp+16+28]					//modulo

			sub dword ptr [esp+16+12],1
			jne y_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////
static void __declspec(naked) asm_rotate180_sse2_row_2_dst_nta
			(const BYTE* srcp,	//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr,			//20
			BYTE* dstp,			//24
			int dstmodulo,		//28
			int dstsizefr)		//32
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov edi,[esp+16+24]		//ptr to dst	
			mov eax,[esp+16+32]		//last byte of last row
			mov ecx,[esp+16+8]		//row
			mov ebp,[esp+16+16]
			add edi,eax				//ptr to last byte of last row
			add ebp,ecx
//...........................................................................................
align 16
testalign:
			mov ecx,[esp+16+8]		//row

			mov eax,esi
			or eax,edi

			test eax,15
			jnz y_loop_0
//...........................................................................................
align 16
loopaligned:

			mov ebx,ecx

			sar ecx,5
			//jz x_loop_0		//not needed

			and ebx,31
//...........................................................................................
align 16
loop64ecxal:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movdqa xmm0,[esi]			//original B15B14B13B12 B11B10B09B08 B07B06B05B04 B03B02B01B00
										//			   DW3			DW2			DW1			DW0
										//         |.......HIGH WORDS.......||.......LOW WORDS.......|
			movdqa xmm2,[esi+16]

			pshufd	xmm0,xmm0,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW3
										//    |.......HIGH WORDS.......||.......LOW WORDS........|

			pshufd	xmm2,xmm2,00011011b// IDEM

										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..

			pshuflw	xmm0,xmm0,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm0,xmm0,10110001B //GOT		B01B00 B03B02 B05B04 B07B06

										//high words..W7.. ..W6.. ..W5.. ..W4..
			pshuflw	xmm2,xmm2,10110001B
			pshufhw xmm2,xmm2,10110001B

			movdqa    xmm1,xmm0			//copy
			movdqa    xmm3,xmm2			//copy

			psllw	xmm0,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm1,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm0,xmm1			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there are no QWSWAP and DQWSWAP instructions.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm2,8
			psrlw	xmm3,8
			por		xmm2,xmm3
//...........................................................................................
			movntdq [edi-16],xmm0
			movntdq [edi-32],xmm2
			lea esi,[esi+32]
			sub edi,32

			sub ecx,1
			jne  loop64ecxal

			sfence

			jmp testremder
//...........................................................................................
align 16
y_loop_0:	test edi,15
			jz   preloop64ecx
align 16
test2align:
			test edi,3
			jz   test4align

			mov al,[esi]
			sub ecx,1
			mov [edi-1],al
			lea esi,[esi+1]

			sub edi,1
			jmp test2align
align 16
test4align: 
			test edi,15
			jz testbothptrs

			movq mm0,[esi]
			pshufw	mm0,mm0,00011011b
			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1
			movd [edi-4],mm0

			sub ecx,4

			lea edi,[edi-4]
			lea esi,[esi+4]

			jmp test4align
align 16
testbothptrs:
			test esi,15
			jz	loopaligned
align 16
preloop64ecx:
			mov ebx,ecx
			sar ecx,5
			//jz x_loop_0
//...........................................................................................
			and ebx,31//63
//...........................................................................................
align 16
loop64ecx:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movdqu xmm0,[esi]			//original B15B14B13B12 B11B10B09B08 B07B06B05B04 B03B02B01B00
										//			   DW3			DW2			DW1			DW0
										//         |.......HIGH WORDS.......||.......LOW WORDS.......|
			movdqu xmm2,[esi+16]

			pshufd	xmm0,xmm0,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW3
										//    |.......HIGH WORDS.......||.......LOW WORDS........|

			pshufd	xmm2,xmm2,00011011b// IDEM

										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm0,xmm0,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm0,xmm0,10110001B //GOT		B01B00 B03B02 B05B04 B07B06
										//high words..W7.. ..W6.. ..W5.. ..W4..
			pshuflw	xmm2,xmm2,10110001B
			pshufhw xmm2,xmm2,10110001B

			movdqa    xmm1,xmm0			//copy
			movdqa    xmm3,xmm2			//copy

			psllw	xmm0,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm1,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm0,xmm1			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there are no QWSWAP and DQWSWAP instructions.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm2,8
			psrlw	xmm3,8
			por		xmm2,xmm3
//...........................................................................................
			movntdq [edi-16],xmm0
			movntdq [edi-32],xmm2

			lea esi,[esi+32]
			sub edi,32

			sub ecx,1
			jne  loop64ecx
//...........................................................................................
			sfence
align 16
testremder:
			test ebx,ebx
			jz nextline
//...........................................................................................
			mov ecx,ebx
			sar ecx,3
			jz prex_loop_0

			and ebx,7
//...........................................................................................
align 16
loop8:
			movq mm0,[esi]
			pshufw	mm0,mm0,00011011b
			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1
			movntq [edi-8],mm0

			lea esi,[esi+8]
			sub edi,8

			sub ecx,1
			jnz loop8
//...........................................................................................
			sfence

			test ebx,ebx
			jz nextline
//......................add a loop four bytes................................................
align 16
prex_loop_0:
//...........................................................................................
			mov ecx,ebx
			sar ecx,2
			jz x_loop_0
//...........................................................................................
			and ebx,3
//...........................................................................................
align 16
loop4:
			movq mm0,[esi]

			pshufw	mm0,mm0,00011011b

			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1
			movd [edi-4],mm0

			sub ecx,1

			lea esi,[esi+4]
			lea edi,[edi-4]

			jnz loop4
//...........................................................................................
			test ebx,ebx
			jz nextline
align 16
x_loop_0:	
			mov al,[esi]
			sub ebx,1
			mov [edi-1],al

			lea esi,[esi+1]
			lea edi,[edi-1]

			jne x_loop_0
//...........................................................................................
align 16
nextline:
			add esi,[esp+16+16]					//modulo
			sub edi,[esp+16+28]					//modulo
			sub dword ptr [esp+16+12],1
			jne testalign
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////////////Copyright (c)2007 Enrique Ardanaz (ARDA)////////////////////////////////
static void __declspec(naked) asm_rotate180_RGB24_2_dst
			(const BYTE* srcp,	//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr,			//20 //no se necesita
			BYTE* dstp,			//24
			int dstmodulo,		//28
			int dstsizefr)		//32
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebp,[esp+16+12]		//height
			mov edi,[esp+16+24]		//ptr to dst	
			mov eax,[esp+16+32]		//last byte of last row

			add edi,eax				//ptr to last byte of last row
//...........................................................................................
align 16
y_loop:
			mov ecx,[esp+16+8]		//row
//...........................................................................................
align 16
ecxis8_0:
			mov ax,[esi]
			mov bl,[esi+2]

			sub ecx,3

			mov [edi-3],ax
			mov [edi-1],bl

			lea esi,[esi+3]
			lea edi,[edi-3]

			cmp ecx,3
			jge ecxis8_0
//...........................................................................................
			sub edi,[esp+16+28]					//modulo
			add esi,[esp+16+16]					//modulo

			sub ebp,1
			jnz y_loop
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
/////////////////////////Copyright (c)2007 Enrique Ardanaz (ARDA)////////////////////////////////
static void __declspec(naked) asm_rotate180_RGB24_2
			(BYTE* srcp,		//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr)			//20
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebx,[esp+16+12]		//height
			mov eax,[esp+16+20]		//last byte of last row

			mov ebp,ebx				//height
			sar ebx,1

			mov edi,esi				//copy of ptr
			and ebp,1				//seems right for odd lines
			mov [esp+16+12],ebx

			add edi,eax				//ptr to last byte of last row
//...........................................................................................
align 16
y_loop:
			mov ecx,[esp+16+8]		//row
//...........................................................................................
align 16
ecxis8_0:
			mov ax,[esi]
			mov bl,[esi+2]

			mov dx,[edi-3]
			mov bh,[edi-1]

			sub ecx,3

			mov [edi-3],ax
			mov [edi-1],bl

			mov [esi],dx
			mov [esi+2],bh

			lea edi,[edi-3]
			lea esi,[esi+3]

			cmp ecx,3
			jge ecxis8_0
//...........................................................................................
			sub edi,[esp+16+16]		//modulo
			add esi,[esp+16+16]		//modulo

			sub dword ptr[esp+16+12],1
			jnz y_loop
//...........................................................................................
			test ebp,ebp
			jz end_loop
//...........................................................................................
			mov ecx,[esp+16+8]		//row
align 16
ecxis6_0:
			mov ax,[esi]
			mov bl,[esi+2]

			mov dx,[edi-3]
			mov bh,[edi-1]

			sub ecx,6

			mov [edi-3],ax
			mov [edi-1],bl

			mov [esi],dx
			mov [esi+2],bh

			lea edi,[edi-3]
			lea esi,[esi+3]

			cmp ecx,6
			jge ecxis6_0
//...........................................................................................
align 16
end_loop:
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
///////////////////////////Copyright (c)2007 Enrique Ardanaz (ARDA)//////////////////////////////
static void __declspec(naked) asm_rotate180_RGB32
			(BYTE* srcp,		//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr)			//20
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebx,[esp+16+12]		//height
			mov eax,[esp+16+20]		//last byte of last row

			mov ebp,ebx				//height

			mov edi,esi				//copy of ptr
			sar ebx,1
			and ebp,1				//seems right when odd lines (middle line)
			add edi,eax				//ptr to last byte of last row
//...........................................................................................
align 16
y_loop:
			mov ecx,[esp+16+8]		//row
			sar ecx,2
//...........................................................................................
align 16
ecxis8_0:
			mov eax,[esi]
			mov edx,[edi-4]

			sub ecx,1

			mov [edi-4],eax
			mov [esi],edx


			lea edi,[edi-4]
			lea esi,[esi+4]

			jne  ecxis8_0
//...........................................................................................
			sub edi,[esp+16+16]		//modulo
			add esi,[esp+16+16]		//modulo

			sub ebx,1
			jnz y_loop
//...........................................................................................
			test ebp,ebp
			jz end_loop
//...........................................................................................
			mov ecx,[esp+16+8]		//row

			sar ecx,3
align 16
ecxis8_00:
			mov eax,[esi]
			mov edx,[edi-4]

			mov [edi-4],eax
			mov [esi],edx

			sub ecx,1

			lea edi,[edi-4]
			lea esi,[esi+4]

			jne ecxis8_00
//...........................................................................................
align 16
end_loop:
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
////////////////////////Copyright (c)2007 Enrique Ardanaz (ARDA)/////////////////////////////////
static void __declspec(naked) SSE2_rotate180_RGB32_al
			(BYTE* srcp,		//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr)			//20
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebx,[esp+16+12]		//height
			mov eax,[esp+16+20]		//last byte of last row
			mov ebp,ebx				//height

			mov edi,esi				//copy of ptr
			sar ebx,1
			and ebp,1				//needed to test if there is a remainder middle line
			add edi,eax				//ptr to last byte of last row
//...........................................................................................
align 16
fstest:
			mov eax,esi
			or  eax,edi

			test eax,15
			jnz	preecxis8_0
//...........................................................................................
			mov ecx,[esp+16+8]		//row
			mov eax,[esp+16+8]		//row
align 16
y_loop0:
			sar	ecx,6
			jz endx1
//...........................................................................................
			and	eax,63
align 16
ecxis8al:							// can pshuf with memory, could it be useful?
			movdqa xmm2,[esi]
			movdqa xmm5,[esi+16]
			movdqa xmm1,[esi+32]
			movdqa xmm4,[esi+48]

			pshufd xmm2,xmm2,00011011b
			pshufd xmm5,xmm5,00011011b
			pshufd xmm1,xmm1,00011011b
			pshufd xmm4,xmm4,00011011b

			movdqa xmm3,[edi-16]
			movdqa xmm7,[edi-32]
			movdqa xmm0,[edi-48]
			movdqa xmm6,[edi-64]

			pshufd xmm3,xmm3,00011011b
			pshufd xmm7,xmm7,00011011b
			pshufd xmm0,xmm0,00011011b
			pshufd xmm6,xmm6,00011011b

			movdqa [edi-16],xmm2
			movdqa [edi-32],xmm5
			movdqa [edi-48],xmm1
			movdqa [edi-64],xmm4

			sub ecx,1

			movdqa [esi+00],xmm3
			movdqa [esi+16],xmm7
			movdqa [esi+32],xmm0
			movdqa [esi+48],xmm6

			lea edi,[edi-64]
			lea esi,[esi+64]

			jne  ecxis8al
//...........................................................................................
			jmp	 endx
align 16
preecxis8_0:
			mov ecx,[esp+16+8]		//row

			test esi,15
			jz	ecxis8_01
//...........................................................................................
align 16
lkf16:
			mov eax,[esi]
			mov edx,[edi-4]

			mov [edi-4],eax
			mov [esi],edx

			sub ecx,4

			lea edi,[edi-4]
			lea esi,[esi+4]

			test esi,15
			jnz lkf16
//...........................................................................................
align 16
ecxis8_01:
			mov	eax,ecx

			test edi,15
			jz	y_loop0
//...........................................................................................
			sar	ecx,6
			jz endx1

			and	eax,63
//...........................................................................................
align 16
ecxis8_0:
			movdqa xmm2,[esi]
			movdqa xmm5,[esi+16]
			movdqa xmm1,[esi+32]
			movdqa xmm4,[esi+48]

			pshufd xmm2,xmm2,00011011b
			pshufd xmm5,xmm5,00011011b
			pshufd xmm1,xmm1,00011011b
			pshufd xmm4,xmm4,00011011b

			movdqu xmm3,[edi-16]
			movdqu xmm7,[edi-32]
			movdqu xmm0,[edi-48]
			movdqu xmm6,[edi-64]

			pshufd xmm3,xmm3,00011011b
			pshufd xmm7,xmm7,00011011b
			pshufd xmm0,xmm0,00011011b
			pshufd xmm6,xmm6,00011011b

			movdqu [edi-16],xmm2
			movdqu [edi-32],xmm5
			movdqu [edi-48],xmm1
			movdqu [edi-64],xmm4

			sub ecx,1

			movdqa [esi+00],xmm3
			movdqa [esi+16],xmm7
			movdqa [esi+32],xmm0
			movdqa [esi+48],xmm6

			lea edi,[edi-64]
			lea esi,[esi+64]

			jne  ecxis8_0
//...........................................................................................
align 16
endx:		test eax,eax
			jz endx0
align 16
endx1:
			mov ecx,eax
			sar ecx,2
align 16
reminder:
			mov eax,[esi]
			mov edx,[edi-4]

			mov [edi-4],eax
			mov [esi],edx

			sub ecx,1

			lea edi,[edi-4]
			lea esi,[esi+4]

			jnz reminder
//...........................................................................................
align 16
endx0:
			sub edi,[esp+16+16]		//modulo
			add esi,[esp+16+16]		//modulo

			sub ebx,1
			jnz fstest
//...........................................................................................
			test ebp,ebp
			jz end_loop
//...........................................................................................
			mov ecx,[esp+16+8]		//row	we have one left middle line to process
			sar ecx,3
align 16
ecxis8_00:							// podemos hacer un loop de 32 o 16 vale la pena?
			mov eax,[esi]
			mov edx,[edi-4]

			mov [edi-4],eax
			mov [esi],edx

			sub ecx,1

			lea edi,[edi-4]
			lea esi,[esi+4]

			jne ecxis8_00
//...........................................................................................
align 16
end_loop:
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
////////////////////////Copyright (c)2007 Enrique Ardanaz (ARDA)/////////////////////////////////
static void __declspec(naked) SSE2_rotate180_RGB32_al_dst
			(const BYTE* srcp,		//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr,			//20//not needed 
			BYTE* dstp,			//24	
			int dstmodulo,		//28
			int dstsizefr)		//32
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebx,[esp+16+12]		//height
			mov eax,[esp+16+32]		//last byte of last row

			mov edi,[esp+16+24]		//ptr to dst
			add edi,eax				//ptr to last byte of last row dst
//...........................................................................................
align 16
fstest:
			mov eax,esi
			or  eax,edi

			test eax,15
			jnz	preecxis8_0
//...........................................................................................
			mov ecx,[esp+16+8]		//row
			mov eax,[esp+16+8]		//row
align 16
y_loop0:
			sar	ecx,7
			jz test16al
//...........................................................................................
			and	eax,127
align 16
ecxis8al:							// can pshuf with memory, could it be useful?
			movdqa xmm2,[esi]
			movdqa xmm5,[esi+16]
			movdqa xmm1,[esi+32]
			movdqa xmm4,[esi+48]

			pshufd xmm2,xmm2,00011011b
			pshufd xmm5,xmm5,00011011b
			pshufd xmm1,xmm1,00011011b
			pshufd xmm4,xmm4,00011011b

			movdqa xmm3,[esi+64]
			movdqa xmm7,[esi+80]
			movdqa xmm0,[esi+96]
			movdqa xmm6,[esi+112]

			pshufd xmm3,xmm3,00011011b
			pshufd xmm7,xmm7,00011011b
			pshufd xmm0,xmm0,00011011b
			pshufd xmm6,xmm6,00011011b

			movdqa [edi-16],xmm2
			movdqa [edi-32],xmm5
			movdqa [edi-48],xmm1
			movdqa [edi-64],xmm4

			movdqa [edi-80],xmm3
			movdqa [edi-96],xmm7
			movdqa [edi-112],xmm0
			movdqa [edi-128],xmm6

			sub ecx,1

			lea edi,[edi-128]
			lea esi,[esi+128]

			jne  ecxis8al
//...........................................................................................
			test eax,eax
			jz endx0
align 16
test16al:
			mov ecx,eax				//bug fixed 9feb 19:05
//...........................................................................................
			sar	ecx,4
			jz endx1
//...........................................................................................
			and	eax,15
align 16
loop16al:							// can pshuf with memory, could it be useful?
			movdqa xmm2,[esi]

			pshufd xmm2,xmm2,00011011b
			sub ecx,1

			movdqa [edi-16],xmm2

			lea esi,[esi+16]
			lea edi,[edi-16]

			jne  loop16al
//...........................................................................................
			jmp	 endx01
align 16
preecxis8_0:
			mov ecx,[esp+16+8]		//row

			test esi,15
			jz	ecxis8_01
//..........warning! presupone alineamiento a 4..................................................
align 16
lkf16:
			mov eax,[esi]
			sub ecx,4
			mov [edi-4],eax

			lea esi,[esi+4]
			lea edi,[edi-4]

			test esi,15
			jnz lkf16
//...........................................................................................
align 16
ecxis8_01:
			mov	eax,ecx

			test edi,15
			jz	y_loop0
//...........................................................................................
			sar	ecx,7
			jz test16unal
//...........................................................................................
			and	eax,127
//...........................................................................................
align 16
ecxis8_0:
			movdqa xmm2,[esi]
			movdqa xmm5,[esi+16]
			movdqa xmm1,[esi+32]
			movdqa xmm4,[esi+48]

			pshufd xmm2,xmm2,00011011b
			pshufd xmm5,xmm5,00011011b
			pshufd xmm1,xmm1,00011011b
			pshufd xmm4,xmm4,00011011b

			movdqa xmm3,[esi+64]
			movdqa xmm7,[esi+80]
			movdqa xmm0,[esi+96]
			movdqa xmm6,[esi+112]

			pshufd xmm3,xmm3,00011011b
			pshufd xmm7,xmm7,00011011b
			pshufd xmm0,xmm0,00011011b
			pshufd xmm6,xmm6,00011011b

			movdqu [edi-16],xmm2
			movdqu [edi-32],xmm5
			movdqu [edi-48],xmm1
			movdqu [edi-64],xmm4


			movdqu [edi-80],xmm3
			movdqu [edi-96],xmm7
			movdqu [edi-112],xmm0
			movdqu [edi-128],xmm6

			sub ecx,1

			lea edi,[edi-128]
			lea esi,[esi+128]

			jne  ecxis8_0
//...........................................................................................
			test eax,eax
			jz endx0
align 16
test16unal:
			mov ecx,eax				 //bug fixed 9feb 19:07
			sar ecx,4
			jz endx1

			and eax,15
//...........................................................................................
align 16
loop16unal:							// can pshuf with memory, could it be useful?
			movdqa xmm2,[esi]

			pshufd xmm2,xmm2,00011011b
			sub ecx,1

			movdqu [edi-16],xmm2

			lea esi,[esi+16]
			lea edi,[edi-16]

			jne  loop16unal
//...........................................................................................
align 16
endx01:
			test eax,eax
			jz endx0
//...........................................................................................
align 16
endx1:
			mov ecx,eax
			sar ecx,2
align 16
reminder:
			mov eax,[esi]
			sub ecx,1
			mov [edi-4],eax
			lea esi,[esi+4]
			lea edi,[edi-4]

			jnz reminder
//...........................................................................................
align 16
endx0:
			sub edi,[esp+16+28]		//modulodst
			add esi,[esp+16+16]		//modulo

			sub ebx,1
			jnz fstest
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
////////////////////////Copyright (c)2007 Enrique Ardanaz (ARDA)/////////////////////////////////
static void __declspec(naked) SSE2_rotate180_RGB32_al_dst_nta
			(const BYTE* srcp,	//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr,			//20//not needed 
			BYTE* dstp,			//24	
			int dstmodulo,		//28
			int dstsizefr)		//32
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebx,[esp+16+12]		//height
			mov ebp,[esp+16+16]		//modulo
			mov eax,[esp+16+32]		//last byte of last row

			mov edi,[esp+16+24]		//ptr to dst
			add ebp,[esp+16+8]		//pitch
			add edi,eax				//ptr to last byte of last row dst
//...........................................................................................
align 16
fstest:
			mov eax,esi
			or  eax,edi

			test eax,15
			jnz	preecxis8_0
//...........................................................................................
			mov ecx,[esp+16+8]		//row
			mov eax,[esp+16+8]		//row
align 16
y_loop0:
			sar	ecx,7
			//jz endx1
						
//...........................................................................................
			and	eax,127
align 16
ecxis8al:							// can pshuf with memory, could it be useful?

			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movdqa xmm2,[esi]
			movdqa xmm5,[esi+16]
			movdqa xmm1,[esi+32]
			movdqa xmm4,[esi+48]

			pshufd xmm2,xmm2,00011011b
			pshufd xmm5,xmm5,00011011b
			pshufd xmm1,xmm1,00011011b
			pshufd xmm4,xmm4,00011011b

			movdqa xmm3,[esi+64]
			movdqa xmm7,[esi+80]
			movdqa xmm0,[esi+96]
			movdqa xmm6,[esi+112]

			pshufd xmm3,xmm3,00011011b
			pshufd xmm7,xmm7,00011011b
			pshufd xmm0,xmm0,00011011b
			pshufd xmm6,xmm6,00011011b

			movntdq [edi-16],xmm2
			movntdq [edi-32],xmm5
			movntdq [edi-48],xmm1
			movntdq [edi-64],xmm4

			movntdq [edi-80],xmm3
			movntdq [edi-96],xmm7
			movntdq [edi-112],xmm0
			movntdq [edi-128],xmm6

			lea esi,[esi+128]
			sub edi,128

			sub ecx,1
			jne  ecxis8al

			sfence
//...........................................................................................
			test eax,eax
			jz endx0

			mov ecx,eax				//bug fixed 9 feb 19:13
//...........................................................................................
			sar	ecx,4
			jz endx1
//...........................................................................................
			and	eax,15
align 16
loop16al:							// can pshuf with memory, could it be useful?
			movdqa xmm2,[esi]
			pshufd xmm2,xmm2,00011011b
			movntdq [edi-16],xmm2

			lea esi,[esi+16]
			sub edi,16

			sub ecx,1
			jne  loop16al

			sfence
//...........................................................................................
			jmp	 endx01
align 16
preecxis8_0:
			mov ecx,[esp+16+8]		//row

			test edi,15
			jz	ecxis8_01
//..........warning! presupone alineamiento a 4..................................................
align 16
lkf16:
			mov eax,[esi]
			sub ecx,4
			mov [edi-4],eax

			lea esi,[esi+4]
			lea edi,[edi-4]

			test edi,15
			jnz lkf16
//...........................................................................................
align 16
ecxis8_01:
			mov	eax,ecx

			test esi,15
			jz	y_loop0
//...........................................................................................
			sar	ecx,7
			//jz endx1
//...........................................................................................
			and	eax,127//63
//...........................................................................................
align 16
ecxis8_0:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movdqu xmm2,[esi]
			movdqu xmm5,[esi+16]
			movdqu xmm1,[esi+32]
			movdqu xmm4,[esi+48]

			pshufd xmm2,xmm2,00011011b
			pshufd xmm5,xmm5,00011011b
			pshufd xmm1,xmm1,00011011b
			pshufd xmm4,xmm4,00011011b

			movdqu xmm3,[esi+64]
			movdqu xmm7,[esi+80]
			movdqu xmm0,[esi+96]
			movdqu xmm6,[esi+112]

			pshufd xmm3,xmm3,00011011b
			pshufd xmm7,xmm7,00011011b
			pshufd xmm0,xmm0,00011011b
			pshufd xmm6,xmm6,00011011b

			movntdq [edi-16],xmm2
			movntdq [edi-32],xmm5
			movntdq [edi-48],xmm1
			movntdq [edi-64],xmm4

			movntdq [edi-80],xmm3
			movntdq [edi-96],xmm7
			movntdq [edi-112],xmm0
			movntdq [edi-128],xmm6

			lea esi,[esi+128]
			sub edi,128
			sub ecx,1

			jne  ecxis8_0
//...........................................................................................
			test eax,eax
			jz endx0
//...........................................................................................
			mov ecx,eax				// bug fixed 9 feb 19:16
			sar ecx,4
			jz endx1

			and eax,15
align 16
loop16unal:							// can pshuf with memory, could it be useful?
			movdqu xmm2,[esi]
			pshufd xmm2,xmm2,00011011b
			movntdq [edi-16],xmm2

			lea esi,[esi+16]
			sub edi,16

			sub ecx,1
			jne  loop16unal

			sfence
//...........................................................................................
align 16
endx01:
			test eax,eax
			jz endx0
//...........................................................................................
align 16
endx1:
			mov ecx,eax
			sar ecx,2
align 16
reminder:							//is this right??
			mov eax,[esi]
			sub ecx,1
			mov [edi-4],eax
			lea esi,[esi+4]
			lea edi,[edi-4]

			jnz reminder
//...........................................................................................
align 16
endx0:
			sub edi,[esp+16+28]		//modulodst
			add esi,[esp+16+16]		//modulo

			sub ebx,1
			jnz fstest
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
/////////////////////////Copyright (c)2007 Enrique Ardanaz (ARDA)////////////////////////////////
static void __declspec(naked) MMX_rotate180_RGB32_1_dst
			(const BYTE* srcp,	//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr,			//20// eliminar no se usa
			BYTE* dstp,			//24	
			int dstmodulo,		//28
			int dstsizefr)		//32
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebx,[esp+16+12]		//height
			mov eax,[esp+16+32]		//last byte of last row

			mov edi,[esp+16+24]		//ptr to dst
			add edi,eax				//ptr to last byte of last row dst
//...........................................................................................
align 16
y_loop:
			mov ecx,[esp+16+8]		//row
			mov eax,[esp+16+8]		//row

			sar ecx,5
			jz endx1

			and eax,31//15
//..............study again this algo........................................................
align 16
ecxis8_0:
			movd mm0,[esi]
			movd mm2,[esi+4]
			movd mm4,[esi+8]
			movd mm5,[esi+12]

			psllq mm0,32
			psllq mm4,32

			movd mm1,[esi+16]
			movd mm3,[esi+20]
			movd mm6,[esi+24]
			movd mm7,[esi+28]

			psllq mm1,32
			psllq mm6,32

			por mm0,mm2
			por mm4,mm5
			por mm1,mm3
			por mm6,mm7

			movq [edi-8],mm0
			movq [edi-16],mm4
			movq [edi-24],mm1
			movq [edi-32],mm6

			sub ecx,1

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne  ecxis8_0
//...........................................................................................
			test eax,eax
			jz nextscline
align 16
endx1:
			mov ecx,eax
			sar ecx,2
align 16
reminder:
			mov eax,[esi]
			sub ecx,1
			mov [edi-4],eax

			lea esi,[esi+4]
			lea edi,[edi-4]

			jnz reminder
align 16
nextscline:
			sub edi,[esp+16+28]		//modulodst
			add esi,[esp+16+16]		//modulo

			sub ebx,1
			jnz y_loop
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////
static void __declspec(naked) MMX_rotate180_RGB32_1
			(BYTE* srcp,		//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr)			//20
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebx,[esp+16+12]		//height
			mov eax,[esp+16+20]		//last byte of last row
			mov ebp,ebx				//height

			mov edi,esi				//copy of ptr
			sar ebx,1
			and ebp,1				//seems right when odd lines (middle line)
			add edi,eax				//ptr to last byte of last row
//...........................................................................................
align 16
y_loop:
			mov ecx,[esp+16+8]		//row
			mov eax,[esp+16+8]		//row

			sar ecx,4
			jz endx1
//...........................................................................................
			and eax,15
//...........................................................................................
align 16
ecxis8_0:
			movd mm0,[esi]
			movd mm2,[esi+4]
			movd mm4,[esi+8]
			movd mm5,[esi+12]

			psllq mm0,32
			psllq mm4,32

			movd mm1,[edi-8]
			movd mm3,[edi-4]
			movd mm6,[edi-16]
			movd mm7,[edi-12]

			psllq mm1,32
			psllq mm6,32

			por mm0,mm2
			por mm4,mm5
			por mm1,mm3
			por mm6,mm7

			movq [edi-8],mm0
			movq [edi-16],mm4

			sub ecx,1

			movq [esi],mm1
			movq [esi+8],mm6

			lea edi,[edi-16]
			lea esi,[esi+16]

			jne  ecxis8_0
//...........................................................................................
			test eax,eax
			jz nextscline
align 16
endx1:
			mov ecx,eax
			sar ecx,2
align 16
reminder:
			mov eax,[esi]
			mov edx,[edi-4]

			mov [edi-4],eax
			mov [esi],edx

			sub ecx,1

			lea edi,[edi-4]
			lea esi,[esi+4]

			jnz reminder
align 16
nextscline:
			sub edi,[esp+16+16]		//modulo
			add esi,[esp+16+16]		//modulo

			sub ebx,1
			jnz y_loop
//...........................................................................................
			test ebp,ebp
			jz end_loop
//...........................................................................................
			mov ecx,[esp+16+8]		//row

			sar ecx,3
align 16
ecxis8_00:
			mov eax,[esi]
			mov edx,[edi-4]

			mov [edi-4],eax
			mov [esi],edx

			sub ecx,1

			lea edi,[edi-4]
			lea esi,[esi+4]

			jne  ecxis8_00
//...........................................................................................
align 16
end_loop:
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
///////////////////////////Copyright (c) Enrique Ardanaz (ARDA)//////////////////////////////
static void __declspec(naked) asm_rotate180_RGB32_dst
						(const BYTE* srcp,		//4
						int row,			//8
						int height,			//12
						int modulo,			//16
						int sizefr,			//20 // not needed anymore
						BYTE* dstp,			//24	
						int dstmodulo,		//28
						int dstsizefr)		//32
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebp,[esp+16+12]		//height
			mov eax,[esp+16+32]		//last byte of last row

			mov edi,[esp+16+24]		//ptr to dst
			add edi,eax				//ptr to last byte of last row dst
//...........................................................................................
align 16
y_loop:
			mov ecx,[esp+16+8]		//row
			mov edx,[esp+16+8]		//row

			sar ecx,3
			jz preloop4
//...........................................................................................
			and edx,7
//...........................................................................................
align 16
ecxis8_0:
			mov eax,[esi]
			mov ebx,[esi+4]

			sub ecx,1

			mov [edi-4],eax
			mov [edi-8],ebx

			lea esi,[esi+4]
			lea edi,[edi-4]

			jne  ecxis8_0
//...........................................................................................
			test edx,edx
			jz	endloopx
align 16
preloop4:	
			sar edx,2

align 16
loop4:
			mov eax,[esi]
			sub edx,1
			mov [edi-4],eax

			lea esi,[esi+4]
			lea edi,[edi-4]

			jnz loop4
align 16
endloopx:
			sub edi,[esp+16+28]		//modulodst
			add esi,[esp+16+16]		//modulo

			sub ebp,1
			jnz y_loop
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
/////////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////
static void __declspec(naked) ISSE_rotate180_RGB32_1_dst
			(const BYTE* srcp,		//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr,			//20 // not needed anymore
			BYTE* dstp,			//24	
			int dstmodulo,		//28
			int dstsizefr)		//32
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebx,[esp+16+12]		//height
			mov eax,[esp+16+32]		//last byte of last row

			mov edi,[esp+16+24]		//ptr to dst
			add edi,eax				//ptr to last byte of last row dst
//...........................................................................................
align 16
y_loop:
			mov ecx,[esp+16+8]		//row
			mov eax,[esp+16+8]		//row

			sar ecx,5
			jz endx1
//...........................................................................................
			and eax,31
align 16
ecxis8_0:
			movq mm2,[esi]
			movq mm5,[esi+8]
			movq mm3,[esi+16]
			movq mm7,[esi+24]

			pshufw mm2,mm2,01001110b
			pshufw mm5,mm5,01001110b

			pshufw mm3,mm3,01001110b
			pshufw mm7,mm7,01001110b

			movq [edi-8],mm2
			movq [edi-16],mm5
			movq [edi-24],mm3
			movq [edi-32],mm7

			sub ecx,1

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne  ecxis8_0
//...........................................................................................
			test eax,eax
			jz nextscline
align 16
endx1:
			mov ecx,eax
			sar ecx,2
align 16
reminder:
			mov eax,[esi]
			sub ecx,1
			mov [edi-4],eax

			lea esi,[esi+4]
			lea edi,[edi-4]

			jnz reminder
//...........................................................................................
align 16
nextscline:
			sub edi,[esp+16+28]		//modulodst
			add esi,[esp+16+16]		//modulo

			sub ebx,1
			jnz y_loop
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////
static void __declspec(naked) ISSE_rotate180_RGB32_1_dst_nta
			(const BYTE* srcp,		//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr,			//20 // not needed anymore
			BYTE* dstp,			//24	
			int dstmodulo,		//28
			int dstsizefr)		//32
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebx,[esp+16+12]		//height
			mov ebp,[esp+16+16]		//modulo
			mov eax,[esp+16+32]		//last byte of last row

			mov edi,[esp+16+24]		//ptr to dst
			add ebp,[esp+16+8]		//pitch
			add edi,eax				//ptr to last byte of last row dst
//...........................................................................................
align 16
y_loop:
			mov ecx,[esp+16+8]		//row
			mov eax,[esp+16+8]		//row

			sar ecx,5
			//jz endx1				//not needed
//...........................................................................................
			and eax,31
align 16
ecxis8_0:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movq mm2,[esi]
			movq mm5,[esi+8]
			movq mm3,[esi+16]
			movq mm7,[esi+24]

			pshufw mm2,mm2,01001110b
			pshufw mm5,mm5,01001110b

			pshufw mm3,mm3,01001110b
			pshufw mm7,mm7,01001110b

			movntq [edi-8],mm2
			movntq [edi-16],mm5
			movntq [edi-24],mm3
			movntq [edi-32],mm7

			lea esi,[esi+32]
			sub edi,32

			sub ecx,1
			jne  ecxis8_0
			sfence
//...........................................................................................
			test eax,eax
			jz nextscline
//align 16
//endx1:
			mov ecx,eax
			sar ecx,2
align 16
reminder:
			mov eax,[esi]
			sub ecx,1
			mov [edi-4],eax

			lea esi,[esi+4]
			lea edi,[edi-4]

			jnz reminder
//...........................................................................................
align 16
nextscline:
			sub edi,[esp+16+28]		//modulodst
			add esi,[esp+16+16]		//modulosrc

			sub ebx,1
			jnz y_loop
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////
static void __declspec(naked) ISSE_rotate180_RGB32_1
			(BYTE* srcp,		//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr)			//20
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebx,[esp+16+12]		//height
			mov eax,[esp+16+20]		//last byte of last row
			mov ebp,ebx				//height

			mov edi,esi				//copy of ptr
			sar ebx,1
			and ebp,1				//needed to test a remainder middle left line
			add edi,eax				//ptr to last byte of last row
//...........................................................................................
align 16
y_loop:
			mov ecx,[esp+16+8]		//row
			mov eax,[esp+16+8]		//row

			sar ecx,4
			jz endx1
//...........................................................................................
			and eax,15
align 16
ecxis8_0:
			movq mm2,[esi]
			movq mm5,[esi+8]

			pshufw mm2,mm2,01001110b
			pshufw mm5,mm5,01001110b

			movq mm3,[edi-8]
			movq mm7,[edi-16]

			pshufw mm3,mm3,01001110b
			pshufw mm7,mm7,01001110b

			movq [edi-8],mm2
			movq [edi-16],mm5

			sub ecx,1

			movq [esi],mm3
			movq [esi+8],mm7

			lea edi,[edi-16]
			lea esi,[esi+16]

			jne  ecxis8_0
//...........................................................................................
			test eax,eax
			jz nextscline
align 16
endx1:
			mov ecx,eax
			sar ecx,2
align 16
reminder:
			mov eax,[esi]
			mov edx,[edi-4]

			mov [edi-4],eax
			mov [esi],edx

			sub ecx,1

			lea edi,[edi-4]
			lea esi,[esi+4]

			jnz reminder
//...........................................................................................
align 16
nextscline:
			sub edi,[esp+16+16]		//modulo
			add esi,[esp+16+16]		//modulo

			sub ebx,1
			jnz y_loop
//.....................we are doing a middle left line......yes..............................
			test ebp,ebp
			jz end_loop
//................here we can use mmx for a bigger loop is it worth while?...................
			mov ecx,[esp+16+8]		//row
			sar ecx,3
align 16
ecxis8_00:
			mov eax,[esi]
			mov edx,[edi-4]

			mov [edi-4],eax
			mov [esi],edx

			sub ecx,1

			lea edi,[edi-4]
			lea esi,[esi+4]

			jne ecxis8_00
//...........................................................................................
//if there is four bytes left is the central pixel ; we do nothing...........................
align 16
end_loop:
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
////////////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////
static void __declspec(naked) asm_sse2_yuy2_180
					(BYTE* srcp,		//4
					int row,			//8
					int height,			//12
					int modulo,			//16
					int sizefr)			//20
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov			eax,0xFF00FF00		//mask
			movd		mm5,eax
			punpckldq	mm5,mm5
			movq2dq		xmm5,mm5
			punpckldq	xmm5,xmm5

			ror eax,8						//mask

			movd mm4,	eax
			punpckldq	mm4,mm4
			movq2dq		xmm4,mm4
			punpckldq	xmm4,xmm4
//...........................................................................................
			mov			esi,[esp+16+4]		//ptr to src
			mov			ebx,[esp+16+12]		//height
			mov			eax,[esp+16+20]		//last byte of last row
			mov			ebp,ebx				//height
//...........................................................................................
			mov			edi,esi				//copy of ptr
			sar			ebx,1				//ok
			and			ebp,1				//seems right for a middle left line
			add			edi,eax				//ptr to last byte of last row
//...........................................................................................
align 16
y_loop:
			mov			ecx,[esp+16+8]		//row
			mov			eax,[esp+16+8]		//row
//...........................................................................................
			cmp			ecx,16
			jl			testreminder0
//...........................................................................................
			test		esi,15
			jz			esiis16
//...........................................................................................
align 16
x_loop_01:
			movq		mm0,[esi]
			movq		mm3,[edi-8]

			psllq		mm0,32

			pshufw		mm1,mm3,00011011b 
			pshufw		mm3,mm3,01001110b 

			pshufw		mm2,mm0,00011011b 
			pshufw		mm0,mm0,01001110b 

			pand		mm2,mm4					//keep chromas,discard lumas
			pand		mm0,mm5					//keep lumas,discard chromas
			pand		mm1,mm4					//keep chromas,discard lumas
			pand		mm3,mm5					//keep lumas,discard chromas

			por			mm0,mm2					//join all
			por			mm3,mm1					//join all

			movd		[edi-4],mm0
			movd		[esi],mm3

			sub			ecx,4

			lea			edi,[edi-4]
			lea			esi,[esi+4]

			test		esi,15
			jnz			x_loop_01
//...........................................................................................
align 16
esiis16:
			test		edi,15
			jz			bothaligned
//...........................................................................................
			mov			eax,ecx
			sar			ecx,4
			jz			testreminder0				//need to be tested 9/1/07
//...........................................................................................
			and			eax,15
//...........................................................................................
align 16
srcptraligned:
			movdqa		xmm0,[esi]
			movdqu		xmm3,[edi-16]
//...........................................................................................
			pshufd		xmm0,xmm0,01001110b
			pshufd		xmm3,xmm3,01001110b

			movdqa		xmm2,xmm0
			movdqa		xmm1,xmm3
//...........................................................................................
			pshuflw		xmm2,xmm2,00011011b
			pshufhw		xmm2,xmm2,00011011b

			pshuflw		xmm0,xmm0,01001110b
			pshufhw		xmm0,xmm0,01001110b
//...........................................................................................
			pshuflw		xmm1,xmm1,00011011b
			pshufhw		xmm1,xmm1,00011011b 

			pshuflw		xmm3,xmm3,01001110b
			pshufhw		xmm3,xmm3,01001110b 
//...........................................................................................
			pand		xmm2,xmm4
			pand		xmm0,xmm5

			pand		xmm1,xmm4
			pand		xmm3,xmm5

			por			xmm0,xmm2						// join all
			por			xmm3,xmm1						// join all

			movdqu		[edi-16],xmm0
			movdqa		[esi],xmm3

			sub			ecx,1

			lea			edi,[edi-16]
			lea			esi,[esi+16]

			jne			srcptraligned
//...........................................................................................
			jmp			testreminder
//...........................................................................................
align 16
bothaligned:
			mov			eax,ecx
			sar			ecx,4
			jz			testreminder0
//...........................................................................................
			and			eax,15
//...........................................................................................
align 16
ecxis8_0:
			movdqa xmm0,[esi]
			movdqa xmm3,[edi-16]
//...........................................................................................
			pshufd xmm0,xmm0,01001110b
			pshufd xmm3,xmm3,01001110b

			movdqa xmm2,xmm0
			movdqa xmm1,xmm3
//...........................................................................................
			pshuflw xmm2,xmm2,00011011b
			pshufhw xmm2,xmm2,00011011b

			pshuflw xmm0,xmm0,01001110b
			pshufhw xmm0,xmm0,01001110b
//...........................................................................................
			pshuflw xmm1,xmm1,00011011b
			pshufhw xmm1,xmm1,00011011b

			pshuflw xmm3,xmm3,01001110b
			pshufhw xmm3,xmm3,01001110b 
//...........................................................................................
			pand	xmm2,xmm4
			pand	xmm0,xmm5

			pand	xmm1,xmm4
			pand	xmm3,xmm5

			por		xmm0,xmm2						 // join all
			por		xmm3,xmm1						 // join all

			movdqa [edi-16],xmm0
			movdqa [esi],xmm3

			sub ecx,1

			lea edi,[edi-16]
			lea esi,[esi+16]

			jne ecxis8_0
//...........................................................................................
align 16
testreminder:
			test eax,eax
			jz nextline
//...........................................................................................
align 16
testreminder0:
			mov ecx,eax
			sar ecx,2								// we can add here a control for lower than 4??
			and eax,3
align 16
lastbytes:
			movq mm0,[esi]
			movq mm3,[edi-8]

			psllq mm0,32							// to take advantage of same code

			pshufw mm1,mm3,00011011b 
			pshufw mm3,mm3,01001110b 

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4
			pand	mm0,mm5
			pand	mm1,mm4
			pand	mm3,mm5

			por mm0,mm2								// join all
			por mm3,mm1								// join all

			movd [edi-4],mm0
			movd [esi],mm3

			sub ecx,1

			lea edi,[edi-4]
			lea esi,[esi+4]

			jnz lastbytes
//...........................................................................................
			test eax,eax
			jz nextline
//...........................................................................................
													// if this happens; is there a bug??
			add esi,eax
			sub edi,eax								// eax should only be two. Unfinished??
//...........................................................................................
align 16
nextline:
			sub edi,[esp+16+16]						// modulo
			add esi,[esp+16+16]						// modulo

			sub ebx,1
			jnz y_loop
//...........................................................................................
			test ebp,ebp
			jz end_loop
//............just when height is odd....a middle left line..................................

			mov ecx,[esp+16+8]						//row
//...........................................................................................
			test esi,15
			jz  ecxis16_0
//...........................................................................................
align 16
x_loop_01mid:
			movq mm0,[esi]
			movq mm3,[edi-8]

			psllq mm0,32

			pshufw mm1,mm3,00011011b 
			pshufw mm3,mm3,01001110b 

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4
			pand	mm0,mm5
			pand	mm1,mm4
			pand	mm3,mm5

			por mm0,mm2							// join all
			por mm3,mm1							// join all

			movd [edi-4],mm0
			movd [esi],mm3

			sub ecx,8

			lea edi,[edi-4]
			lea esi,[esi+4]

			test esi,15
			jnz x_loop_01mid
//...........................................................................................
align 16
ecxis16_0:
			test edi,15
			jz bothaligned_0
//...........................................................................................
			mov	eax,ecx
			sar ecx,5
			jz	tstremidl0
//...........................................................................................
			and eax,31
align 16
srcalmid:
			movdqa xmm0,[esi]
			movdqu xmm3,[edi-16]
//...........................................................................................
			pshufd xmm0,xmm0,01001110b
			pshufd xmm3,xmm3,01001110b

			movdqa xmm2,xmm0
			movdqa xmm1,xmm3
//...........................................................................................
			pshuflw xmm2,xmm2,00011011b
			pshufhw xmm2,xmm2,00011011b

			pshuflw xmm0,xmm0,01001110b
			pshufhw xmm0,xmm0,01001110b
//...........................................................................................
			pshuflw xmm1,xmm1,00011011b
			pshufhw xmm1,xmm1,00011011b

			pshuflw xmm3,xmm3,01001110b
			pshufhw xmm3,xmm3,01001110b
//...........................................................................................
			pand	xmm2,xmm4
			pand	xmm0,xmm5

			pand	xmm1,xmm4
			pand	xmm3,xmm5

			por		xmm0,xmm2					// join all
			por		xmm3,xmm1					// join all

			movdqu [edi-16],xmm0
			movdqa [esi],xmm3

			sub ecx,1

			lea edi,[edi-16]
			lea esi,[esi+16]

			jne srcalmid
//...........................................................................................
			jmp tstremidl
//...........................................................................................
align 16
bothaligned_0:
			mov	eax,ecx
			sar ecx,5
			jz	tstremidl0
//...........................................................................................
			and eax,31
//...........................................................................................
align 16
ecxis8_0mid:
			movdqa xmm0,[esi]
			movdqa xmm3,[edi-16]
//...........................................................................................
			pshufd xmm0,xmm0,01001110b
			pshufd xmm3,xmm3,01001110b

			movdqa xmm2,xmm0
			movdqa xmm1,xmm3
//...........................................................................................
			pshuflw xmm2,xmm2,00011011b
			pshufhw xmm2,xmm2,00011011b

			pshuflw xmm0,xmm0,01001110b
			pshufhw xmm0,xmm0,01001110b
//...........................................................................................
			pshuflw xmm1,xmm1,00011011b
			pshufhw xmm1,xmm1,00011011b

			pshuflw xmm3,xmm3,01001110b
			pshufhw xmm3,xmm3,01001110b
//...........................................................................................
			pand	xmm2,xmm4
			pand	xmm0,xmm5

			pand	xmm1,xmm4
			pand	xmm3,xmm5

			por		xmm0,xmm2
			por		xmm3,xmm1

			movdqa [edi-16],xmm0
			movdqa [esi],xmm3

			sub ecx,1

			lea edi,[edi-16]
			lea esi,[esi+16]

			jne ecxis8_0mid
//...........................................................................................
align 16
tstremidl:
			test eax,eax
			jz end_loop
//.....................added code for mod2...................................................
align 16
tstremidl0:
			mov ecx,eax
			sar eax,3
			jz lower8
//...........................................................................................
			and ecx,7
//...........................................................................................
align 16
ctrmidline:
			movq mm0,[esi]
			movq mm3,[edi-8]

			psllq mm0,32

			pshufw mm1,mm3,00011011b 
			pshufw mm3,mm3,01001110b 

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4
			pand	mm0,mm5
			pand	mm1,mm4
			pand	mm3,mm5

			por mm0,mm2							// join all
			por mm3,mm1							// join all

			movd [edi-4],mm0
			movd [esi],mm3

			sub eax,1

			lea edi,[edi-4]
			lea esi,[esi+4]

			jne ctrmidline
//...........................................................................................
			test ecx,ecx
			jz end_loop
//...........................................................................................

align 16
lower8:		sar ecx,1

			mov al,[esi]
			mov dl,[edi-2]

			mov [edi-2],al
			mov [esi],dl

			sub edi,1			//cambiar
			add esi,1

			dec ecx				//cambiar
			jz end_loop
//...........................................................................................
			mov al,[esi]
			mov dl,[edi-2]

			mov [edi-2],al
			mov [esi],dl

			sub edi,1
			add esi,1

			dec ecx
			jz end_loop
//...........................................................................................
			mov al,[esi]
			mov dl,[edi-2]

			mov [edi-2],al
			mov [esi],dl
//...........................................................................................
align 16
end_loop:
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
////////////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////
static void __declspec(naked) asm_sse2_yuy2_180_dst
					(const BYTE* srcp,	//4
					int row,			//8
					int height,			//12 cuidado debe venir entera
					int modulo,			//16
					int sizefr,			//20//not needed??
					BYTE* dstp,			//24
					int dstmodulo,		//28
					int dstsizefr)		//32
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov			eax,0xFF00FF00		//mask
			movd		mm5,eax
			punpckldq	mm5,mm5
			movq2dq		xmm5,mm5
			punpckldq	xmm5,xmm5

			ror eax,8						//mask

			movd		mm4,eax
			punpckldq	mm4,mm4
			movq2dq		xmm4,mm4
			punpckldq	xmm4,xmm4
//...........................................................................................
			mov			esi,[esp+16+4]		//ptr to src
			mov			eax,[esp+16+32]		//last byte of last row
			mov			ebx,[esp+16+12]		//height
//...........................................................................................
			mov			edi,[esp+16+24]		
			add			edi,eax				//ptr to last byte of last row
//...........................................................................................
align 16
y_loop:
			mov			ecx,[esp+16+8]		//row
			mov			eax,[esp+16+8]		//row
//...........................................................................................
			cmp			ecx,16
			jl			testreminder0
//...........................................................................................
			test		esi,15
			jz			esiis16
//...........................................................................................
align 16
x_loop_01:
			movq		mm0,[esi]

			psllq		mm0,32

			pshufw		mm2,mm0,00011011b 
			pshufw		mm0,mm0,01001110b 

			pand		mm2,mm4					//keep chromas,discard lumas
			pand		mm0,mm5					//keep lumas,discard chromas

			por			mm0,mm2					//join all

			movd		[edi-4],mm0
			sub			ecx,4
			lea			esi,[esi+4]
			lea			edi,[edi-4]

			test		esi,15
			jnz			x_loop_01
//...........................................................................................
align 16
esiis16:
			test		edi,15
			jz			bothaligned
//...........................................................................................
			mov			eax,ecx
			sar			ecx,5
			jz			testreminder0				//need to be tested 9/1/07
//...........................................................................................
			and			eax,31
//...........................................................................................
align 16
srcptraligned:
			movdqa		xmm0,[esi]
			movdqa		xmm3,[esi+16]
//...........................................................................................
			pshufd		xmm0,xmm0,01001110b
			pshufd		xmm3,xmm3,01001110b

			movdqa		xmm2,xmm0
			movdqa		xmm1,xmm3
//...........................................................................................
			pshuflw		xmm2,xmm2,00011011b
			pshufhw		xmm2,xmm2,00011011b

			pshuflw		xmm0,xmm0,01001110b
			pshufhw		xmm0,xmm0,01001110b
//...........................................................................................
			pshuflw		xmm1,xmm1,00011011b
			pshufhw		xmm1,xmm1,00011011b 

			pshuflw		xmm3,xmm3,01001110b
			pshufhw		xmm3,xmm3,01001110b 
//...........................................................................................
			pand		xmm2,xmm4
			pand		xmm0,xmm5

			pand		xmm1,xmm4
			pand		xmm3,xmm5

			por			xmm0,xmm2						 // join all
			por			xmm3,xmm1						 // join all

			movdqu		[edi-16],xmm0
			movdqu		[edi-32],xmm3

			sub			ecx,1

			lea			edi,[edi-32]					//bug fixed 8 feb 12:28 pm
			lea			esi,[esi+32]

			jne			srcptraligned
//...........................................................................................
			jmp			testreminder
//...........................................................................................
align 16
bothaligned:
			mov			eax,ecx
			sar			ecx,5
			jz			testreminder0
//...........................................................................................
			and			eax,31
//...........................................................................................
align 16
ecxis8_0:
			movdqa xmm0,[esi]
			movdqa xmm3,[esi+16]
//...........................................................................................
			pshufd xmm0,xmm0,01001110b
			pshufd xmm3,xmm3,01001110b

			movdqa xmm2,xmm0
			movdqa xmm1,xmm3
//...........................................................................................
			pshuflw xmm2,xmm2,00011011b
			pshufhw xmm2,xmm2,00011011b

			pshuflw xmm0,xmm0,01001110b
			pshufhw xmm0,xmm0,01001110b
//...........................................................................................
			pshuflw xmm1,xmm1,00011011b
			pshufhw xmm1,xmm1,00011011b

			pshuflw xmm3,xmm3,01001110b
			pshufhw xmm3,xmm3,01001110b 
//...........................................................................................
			pand	xmm2,xmm4
			pand	xmm0,xmm5

			pand	xmm1,xmm4
			pand	xmm3,xmm5

			por		xmm0,xmm2						 // join all
			por		xmm3,xmm1						 // join all

			movdqa [edi-16],xmm0
			movdqa [edi-32],xmm3

			sub ecx,1

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne ecxis8_0
//...........................................................................................
align 16
testreminder:
			test eax,eax
			jz nextline
//...........................................................................................
align 16
testreminder0:
			mov ecx,eax
			sar ecx,2								// we can add here a control for lower than 4??
			and eax,3
align 16
lastbytes:
			movq mm0,[esi]

			psllq mm0,32							// to take advantage of same code


			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4
			pand	mm0,mm5

			por mm0,mm2								// join all

			movd [edi-4],mm0

			sub ecx,1

			lea edi,[edi-4]
			lea esi,[esi+4]

			jnz lastbytes
//...........................................................................................
			test eax,eax
			jz nextline
													//if this happens; is there a bug??
			add esi,eax
			sub edi,eax								// eax should only be two. Unfinished??
//...........................................................................................
align 16
nextline:
			add esi,[esp+16+16]						// modulosrc
			sub edi,[esp+16+28]						// modulodst

			sub ebx,1
			jnz y_loop
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
////////////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////
static void __declspec(naked) asm_sse2_yuy2_180_dst_nta
					(const BYTE* srcp,	//4
					int row,			//8
					int height,			//12	//cuidado debe venir entera
					int modulo,			//16
					int sizefr,			//20	//not needed??
					BYTE* dstp,			//24
					int dstmodulo,		//28
					int dstsizefr)		//32
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov			eax,0xFF00FF00		//mask
			movd		mm5,eax
			punpckldq	mm5,mm5
			movq2dq		xmm5,mm5
			punpckldq	xmm5,xmm5

			ror eax,8						//mask

			movd mm4,	eax
			punpckldq	mm4,mm4
			movq2dq		xmm4,mm4
			punpckldq	xmm4,xmm4
//...........................................................................................
			mov			esi,[esp+16+4]		//ptr to src
			mov			ecx,[esp+16+8]		//row
			mov			eax,[esp+16+32]		//last byte of last row
			mov			ebx,[esp+16+12]		//height
			mov			ebp,[esp+16+16]		//modulosrc
//...........................................................................................
			mov			edi,[esp+16+24]		
			add			edi,eax				//ptr to last byte of last row
			add			ebp,ecx
//...........................................................................................
align 16
y_loop:

			mov			ecx,[esp+16+8]		//row
			mov			eax,[esp+16+8]		//row
//...........................................................................................
			//cmp			ecx,16
			//jl			testreminder0	//innecesario

//...........................................................................................
			test		edi,15
			jz			esiis16
//...........................................................................................
align 16
x_loop_01:
			movq		mm0,[esi]

			psllq		mm0,32

			pshufw		mm2,mm0,00011011b 
			pshufw		mm0,mm0,01001110b 

			pand		mm2,mm4					//keep chromas,discard lumas
			pand		mm0,mm5					//keep lumas,discard chromas

			por			mm0,mm2					//join all

			movd		[edi-4],mm0
			sub			ecx,4
			lea			esi,[esi+4]
			lea			edi,[edi-4]

			test		edi,15
			jnz			x_loop_01
//...........................................................................................
align 16
esiis16:
			test		esi,15
			jz			bothaligned
//...........................................................................................
			mov			eax,ecx
			sar			ecx,5
			//jz			testreminder0			//innecesario
//...........................................................................................
			and			eax,31
//...........................................................................................
align 16
srcptraligned:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movdqu		xmm0,[esi]
			movdqu		xmm3,[esi+16]
//...........................................................................................
			pshufd		xmm0,xmm0,01001110b
			pshufd		xmm3,xmm3,01001110b

			movdqa		xmm2,xmm0
			movdqa		xmm1,xmm3
//...........................................................................................
			pshuflw		xmm2,xmm2,00011011b
			pshufhw		xmm2,xmm2,00011011b

			pshuflw		xmm0,xmm0,01001110b
			pshufhw		xmm0,xmm0,01001110b
//...........................................................................................
			pshuflw		xmm1,xmm1,00011011b
			pshufhw		xmm1,xmm1,00011011b 

			pshuflw		xmm3,xmm3,01001110b
			pshufhw		xmm3,xmm3,01001110b 
//...........................................................................................
			pand		xmm2,xmm4
			pand		xmm0,xmm5

			pand		xmm1,xmm4
			pand		xmm3,xmm5

			por			xmm0,xmm2						 // join all
			por			xmm3,xmm1						 // join all

			movntdq		[edi-16],xmm0
			movntdq		[edi-32],xmm3

			lea			esi,[esi+32]
			sub			edi,32
			sub			ecx,1

			jne			srcptraligned
			sfence
//...........................................................................................
			jmp			testreminder
//...........................................................................................
align 16
bothaligned:
			mov			eax,ecx
			sar			ecx,5
			//jz			testreminder0				//not needed
//...........................................................................................
			and			eax,31
//...........................................................................................
align 16
ecxis8_0:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movdqa xmm0,[esi]
			movdqa xmm3,[esi+16]
//...........................................................................................
			pshufd xmm0,xmm0,01001110b
			pshufd xmm3,xmm3,01001110b

			movdqa xmm2,xmm0
			movdqa xmm1,xmm3
//...........................................................................................
			pshuflw xmm2,xmm2,00011011b
			pshufhw xmm2,xmm2,00011011b

			pshuflw xmm0,xmm0,01001110b
			pshufhw xmm0,xmm0,01001110b
//...........................................................................................
			pshuflw xmm1,xmm1,00011011b
			pshufhw xmm1,xmm1,00011011b

			pshuflw xmm3,xmm3,01001110b
			pshufhw xmm3,xmm3,01001110b 
//...........................................................................................
			pand	xmm2,xmm4
			pand	xmm0,xmm5

			pand	xmm1,xmm4
			pand	xmm3,xmm5

			por		xmm0,xmm2						 // join all
			por		xmm3,xmm1						 // join all

			movntdq [edi-16],xmm0
			movntdq [edi-32],xmm3

			lea esi,[esi+32]
			sub edi,32

			sub ecx,1
			jne ecxis8_0

			sfence
//...........................................................................................
align 16
testreminder:
			test eax,eax
			jz nextline
//...........................................................................................
//align 16
//testreminder0:
			mov ecx,eax
			sar ecx,2								// we can add here a control for lower than 4??
			and eax,3
align 16
lastbytes:
			movq mm0,[esi]
			psllq mm0,32							// to take advantage of same code

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4
			pand	mm0,mm5

			por mm0,mm2								// join all

			movd [edi-4],mm0

			sub ecx,1

			lea edi,[edi-4]
			lea esi,[esi+4]

			jnz lastbytes
//...........................................................................................
			test eax,eax
			jz nextline
//...........................................................................................
			add esi,eax								//if this happens; is there a bug??
			sub edi,eax								// eax should only be two. Unfinished??
//...........................................................................................
align 16
nextline:
			add esi,[esp+16+16]						// modulosrc
			sub edi,[esp+16+28]						// modulodst

			sub ebx,1
			jnz y_loop
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
////////////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////
static void __declspec(naked) asm_isse_yuy2_180_3_dst
					(const BYTE* srcp,	//4
					int row,			//8
					int height,			//12	//cuidado debe venir entera
					int modulo,			//16
					int sizefr,			//20	//no needed
					BYTE* dstp,			//24
					int dstmodulo,		//28
					int dstsizefr)		//32		
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov eax,0xFF00FF00			//mask
			movd mm5,eax
			punpckldq mm5,mm5

			ror eax,8					//mask

			movd mm4,eax
			punpckldq mm4,mm4
//...........................................................................................
			mov			esi,[esp+16+4]		//ptr to src
			mov			eax,[esp+16+32]		//last byte of last row
			mov			ebx,[esp+16+12]		//height
//...........................................................................................
			mov			edi,[esp+16+24]		
			add			edi,eax				//ptr to last byte of last row
//...........................................................................................

align 16
y_loop:
			mov ecx,[esp+16+8]			//row
			mov eax,ecx					//[esp+16+8]			//row

			sar ecx,4
			jz testreminder0
//...........................................................................................
			and eax,15
//...........................................................................................
align 16
x_loop:
			movq mm0,[esi]
			movq mm3,[esi+8]

			pshufw mm1,mm3,00011011b 
			pshufw mm3,mm3,01001110b 

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4					//keep chromas,discard lumas
			pand	mm0,mm5					//keep lumas,discard chromas
			pand	mm1,mm4					//keep chromas,discard lumas
			pand	mm3,mm5					//keep lumas,discard chromas

			por mm0,mm2						//join all
			por mm3,mm1						//join all

			movq [edi-8],mm0
			movq [edi-16],mm3

			sub ecx,1

			lea edi,[edi-16]
			lea esi,[esi+16]

			jne x_loop
//...........................................................................................
			test eax,eax
			jz nextline
//...........................................................................................
align 16
testreminder0:
			mov ecx,eax
			sar ecx,2						//we can add here a control for lower than 4??
			and eax,3
align 16
lastbytes:
			movq mm0,[esi]
			psllq mm0,32					//to take advantage of same code

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4
			pand	mm0,mm5

			por mm0,mm2						//join all

			movd [edi-4],mm0

			sub ecx,1

			lea edi,[edi-4]
			lea esi,[esi+4]

			jnz lastbytes
//...........................................................................................
			test eax,eax
			jz nextline
											//if this happens; is there a bug??
			 add esi,eax
			 sub edi,eax					//eax should only be two. Unfinished
//...........................................................................................
align 16
nextline:
			sub edi,[esp+16+28]				//modulodst
			add esi,[esp+16+16]				//modulo

			sub ebx,1
			jnz y_loop
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
////////////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////
static void __declspec(naked) asm_isse_yuy2_180_3_dst_nta
					(const BYTE* srcp,	//4
					int row,			//8
					int height,			//12 cuidado debe venir entera
					int modulo,			//16
					int sizefr,			//20//no needed
					BYTE* dstp,			//24
					int dstmodulo,		//28
					int dstsizefr)		//32		
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov eax,0xFF00FF00			//mask
			movd mm5,eax
			punpckldq mm5,mm5

			ror eax,8					//mask

			movd mm4,eax
			punpckldq mm4,mm4
//...........................................................................................
			mov			esi,[esp+16+4]		//ptr to src
			mov			eax,[esp+16+32]		//last byte of last row
			mov			ebx,[esp+16+12]		//height
			mov			ebp,[esp+16+16]

//...........................................................................................
			mov			edi,[esp+16+24]		
			add			edi,eax				//ptr to last byte of last row
			add			ebp,ecx
//...........................................................................................

align 16
y_loop:
			mov ecx,[esp+16+8]			//row
			mov eax,[esp+16+8]			//row

			sar ecx,4
			//jz testreminder0			//not needed 
//...........................................................................................
			and eax,15
//...........................................................................................
align 16
x_loop:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movq mm0,[esi]
			movq mm3,[esi+8]

			pshufw mm1,mm3,00011011b 
			pshufw mm3,mm3,01001110b 

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4					//keep chromas,discard lumas
			pand	mm0,mm5					//keep lumas,discard chromas
			pand	mm1,mm4					//keep chromas,discard lumas
			pand	mm3,mm5					//keep lumas,discard chromas

			por mm0,mm2						//join all
			por mm3,mm1						//join all

			movntq [edi-8],mm0
			movntq [edi-16],mm3

			lea esi,[esi+16]
			sub edi,16

			sub ecx,1
			jne x_loop
//...........................................................................................
			sfence

			test eax,eax
			jz nextline
//...........................................................................................
//align 16
//testreminder0:
			mov ecx,eax
			sar ecx,2						//we can add here a control for lower than 4??
			and eax,3
align 16
lastbytes:
			movq mm0,[esi]

			psllq mm0,32					//to take advantage of same code

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4
			pand	mm0,mm5

			por mm0,mm2						//join all

			movd [edi-4],mm0

			sub ecx,1

			lea edi,[edi-4]
			lea esi,[esi+4]

			jnz lastbytes
//...........................................................................................
			test eax,eax
			jz nextline
											//if this happens; is there a bug??
			add esi,eax
			sub edi,eax						//eax should only be two. Unfinished
//...........................................................................................
align 16
nextline:
			sub edi,[esp+16+28]				//modulodst
			add esi,[esp+16+16]				//modulo

			sub ebx,1
			jnz y_loop
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////////borrowed and adapted from Dscaler 4.18 Copyright (c) 2002 Rob Muller////
static void __declspec(naked) asm_MMX2_yuy2_180_3_dst
					(const BYTE* srcp,		//4
					int row,				//8
					int height,				//12 cuidado debe venir entera
					int modulo,				//16
					int sizefr,				//20//no needed
					BYTE* dstp,				//24
					int dstmodulo,			//28
					int dstsizefr)			//32		
{

__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov eax,0xFF00FF00
			movd mm5,eax
			punpckldq mm5,mm5

			mov eax,0x00FF0000
			movd mm6,eax
			punpckldq mm6,mm6

			ror eax,16

			movd mm7,eax
			punpckldq mm7,mm7
//...........................................................................................
			mov			esi,[esp+16+4]		//ptr to src
			mov			eax,[esp+16+32]		//last byte of last row
			mov			ebx,[esp+16+12]		//height
//...........................................................................................
			mov			edi,[esp+16+24]		
			add			edi,eax				//ptr to last byte of last row
//...........................................................................................
align 16
y_loop:
			mov ecx,[esp+16+8]		//row
			mov eax,[esp+16+8]		//row

			sar ecx,4//3
			jz testreminder0
//...........................................................................................
			and eax,15//7
//...........................................................................................
align 16
x_loop:
			movq mm0,[esi]
			movq mm3,[esi+8]

			movq mm1,mm0
			psllq mm0,32
			psrlq mm1,32
			por mm0,mm1
			movq mm2,mm0
			movq mm1,mm0
			pand mm2,mm5
			psllq mm0,16
			psrlq mm1,16
			pand mm0,mm6
			pand mm1,mm7
			por mm0,mm2
			por mm0,mm1
//...........................................................................................
			movq mm1,mm3
			psllq mm3,32
			psrlq mm1,32
			por mm3,mm1
			movq mm2,mm3
			movq mm1,mm3
			pand mm2,mm5
			psllq mm3,16
			psrlq mm1,16
			pand mm3,mm6
			pand mm1,mm7
			por mm3,mm2
			por mm3,mm1

			movq [edi-8],mm0
			movq [edi-16],mm3

			sub ecx,1

			lea edi,[edi-16]
			lea esi,[esi+16]

			jne x_loop
//...........................................................................................
			test eax,eax
			jz nextline
//...........................................................................................
align 16
testreminder0:
			mov ecx,eax
			sar ecx,2						//we can add here a control for lower than 4??
			and eax,3
align 16
lastbytes:
			movq mm0,[esi]

			psllq mm0,32					//to take advantage of same code

			movq mm1,mm0
			psllq mm0,32
			psrlq mm1,32
			por mm0,mm1
			movq mm2,mm0
			movq mm1,mm0
			pand mm2,mm5
			psllq mm0,16
			psrlq mm1,16
			pand mm0,mm6
			pand mm1,mm7
			por mm0,mm2
			por mm0,mm1

			movd [edi-4],mm0

			sub ecx,1

			lea edi,[edi-4]
			lea esi,[esi+4]

			jnz lastbytes
//...........................................................................................
			test eax,eax
			jz nextline
											//if this happens; is there a bug??
			 add esi,eax
			 sub edi,eax					//eax should only be two. Unfinished
//...........................................................................................
align 16
nextline:
			sub edi,[esp+16+28]				//modulo
			add esi,[esp+16+16]				//modulo

			sub ebx,1
			jnz y_loop
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
////////////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////
static void __declspec(naked) asm_isse_yuy2_180_3
					(BYTE* srcp,		//4
					int row,			//8
					int height,			//12
					int modulo,			//16
					int sizefr)			//20
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov eax,0xFF00FF00			//mask
			movd mm5,eax
			punpckldq mm5,mm5

			ror eax,8					//mask

			movd mm4,eax
			punpckldq mm4,mm4
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ebx,[esp+16+12]			//height
			mov eax,[esp+16+20]			//last byte of last row
			mov ebp,ebx					//height
//...........................................................................................
			mov edi,esi					//copy of ptr
			sar ebx,1					//ok
			and ebp,1					//seems right for a middle left line
			add edi,eax					//ptr to last byte of last row
//...........................................................................................

align 16
y_loop:
			mov ecx,[esp+16+8]			//row
			mov eax,[esp+16+8]			//row

			sar ecx,3
			jz testreminder0
//...........................................................................................
			and eax,7
//...........................................................................................
align 16
x_loop:
			movq mm0,[esi]
			movq mm3,[edi-8]

			pshufw mm1,mm3,00011011b 
			pshufw mm3,mm3,01001110b 

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4					//keep chromas,discard lumas
			pand	mm0,mm5					//keep lumas,discard chromas
			pand	mm1,mm4					//keep chromas,discard lumas
			pand	mm3,mm5					//keep lumas,discard chromas

			por mm0,mm2						//join all
			por mm3,mm1						//join all

			movq [edi-8],mm0
			movq [esi],mm3

			sub ecx,1

			lea edi,[edi-8]
			lea esi,[esi+8]

			jne x_loop
//...........................................................................................
			test eax,eax
			jz nextline
//...........................................................................................
align 16
testreminder0:
			mov ecx,eax
			sar ecx,2						//we can add here a control for lower than 4??
			and eax,3
align 16
lastbytes:
			movq mm0,[esi]
			movq mm3,[edi-8]

			psllq mm0,32					//to take advantage of same code

			pshufw mm1,mm3,00011011b 
			pshufw mm3,mm3,01001110b 

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4
			pand	mm0,mm5
			pand	mm1,mm4
			pand	mm3,mm5

			por mm0,mm2						//join all
			por mm3,mm1						//join all

			movd [edi-4],mm0
			sub ecx,1
			lea edi,[edi-4]
			movd [esi],mm3

			lea esi,[esi+4]
			jnz lastbytes
//...........................................................................................
			test eax,eax
			jz nextline
											//if this happens; is there a bug??
			 add esi,eax
			 sub edi,eax					//eax should only be two. Unfinished
//...........................................................................................
align 16
nextline:
			sub edi,[esp+16+16]				//modulo
			add esi,[esp+16+16]				//modulo

			sub ebx,1
			jnz y_loop
//..........................is there an odd middle line?.....yes.............................
			test ebp,ebp
			jz end_loop

//............just when height is odd....a middle left line..................................
			mov ecx,[esp+16+8]				//row
			mov eax,[esp+16+8]				//row

			sar ecx,4
			jz lower16
//...........................................................................................
			and eax,15
//...........................................................................................
align 16
x_loop00:
			movq mm0,[esi]
			movq mm3,[edi-8]

			pshufw mm1,mm3,00011011b
			pshufw mm3,mm3,01001110b

			pshufw mm2,mm0,00011011b
			pshufw mm0,mm0,01001110b

			pand	mm2,mm4
			pand	mm0,mm5
			pand	mm1,mm4
			pand	mm3,mm5

			por mm0,mm2						//join all
			por mm3,mm1						//join all

			sub ecx,1

			movq [edi-8],mm0
			movq [esi],mm3

			lea edi,[edi-8]
			lea esi,[esi+8]

			jne x_loop00
//...........................................................................................
			test eax,eax
			jz end_loop
align 16
lower16:
			mov ecx,eax
			sar ecx,3
			jz lower8
//...........................................................................................
			and eax,7
align 16
x_loop_01mid:
			movq mm0,[esi]
			movq mm3,[edi-8]

			psllq mm0,32

			pshufw mm1,mm3,00011011b
			pshufw mm3,mm3,01001110b

			pshufw mm2,mm0,00011011b
			pshufw mm0,mm0,01001110b

			pand	mm2,mm4
			pand	mm0,mm5
			pand	mm1,mm4
			pand	mm3,mm5

			por		mm0,mm2						//join all
			por		mm3,mm1						//join all

			sub ecx,1

			movd [edi-4],mm0
			movd [esi],mm3

			lea edi,[edi-4]
			lea esi,[esi+4]

			jnz x_loop_01mid
//...........................................................................................
			test eax,eax
			jz end_loop
align 16
lower8:											//test if happens cause if it is four or six ???
			sar eax,1

			mov cl,[esi]
			mov dl,[edi-2]

			mov [edi-2],cl
			mov [esi],dl

			sub edi,1
			add esi,1

			sub eax,1
		    jz end_loop
//...........................................................................................
			mov cl,[esi]
			mov dl,[edi-2]

			mov [edi-2],cl
			mov [esi],dl

			sub edi,1
			add esi,1

			sub eax,1
			jz end_loop
//...........................................................................................
			mov cl,[esi]
			mov dl,[edi-2]

			mov [edi-2],cl
			mov [esi],dl
//...........................................................................................
align 16
end_loop:
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////////borrowed and adapted from Dscaler 4.18 Copyright (c) 2002 Rob Muller////
static void __declspec(naked) asm_MMX2_yuy2_180_3
			(BYTE* srcp,		//4
			int row,			//8
			int height,			//12
			int modulo,			//16
			int sizefr)			//20
{

__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov eax,0xFF00FF00
			movd mm5,eax
			punpckldq mm5,mm5

			mov eax,0x00FF0000
			movd mm6,eax
			punpckldq mm6,mm6

			ror eax,16

			movd mm7,eax
			punpckldq mm7,mm7
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ebx,[esp+16+12]		//height
			mov eax,[esp+16+20]		//last byte of last row
			mov ebp,ebx				//height
//...........................................................................................
			mov edi,esi				//copy of ptr
			sar ebx,1				//ok
			and ebp,1				//seems right when odd lines (middle line)
			add edi,eax				//ptr to last byte of last row
//...........................................................................................
align 16
y_loop:
			mov ecx,[esp+16+8]		//row
			mov eax,[esp+16+8]		//row

			sar ecx,3
			jz testreminder0
//...........................................................................................
			and eax,7
//...........................................................................................
align 16
x_loop:
			movq mm0,[esi]
			movq mm3,[edi-8]

			movq mm1,mm0
			psllq mm0,32
			psrlq mm1,32
			por mm0,mm1
			movq mm2,mm0
			movq mm1,mm0
			pand mm2,mm5
			psllq mm0,16
			psrlq mm1,16
			pand mm0,mm6
			pand mm1,mm7
			por mm0,mm2
			por mm0,mm1

			movq [edi-8],mm0

			movq mm1,mm3
			psllq mm3,32
			psrlq mm1,32
			por mm3,mm1
			movq mm2,mm3
			movq mm1,mm3
			pand mm2,mm5
			psllq mm3,16
			psrlq mm1,16
			pand mm3,mm6
			pand mm1,mm7
			por mm3,mm2
			por mm3,mm1

			sub ecx,1

			movq [esi],mm3

			lea esi,[esi+8]
			lea edi,[edi-8]

			jne x_loop
//...........................................................................................
			test eax,eax
			jz nextline
//...........................................................................................
align 16
testreminder0:
			mov ecx,eax
			sar ecx,2						//we can add here a control for lower than 4??
			and eax,3
align 16
lastbytes:
			movq mm0,[esi]
			movq mm3,[edi-8]

			psllq mm0,32					//to take advantage of same code

			movq mm1,mm0
			psllq mm0,32
			psrlq mm1,32
			por mm0,mm1
			movq mm2,mm0
			movq mm1,mm0
			pand mm2,mm5
			psllq mm0,16
			psrlq mm1,16
			pand mm0,mm6
			pand mm1,mm7
			por mm0,mm2
			por mm0,mm1

			movq mm1,mm3
			psllq mm3,32
			psrlq mm1,32
			por mm3,mm1
			movq mm2,mm3
			movq mm1,mm3
			pand mm2,mm5
			psllq mm3,16
			psrlq mm1,16
			pand mm3,mm6
			pand mm1,mm7
			por mm3,mm2
			por mm3,mm1


			movd [edi-4],mm0
			movd [esi],mm3

			sub ecx,1

			lea esi,[esi+4]
			lea edi,[edi-4]

			jnz lastbytes
//...........................................................................................
			test eax,eax
			jz nextline
											//if this happens; is there a bug??
			 add esi,eax
			 sub edi,eax					//eax should only be two. Unfinished
//...........................................................................................
align 16
nextline:
			sub edi,[esp+16+16]		//modulo
			add esi,[esp+16+16]		//modulo

			sub ebx,1
			jnz y_loop
//...........................................................................................
			test ebp,ebp
			jz end_loop

//............just when height is odd.......middle line......................................

			mov ecx,[esp+16+8]				//row
//...........................................................................................

			mov eax,ecx
			sar ecx,4
			jz lower16
//...........................................................................................
			and eax,15
//...........................................................................................
align 16
x_loop00:
			movq mm0,[esi]
			movq mm3,[edi-8]

			movq mm1,mm0
			psllq mm0,32
			psrlq mm1,32
			por mm0,mm1
			movq mm2,mm0
			movq mm1,mm0
			pand mm2,mm5
			psllq mm0,16
			psrlq mm1,16
			pand mm0,mm6
			pand mm1,mm7
			por mm0,mm2
			por mm0,mm1


			movq mm1,mm3
			psllq mm3,32
			psrlq mm1,32
			por mm3,mm1
			movq mm2,mm3
			movq mm1,mm3
			pand mm2,mm5
			psllq mm3,16
			psrlq mm1,16
			pand mm3,mm6
			pand mm1,mm7
			por mm3,mm2
			por mm3,mm1

			sub ecx,1

			movq [edi-8],mm0
			movq [esi],mm3

			lea edi,[edi-8]
			lea esi,[esi+8]

			jne x_loop00
//...........................................................................................
			test eax,eax
			jz end_loop
//...........................................................................................
align 16
lower16:
//...........................................................................................
			mov ecx,eax
			sar ecx,3
			jz lower8
//...........................................................................................
			and eax,7
//...........................................................................................
align 16
x_loop_01mid:
			movq mm0,[esi]
			movq mm3,[edi-8]

			psllq mm0,32				//to take advantage of same code

			movq mm1,mm0
			psllq mm0,32
			psrlq mm1,32
			por mm0,mm1
			movq mm2,mm0
			movq mm1,mm0
			pand mm2,mm5
			psllq mm0,16
			psrlq mm1,16
			pand mm0,mm6
			pand mm1,mm7
			por mm0,mm2
			por mm0,mm1

			movq mm1,mm3
			psllq mm3,32
			psrlq mm1,32
			por mm3,mm1
			movq mm2,mm3
			movq mm1,mm3
			pand mm2,mm5
			psllq mm3,16
			psrlq mm1,16
			pand mm3,mm6
			pand mm1,mm7
			por mm3,mm2
			por mm3,mm1


			movd [edi-4],mm0
			movd [esi],mm3

			sub ecx,1

			lea esi,[esi+4]
			lea edi,[edi-4]

			jnz x_loop_01mid
//...........................................................................................
			test eax,eax
			jz end_loop
//...........................................................................................
align 16
lower8:
			sar eax,1

			mov cl,[esi]
			mov dl,[edi-2]

			mov [edi-2],cl
			mov [esi],dl

			sub edi,1
			add esi,1

			sub eax,1
		    jz end_loop
//...........................................................................................
			mov cl,[esi]
			mov dl,[edi-2]

			mov [edi-2],cl
			mov [esi],dl

			sub edi,1
			add esi,1

			sub eax,1
			jz end_loop
//...........................................................................................
			mov cl,[esi]
			mov dl,[edi-2]

			mov [edi-2],cl
			mov [esi],dl
//...........................................................................................
align 16
end_loop:
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////////borrowed from Dscaler 4.18 Copyright (c) 2002 Rob Muller/////////////////
static void __declspec(naked) asm_MMX_yuy2_mirror2
						(BYTE* srcp,	//4
						int row,		//8
						int height,		//12
						int modulo)		//16
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov eax,0xFF00FF00
			movd mm5,eax
			punpckldq mm5,mm5

			mov eax,0x00FF0000
			movd mm6,eax
			punpckldq mm6,mm6

			ror eax,16

			movd mm7,eax
			punpckldq mm7,mm7
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+8]			//row
			mov ebx,[esp+16+12]			//height
			mov edi,esi					//copy of ptr
			mov ebp,[esp+16+16]			//modulo
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
x_loop_0:
			mov eax,ecx
			sar ecx,4
			jz	testtail
//...........................................................................................
			and eax,15
//...........................................................................................
align 16
ecxis8_0:
			movq mm0,[esi]
			movq mm3,[edi-8]

			movq mm1,mm0
			psllq mm0,32
			psrlq mm1,32
			por mm0,mm1
			movq mm2,mm0
			movq mm1,mm0
			pand mm2,mm5
			psllq mm0,16
			psrlq mm1,16
			pand mm0,mm6
			pand mm1,mm7
			por mm0,mm2
			por mm0,mm1

			movq [edi-8],mm0

			movq mm1,mm3
			psllq mm3,32
			psrlq mm1,32
			por mm3,mm1
			movq mm2,mm3
			movq mm1,mm3
			pand mm2,mm5
			psllq mm3,16
			psrlq mm1,16
			pand mm3,mm6
			pand mm1,mm7
			por mm3,mm2
			por mm3,mm1


			movq [esi],mm3

			lea edi,[edi-8]
			lea esi,[esi+8]

			dec ecx

			jne ecxis8_0
//...........................................................................................
			test eax,eax
			jz nextline
//...........................................................................................
align 16
testtail:
//...........................................................................................
			mov ecx,eax
			sar ecx,3
			jz lower8
//...........................................................................................
			and eax,7
//...........................................................................................
align 16
x_loop_01:
			movq mm0,[esi]
			movq mm3,[edi-8]

			psllq mm0,32

			movq mm1,mm0
			psllq mm0,32
			psrlq mm1,32
			por mm0,mm1
			movq mm2,mm0
			movq mm1,mm0
			pand mm2,mm5
			psllq mm0,16
			psrlq mm1,16
			pand mm0,mm6
			pand mm1,mm7
			por mm0,mm2
			por mm0,mm1

			movq mm1,mm3
			psllq mm3,32
			psrlq mm1,32
			por mm3,mm1
			movq mm2,mm3
			movq mm1,mm3
			pand mm2,mm5
			psllq mm3,16
			psrlq mm1,16
			pand mm3,mm6
			pand mm1,mm7
			por mm3,mm2
			por mm3,mm1


			movd [esi],mm3
			movd [edi-4],mm0

			sub ecx,1

			lea esi,[esi+4]
			lea edi,[edi-4]
			jnz x_loop_01
//...........................................................................................
			test eax,eax
			jz nextline
align 16
lower8:
			mov cl,[esi]
			mov dl,[edi-2]

			mov [edi-2],cl
			mov [esi],dl

			sub edi,1
			add esi,1					//?

			sub eax,2
			test eax,eax
			jnz lower8
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+8]			//row
										//lea edi,[edi+ebp]			//ptr + modulo
			lea esi,[esi+ebp]			//first ptr+modulo

			mov eax,ecx					//eax=row
			sar eax,1					//eax=row/2 just to add with ptrs

			sub ebx,1					//height -1

			lea esi,[esi+eax]			//first ptr+row/2
										//lea edi,[edi+eax]			//sec ptr+row/2. Finally row+(1 1/2row)??
			mov edi,esi
			lea edi,[edi+ecx]			//sec ptr+row

			jne x_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
///////////////////////Copyright (c) Enrique Ardanaz (ARDA)//////////////////////////////////
static void __declspec(naked) asm_isse_yuy2_mirror2
						(BYTE* srcp,	//4
						int row,		//8
						int height,		//12
						int modulo)		//16
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov eax,0xFF00FF00			//mask
			movd mm5,eax
			punpckldq mm5,mm5

			ror eax,8					//mask
			movd mm4,eax
			punpckldq mm4,mm4
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+8]			//row
			mov ebx,[esp+16+12]			//height
			mov edi,esi					//copy of ptr
			mov ebp,[esp+16+16]			//modulo
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
x_loop_0:
			mov eax,ecx
			sar ecx,4
			jz  testtail				//don't like controls in the loop
//...........................................................................................
			and eax,15
//...........................................................................................
align 16
ecxis8_0:
			pshufw mm2,[esi],00011011b
			pshufw mm0,[esi],01001110b

			pand	mm2,mm4						//keep chromas,discard lumas
			pand	mm0,mm5						//keep lumas,discard chromas

			pshufw mm1,[edi-8],00011011b 
			pshufw mm3,[edi-8],01001110b 

			pand	mm1,mm4						//keep chromas,discard lumas
			pand	mm3,mm5						//keep lumas,discard chromas

			por mm0,mm2							//join all
			por mm3,mm1							//join all

			movq [edi-8],mm0

			dec ecx
			lea edi,[edi-8]

			movq [esi],mm3

			lea esi,[esi+8]

			jne ecxis8_0
//...........................................................................................
			test eax,eax
			jz nextline
//...........................................................................................
align 16
testtail:
//...........................................................................................
			mov ecx,eax
			sar ecx,3
			jl lower8
//...........................................................................................
			and eax,7
align 16
x_loop_01:
			movq mm0,[esi]
			movq mm3,[edi-8]

			psllq mm0,32						//to take advantage of same code

			pshufw mm1,mm3,00011011b 
			pshufw mm3,mm3,01001110b 

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4						//keep chromas,discard lumas
			pand	mm0,mm5						//keep lumas,discard chromas
			pand	mm1,mm4						//keep chromas,discard lumas
			pand	mm3,mm5						//keep lumas,discard chromas

			por mm0,mm2							//join all
			por mm3,mm1							//join all

			movd [edi-4],mm0
			dec ecx

			lea edi,[edi-4]

			movd [esi],mm3

			lea esi,[esi+4]

			jnz x_loop_01
//...........................................................................................
			test eax,eax
			jz nextline
//...........................................................................................
align 16
lower8:
			mov cl,[esi]
			mov dl,[edi-2]

			sub eax,2

			mov [edi-2],cl
			mov [esi],dl

			sub edi,1
			add esi,1

			test eax,eax
			jnz lower8
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+8]			//row
			lea esi,[esi+ebp]			//first ptr+modulo

			mov eax,ecx					//eax=row
			sar eax,1					//eax=row/2 just to add with ptrs

			sub ebx,1					//height -1

			lea esi,[esi+eax]			//first ptr+row/2
			mov edi,esi					//copy of ptr										//lea edi,[edi+eax]			//sec ptr+row/2. Finally row+(1 1/2row)
			lea edi,[edi+ecx]			//sec ptr+row


			jne x_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
//////////////////////Copyright (c) Enrique Ardanaz (ARDA)///////////////////////////////////
static void __declspec(naked) asm_sse2_yuy2_mirror
						(BYTE* srcp,	//4
						int row,		//8
						int height,		//12
						int modulo)		//16
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov eax,0xFF00FF00			//mask
			movd mm5,eax
			punpckldq mm5,mm5
			movq2dq		xmm5,mm5
			punpckldq	xmm5,xmm5

			ror eax,8					//mask
			movd mm4,eax
			punpckldq	mm4,mm4
			movq2dq		xmm4,mm4
			punpckldq	xmm4,xmm4
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+8]			//row
			mov ebx,[esp+16+12]			//height
			mov edi,esi					//copy of ptr
			mov ebp,[esp+16+16]			//modulo
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
x_loop_0:
			mov eax,ecx
			cmp ecx,32
			jl	lower32
//...........................................................................................
			test esi,15
			jz ecxis16
//...........................................................................................
align 16
x_loop_01:
			movq mm0,[esi]
			movq mm3,[edi-8]

			psllq mm0,32				//to take advantage of same code

			pshufw mm1,mm3,00011011b 
			pshufw mm3,mm3,01001110b 

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4					//keep chromas,discard lumas
			pand	mm0,mm5					//keep lumas,discard chromas
			pand	mm1,mm4					//keep chromas,discard lumas
			pand	mm3,mm5					//keep lumas,discard chromas

			por mm0,mm2						//join all
			por mm3,mm1						//join all

			movd [edi-4],mm0
			sub ecx,8
			lea edi,[edi-4]
			movd [esi],mm3

			lea esi,[esi+4]

			test esi,15
			jnz x_loop_01
//...........................................................................................
align 16
ecxis16:
			test edi,15
			jz bothaligned
//...........................................................................................
			mov	eax,ecx
			sar ecx,5
			jz	lower32
//...........................................................................................
			and eax,31
//...........................................................................................
align 16
srcptraligned:
			movdqa xmm0,[esi]
			movdqu xmm3,[edi-16]
//...........................................................................................
			pshufd xmm0,xmm0,01001110b
			pshufd xmm3,xmm3,01001110b

			movdqa xmm2,xmm0
			movdqa xmm1,xmm3
//...........................................................................................
			pshuflw xmm2,xmm2,00011011b
			pshufhw xmm2,xmm2,00011011b

			pshuflw xmm0,xmm0,01001110b
			pshufhw xmm0,xmm0,01001110b
//...........................................................................................
			pshuflw xmm1,xmm1,00011011b
			pshufhw xmm1,xmm1,00011011b 

			pshuflw xmm3,xmm3,01001110b
			pshufhw xmm3,xmm3,01001110b 
//...........................................................................................

			pand	xmm2,xmm4					 //keep chromas,discard lumas
			pand	xmm0,xmm5					 //keep lumas,discard chromas

			pand	xmm1,xmm4					 //keep chromas,discard lumas
			pand	xmm3,xmm5					 //keep lumas,discard chromas

			por		xmm0,xmm2					 //join all
			por		xmm3,xmm1					 //join all

			movdqu [edi-16],xmm0
			sub ecx,1
			lea edi,[edi-16]
			movdqa [esi],xmm3
			lea esi,[esi+16]

			jne srcptraligned
//...........................................................................................
			jmp testreminder
//...........................................................................................
align 16
bothaligned:
			mov	eax,ecx
			sar ecx,5
			jz	lower32
//...........................................................................................
			and eax,31
//...........................................................................................
align 16
ecxis8_0:
			movdqa xmm0,[esi]
			movdqa xmm3,[edi-16]
//...........................................................................................
			pshufd xmm0,xmm0,01001110b
			pshufd xmm3,xmm3,01001110b

			movdqa xmm2,xmm0
			movdqa xmm1,xmm3
//...........................................................................................
			pshuflw xmm2,xmm2,00011011b
			pshufhw xmm2,xmm2,00011011b

			pshuflw xmm0,xmm0,01001110b
			pshufhw xmm0,xmm0,01001110b
//...........................................................................................
			pshuflw xmm1,xmm1,00011011b
			pshufhw xmm1,xmm1,00011011b 

			pshuflw xmm3,xmm3,01001110b
			pshufhw xmm3,xmm3,01001110b 
//...........................................................................................
			pand	xmm2,xmm4
			pand	xmm0,xmm5

			pand	xmm1,xmm4
			pand	xmm3,xmm5

			por		xmm0,xmm2					//join all
			por		xmm3,xmm1					//join all

			movdqa [edi-16],xmm0
			sub ecx,1
			lea edi,[edi-16]
			movdqa [esi],xmm3
			lea esi,[esi+16]

			jne ecxis8_0
//...........................................................................................
align 16
testreminder:
			test eax,eax
			jz nextline
align 16
lower32:
//...........................................................................................
			mov ecx,eax
			sar ecx,3
			jz lower8
//...........................................................................................
			and eax,7
//...........................................................................................
align 16
lastbytes:
			movq mm0,[esi]
			movq mm3,[edi-8]

			psllq mm0,32

			pshufw mm1,mm3,00011011b 
			pshufw mm3,mm3,01001110b 

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4
			pand	mm0,mm5
			pand	mm1,mm4
			pand	mm3,mm5

			por mm0,mm2							//join all
			por mm3,mm1							//join all

			movd [edi-4],mm0
			sub ecx,1
			lea edi,[edi-4]
			movd [esi],mm3

			lea esi,[esi+4]
			jnz lastbytes
//...........................................................................................
			test eax,eax
			jz nextline
align 16
lower8:
			mov cl,[esi]
			mov dl,[edi-2]

			sub eax,2

			mov [edi-2],cl
			mov [esi],dl
//...........................................................................................			
			add esi,1
			sub edi,1

			test eax,eax
			jnz lower8
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+8]			//row
										//lea edi,[edi+ebp]			//ptr2 + modulo
			lea esi,[esi+ebp]			//first ptr+modulo

			mov eax,ecx					//eax=row
			sar eax,1					//eax=row/2 just to add with ptrs

			sub ebx,1					//height -1

			lea esi,[esi+eax]			//first ptr+row/2
			mov edi,esi					//copy of ptr
										//lea edi,[edi+eax]//sec ptr+row/2. Finally row+(1 1/2row)
			lea edi,[edi+ecx]			//sec ptr+row

			jne x_loop_0
//...........................................................................................

		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
//////////////////////Copyright (c) Enrique Ardanaz (ARDA)///////////////////////////////////
static void __declspec(naked) asm_sse2_yuy2_mirror_dst
						(const BYTE* srcp,	//4
						BYTE* dstp,			//8
						int row,		//12
						int height,		//16
						int modulosrc,	//20
						int modulodst)	//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov eax,0xFF00FF00			//mask
			movd mm5,eax
			punpckldq mm5,mm5
			movq2dq		xmm5,mm5
			punpckldq	xmm5,xmm5

			ror eax,8					//mask
			movd mm4,eax
			punpckldq	mm4,mm4
			movq2dq		xmm4,mm4
			punpckldq	xmm4,xmm4
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst// 
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
x_loop_0:
			mov eax,ecx
			cmp ecx,32
			jl	lower32
//...........................................................................................
			test esi,15
			jz ecxis16
//...........................................................................................
align 16
x_loop_01:
			movq mm0,[esi]
			psllq mm0,32				//to take advantage of same code

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4					//keep chromas,discard lumas
			pand	mm0,mm5					//keep lumas,discard chromas
			por		mm0,mm2					//join all

			sub		ecx,4

			movd	[edi-4],mm0

			lea		esi,[esi+4]
			lea		edi,[edi-4]

			test esi,15
			jnz x_loop_01
//...........................................................................................
align 16
ecxis16:
			test edi,15
			jz bothaligned
//...........................................................................................
			mov	eax,ecx
			sar ecx,5
			jz	lower32
//...........................................................................................
			and eax,31
//...........................................................................................
align 16
srcptraligned:
			movdqa xmm0,[esi]
			movdqa xmm3,[esi+16]
//...........................................................................................
			pshufd xmm0,xmm0,01001110b
			pshufd xmm3,xmm3,01001110b

			movdqa xmm2,xmm0
			movdqa xmm1,xmm3
//...........................................................................................
			pshuflw xmm2,xmm2,00011011b
			pshufhw xmm2,xmm2,00011011b

			pshuflw xmm1,xmm1,00011011b
			pshufhw xmm1,xmm1,00011011b

			pshuflw xmm0,xmm0,01001110b
			pshufhw xmm0,xmm0,01001110b

			pshuflw xmm3,xmm3,01001110b
			pshufhw xmm3,xmm3,01001110b
//...........................................................................................
			pand	xmm2,xmm4					//keep chromas,discard lumas
			pand	xmm0,xmm5					//keep lumas,discard chromas

			pand	xmm1,xmm4					//keep chromas,discard lumas
			pand	xmm3,xmm5					//keep lumas,discard chromas

			por		xmm0,xmm2					//join all
			por		xmm3,xmm1					//join all

			movdqu [edi-16],xmm0
			movdqu [edi-32],xmm3

			sub ecx,1
			lea esi,[esi+32]
			lea edi,[edi-32]

			jne srcptraligned
//...........................................................................................
			jmp testreminder
//...........................................................................................
align 16
bothaligned:
			mov	eax,ecx
			sar ecx,5
			jz	lower32
//...........................................................................................
			and eax,31
//...........................................................................................
align 16
ecxis8_0:
			movdqa xmm0,[esi]
			movdqa xmm3,[esi+16]

//...........................................................................................
			pshufd xmm0,xmm0,01001110b
			pshufd xmm3,xmm3,01001110b

			movdqa xmm2,xmm0
			movdqa xmm1,xmm3
//...........................................................................................
			pshuflw xmm2,xmm2,00011011b
			pshufhw xmm2,xmm2,00011011b

			pshuflw xmm1,xmm1,00011011b
			pshufhw xmm1,xmm1,00011011b

			pshuflw xmm0,xmm0,01001110b
			pshufhw xmm0,xmm0,01001110b

			pshuflw xmm3,xmm3,01001110b
			pshufhw xmm3,xmm3,01001110b

//...........................................................................................
			pand	xmm2,xmm4					//keep chromas,discard lumas
			pand	xmm0,xmm5					//keep lumas,discard chromas

			pand	xmm1,xmm4					//keep chromas,discard lumas
			pand	xmm3,xmm5					//keep lumas,discard chromas

			por		xmm0,xmm2					//join all
			por		xmm3,xmm1					//join all

			movdqa [edi-16],xmm0
			movdqa [edi-32],xmm3

			sub ecx,1

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne ecxis8_0
//...........................................................................................
align 16
testreminder:
			test eax,eax
			jz nextline
align 16
lower32:
//...........................................................................................
			mov ecx,eax

			sar ecx,2
			jz lower8
//...........................................................................................
			and eax,3
//...........................................................................................
align 16
lastbytes:
			movq	mm0,[esi]
			psllq	mm0,32
			pshufw	mm2,mm0,00011011b 
			pshufw	mm0,mm0,01001110b 
			pand	mm2,mm4
			pand	mm0,mm5
			por		mm0,mm2							//join all

			sub ecx,1

			movd	[edi-4],mm0
			lea		esi,[esi+4]
			lea		edi,[edi-4]

			jnz lastbytes
//...........................................................................................
			test eax,eax
			jz nextline
align 16
lower8:
			mov cl,[esi]
			sub eax,1
			mov [edi-1],cl
//...........................................................................................			
			lea esi,[esi+1]
			lea edi,[edi-1]

			jnz lower8
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+12]			//row
			add esi,[esp+16+20]			//first ptr+modulo

			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx					//

			sub dword ptr [esp+16+16],1	//height -1

			jne x_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
//////////////////////Copyright (c) Enrique Ardanaz (ARDA)///////////////////////////////////
static void __declspec(naked) asm_sse2_yuy2_mirror_dst_nta
						(const BYTE* srcp,	//4
						BYTE* dstp,			//8
						int row,			//12
						int height,			//16
						int modulosrc,		//20
						int modulodst)		//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov eax,0xFF00FF00			//mask
			movd mm5,eax
			punpckldq mm5,mm5
			movq2dq		xmm5,mm5
			punpckldq	xmm5,xmm5

			ror eax,8					//mask
			movd mm4,eax
			punpckldq	mm4,mm4
			movq2dq		xmm4,mm4
			punpckldq	xmm4,xmm4
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst//
			mov ebp,[esp+16+20]
			add edi,ecx					//ptr to last byte same row
			add ebp,ecx
//...........................................................................................
align 16
x_loop_0:
			mov eax,ecx
//...........................................................................................
			test edi,15
			jz ecxis16
//...........................................................................................
align 16
x_loop_01:
			movq mm0,[esi]
			psllq mm0,32				//to take advantage of same code

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4					//keep chromas,discard lumas
			pand	mm0,mm5					//keep lumas,discard chromas
			por		mm0,mm2					//join all

			sub		ecx,4

			movd	[edi-4],mm0

			lea		esi,[esi+4]
			lea		edi,[edi-4]

			test edi,15
			jnz x_loop_01
//...........................................................................................
align 16
ecxis16:
			test esi,15
			jz bothaligned
//...........................................................................................
			mov	eax,ecx

			sar ecx,5
//...........................................................................................
			and eax,31
//...........................................................................................
align 16
srcptraligned:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movdqu xmm0,[esi]
			movdqu xmm3,[esi+16]
//...........................................................................................
			pshufd xmm0,xmm0,01001110b
			pshufd xmm3,xmm3,01001110b

			movdqa xmm2,xmm0
			movdqa xmm1,xmm3
//...........................................................................................
			pshuflw xmm2,xmm2,00011011b
			pshufhw xmm2,xmm2,00011011b

			pshuflw xmm1,xmm1,00011011b
			pshufhw xmm1,xmm1,00011011b

			pshuflw xmm0,xmm0,01001110b
			pshufhw xmm0,xmm0,01001110b

			pshuflw xmm3,xmm3,01001110b
			pshufhw xmm3,xmm3,01001110b
//...........................................................................................
			pand	xmm2,xmm4					//keep chromas,discard lumas
			pand	xmm0,xmm5					//keep lumas,discard chromas

			pand	xmm1,xmm4					//keep chromas,discard lumas
			pand	xmm3,xmm5					//keep lumas,discard chromas

			por		xmm0,xmm2					//join all
			por		xmm3,xmm1					//join all

			movntdq [edi-16],xmm0
			movntdq [edi-32],xmm3

			lea esi,[esi+32]
			sub edi,32

			sub ecx,1

			jne srcptraligned
			sfence
//...........................................................................................
			jmp testreminder
//...........................................................................................
align 16
bothaligned:
			mov	eax,ecx
			sar ecx,5
//...........................................................................................
			and eax,31
//...........................................................................................
align 16
ecxis8_0:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movdqa xmm0,[esi]
			movdqa xmm3,[esi+16]
//...........................................................................................
			pshufd xmm0,xmm0,01001110b
			pshufd xmm3,xmm3,01001110b

			movdqa xmm2,xmm0
			movdqa xmm1,xmm3
//...........................................................................................
			pshuflw xmm2,xmm2,00011011b
			pshufhw xmm2,xmm2,00011011b

			pshuflw xmm1,xmm1,00011011b
			pshufhw xmm1,xmm1,00011011b

			pshuflw xmm0,xmm0,01001110b
			pshufhw xmm0,xmm0,01001110b

			pshuflw xmm3,xmm3,01001110b
			pshufhw xmm3,xmm3,01001110b

//...........................................................................................
			pand	xmm2,xmm4					//keep chromas,discard lumas
			pand	xmm0,xmm5					//keep lumas,discard chromas

			pand	xmm1,xmm4					//keep chromas,discard lumas
			pand	xmm3,xmm5					//keep lumas,discard chromas

			por		xmm0,xmm2					//join all
			por		xmm3,xmm1					//join all

			movntdq [edi-16],xmm0
			movntdq [edi-32],xmm3

			lea esi,[esi+32]
			sub edi,32

			sub ecx,1
			jne ecxis8_0
			sfence
//...........................................................................................
align 16
testreminder:
			test eax,eax
			jz nextline
//...........................................................................................
			mov ecx,eax

			sar ecx,2
			jz lower8
//...........................................................................................
			and eax,3
//...........................................................................................
align 16
lastbytes:
			movq	mm0,[esi]
			psllq	mm0,32
			pshufw	mm2,mm0,00011011b 
			pshufw	mm0,mm0,01001110b 
			pand	mm2,mm4
			pand	mm0,mm5
			por		mm0,mm2							//join all

			sub ecx,1

			movd	[edi-4],mm0
			lea		esi,[esi+4]
			lea		edi,[edi-4]

			jnz lastbytes
//...........................................................................................
			test eax,eax
			jz nextline
align 16
lower8:
			mov cl,[esi]
			sub eax,1
			mov [edi-1],cl
//...........................................................................................			
			lea esi,[esi+1]
			lea edi,[edi-1]

			jnz lower8
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+12]			//row
			add esi,[esp+16+20]			//first ptr+modulo

			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx					//

			sub dword ptr [esp+16+16],1	//height -1
			jne x_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
///////////////////////Copyright (c) Enrique Ardanaz (ARDA)//////////////////////////////////
static void __declspec(naked) asm_isse_yuy2_mirror2_dst
						(const BYTE* srcp,	//4
						BYTE* dstp,			//8
						int row,			//12
						int height,			//16
						int modulo,			//20
						int modulodst)		//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov eax,0xFF00FF00			//mask
			movd mm5,eax
			punpckldq mm5,mm5

			ror eax,8					//mask
			movd mm4,eax
			punpckldq mm4,mm4
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst// 
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
x_loop_0:
			mov eax,ecx
			sar ecx,4
			jz  testtail
//...........................................................................................
			and eax,15
//...........................................................................................
align 16
ecxis8_0:
			pshufw mm2,[esi],00011011b
			pshufw mm0,[esi],01001110b

			pand	mm2,mm4						//keep chromas,discard lumas
			pand	mm0,mm5						//keep lumas,discard chromas

			pshufw mm1,[esi+8],00011011b 
			pshufw mm3,[esi+8],01001110b 

			pand	mm1,mm4						//keep chromas,discard lumas
			pand	mm3,mm5						//keep lumas,discard chromas

			por		mm0,mm2						//join all
			por		mm3,mm1						//join all

			movq [edi-8],mm0
			movq [edi-16],mm3

			dec ecx

			lea esi,[esi+16]
			lea edi,[edi-16]

			jne ecxis8_0
//...........................................................................................
			test eax,eax
			jz nextline
//...........................................................................................
align 16
testtail:
//...........................................................................................
			mov ecx,eax
			sar ecx,2
			jz lower8
//...........................................................................................
			and eax,3
align 16
x_loop_01:
			movq mm0,[esi]

			psllq mm0,32						//to take advantage of same code

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4						//keep chromas,discard lumas
			pand	mm0,mm5						//keep lumas,discard chromas

			movd [edi-4],mm0

			dec ecx

			lea esi,[esi+4]
			lea edi,[edi-4]

			jnz x_loop_01
//...........................................................................................
			test eax,eax
			jz nextline
//...........................................................................................
align 16
lower8:
			mov cl,[esi]
			mov [edi-1],cl

			sub edi,1
			add esi,1

			sub eax,1
			jnz lower8
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+12]			//row
			add esi,[esp+16+20]			//first ptr+modulo

			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx					//

			sub dword ptr [esp+16+16],1	//height -1

			jne x_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
///////////////////////Copyright (c) Enrique Ardanaz (ARDA)//////////////////////////////////
static void __declspec(naked) asm_isse_yuy2_mirror2_dst_nta
						(const BYTE* srcp,	//4
						BYTE* dstp,			//8
						int row,			//12
						int height,			//16
						int modulo,			//20
						int modulodst)		//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov eax,0xFF00FF00			//mask
			movd mm5,eax
			punpckldq mm5,mm5

			ror eax,8					//mask
			movd mm4,eax
			punpckldq mm4,mm4
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst//
			mov ebp,[esp+16+20]
			add edi,ecx					//ptr to last byte same row
			add ebp,ecx
//...........................................................................................
align 16
x_loop_0:
			mov eax,ecx

			sar ecx,4
			//jz  testtail				//not needed
//...........................................................................................
			and eax,15
//...........................................................................................
align 16
ecxis8_0:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			pshufw mm2,[esi],00011011b
			pshufw mm0,[esi],01001110b

			pand	mm2,mm4						//keep chromas,discard lumas
			pand	mm0,mm5						//keep lumas,discard chromas

			pshufw mm1,[esi+8],00011011b 
			pshufw mm3,[esi+8],01001110b 

			pand	mm1,mm4						//keep chromas,discard lumas
			pand	mm3,mm5						//keep lumas,discard chromas

			por		mm0,mm2						//join all
			por		mm3,mm1						//join all

			movntq [edi-8],mm0
			movntq [edi-16],mm3

			lea esi,[esi+16]
			sub edi,16

			dec ecx

			jne ecxis8_0
//...........................................................................................
			sfence

			test eax,eax
			jz nextline
//...........................................................................................
//align 16
//testtail:
//...........................................................................................
			mov ecx,eax
			sar ecx,2
			jz lower8
//...........................................................................................
			and eax,3
align 16
x_loop_01:
			movq mm0,[esi]

			psllq mm0,32						//to take advantage of same code

			pshufw mm2,mm0,00011011b 
			pshufw mm0,mm0,01001110b 

			pand	mm2,mm4						//keep chromas,discard lumas
			pand	mm0,mm5						//keep lumas,discard chromas

			movd [edi-4],mm0
			dec ecx

			lea esi,[esi+4]
			lea edi,[edi-4]

			jnz x_loop_01
//...........................................................................................
			test eax,eax
			jz nextline
//...........................................................................................
align 16
lower8:
			mov cl,[esi]
			mov [edi-1],cl

			sub edi,1
			add esi,1

			sub eax,1
			jnz lower8
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+12]			//row
			add esi,[esp+16+20]			//first ptr+modulo

			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx					//

			sub dword ptr [esp+16+16],1	//height -1

			jne x_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////////borrowed from Dscaler 4.18 Copyright (c) 2002 Rob Muller/////////////////
static void __declspec(naked) asm_MMX_yuy2_mirror2_dst
						(const BYTE* srcp,	//4
						BYTE* dstp,			//8
						int row,			//12
						int height,			//16
						int modulo,			//20
						int modulodst)		//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov eax,0xFF00FF00
			movd mm5,eax
			punpckldq mm5,mm5

			mov eax,0x00FF0000
			movd mm6,eax
			punpckldq mm6,mm6

			ror eax,16

			movd mm7,eax
			punpckldq mm7,mm7
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst// 
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
x_loop_0:
			mov eax,ecx
			sar ecx,4
			jz	testtail
//...........................................................................................
			and eax,15
//...........................................................................................
align 16
ecxis8_0:
			movq mm0,[esi]
			movq mm3,[esi+8]

			movq mm1,mm0
			psllq mm0,32
			psrlq mm1,32
			por mm0,mm1
			movq mm2,mm0
			movq mm1,mm0
			pand mm2,mm5
			psllq mm0,16
			psrlq mm1,16
			pand mm0,mm6
			pand mm1,mm7
			por mm0,mm2
			por mm0,mm1

			movq mm1,mm3
			psllq mm3,32
			psrlq mm1,32
			por mm3,mm1
			movq mm2,mm3
			movq mm1,mm3
			pand mm2,mm5
			psllq mm3,16
			psrlq mm1,16
			pand mm3,mm6
			pand mm1,mm7
			por mm3,mm2
			por mm3,mm1

			movq [edi-8],mm0
			movq [edi-16],mm3

			lea esi,[esi+16]
			lea edi,[edi-16]

			dec ecx

			jne ecxis8_0
//...........................................................................................
			test eax,eax
			jz nextline
//...........................................................................................
align 16
testtail:
//...........................................................................................
			mov ecx,eax
			sar ecx,2
			jz lower8

			and eax,3
//...........................................................................................
align 16
x_loop_01:
			movq mm0,[esi]

			psllq mm0,32

			movq mm1,mm0
			psllq mm0,32
			psrlq mm1,32
			por mm0,mm1
			movq mm2,mm0
			movq mm1,mm0
			pand mm2,mm5
			psllq mm0,16
			psrlq mm1,16
			pand mm0,mm6
			pand mm1,mm7
			por mm0,mm2
			por mm0,mm1

			movd [edi-4],mm0

			sub ecx,1
			lea esi,[esi+4]
			lea edi,[edi-4]
			jnz x_loop_01
//...........................................................................................
			test eax,eax
			jz nextline
align 16
lower8:
			mov cl,[esi]

			mov [edi-1],cl

			sub edi,1
			add esi,1					//?

			sub eax,1
			jnz lower8
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+12]			//row
			add esi,[esp+16+20]			//first ptr+modulo
			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx					//

			sub dword ptr [esp+16+16],1	//height -1

			jne x_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////////
/*static void __declspec(naked) asm_mirror_isse_row_1
						(BYTE* srcp,	//4
						int row,		//8
						int height,		//12
						int modulo)		//16
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov ecx,[esp+16+8]			//row
			mov esi,[esp+16+4]			//ptr to src
										//mov ebx,[esp+16+12]			//height
			mov edi,esi					//copy of ptr
			mov ebp,[esp+16+16]			//modulo
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
y_loop_0:	mov ebx,ecx
			sar ecx,5
			jz x_loop_0					//don't like controls inside the loop

			and ebx,31//15
//....................prefetcht0.......................................................................
align 16
ecxis8_0:
			//movq mm0,[esi]
			//movq mm2,[esi+8]

			//prefetchnta [edi-64]

			pshufw	mm0,[esi],00011011b
			pshufw	mm1,[esi],00011011b

			pshufw	mm2,[esi+8],00011011b
			pshufw	mm3,[esi+8],00011011b

			//movq    mm1,mm0
			//movq    mm3,mm2
			//pshufw	mm1,mm0,11100100b
			//pshufw	mm3,mm2,11100100b

			psllw	mm2,8
			psrlw	mm3,8 

			psllw	mm0,8
			psrlw	mm1,8

			por		mm0,mm1
			por		mm2,mm3

			//movq mm4,[edi-8]
			//movq mm6,[edi-16]

			//prefetchnta [esi+64]

			pshufw	mm4,[edi-8],00011011b
			pshufw	mm5,[edi-8],00011011b

			pshufw	mm6,[edi-16],00011011b
			pshufw	mm7,[edi-16],00011011b

			//movq	mm5,mm4
			//movq	mm7,mm6
			//pshufw	mm5,mm4,11100100b
			//pshufw	mm7,mm6,11100100b
			
			psllw	mm4,8
			psrlw	mm5,8

			psllw	mm6,8
			psrlw	mm7,8
			
			por		mm4,mm5
			por		mm6,mm7

			movq [edi-8],mm0
			movq [edi-16],mm2

			sub ecx,1

			movq [esi],mm4
			movq [esi+8],mm6

			lea edi,[edi-16]
			lea esi,[esi+16]

			jne  ecxis8_0
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
			mov ecx,ebx

			sar ecx,4
			jz prex_loop_0

			and ebx,15
//...........................................................................................
align 16
loop8:
			movq mm0,[esi]
			pshufw	mm0,mm0,00011011b
			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1

			movq mm4,[edi-8]
			pshufw	mm4,mm4,00011011b
			movq	mm5,mm4
			psllw	mm4,8
			psrlw	mm5,8
			por		mm4,mm5

			movq [edi-8],mm0
			sub ecx,1
			movq [esi],mm4

			lea edi,[edi-8]
			lea esi,[esi+8]

			jnz loop8
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
prex_loop_0:
			cmp ebx,2
			jl prenextline
//...........................................................................................
align 16
x_loop_0:	
			mov al,[esi]
			mov dl,[edi-1]

			sub ebx,2

			mov [edi-1],al
			mov [esi],dl

			lea edi,[edi-1]
			lea esi,[esi+1]

			cmp ebx,2
			jge x_loop_0
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
prenextline:
			lea esi,[esi+1]				//odd row, just chromas when luma just mod 2
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+8]			//row
			mov eax,ecx					//eax=row
			lea esi,[esi+ebp]			//first ptr+modulo
			sar eax,1					//eax=row/2 just to add with ptrs
			sub dword ptr[esp+16+12],1	//height -1

			lea esi,[esi+eax]			//first ptr+row/2
			mov edi,esi					//copy of ptr
			lea edi,[edi+ecx]			//sec ptr+row

			jne y_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}*/
/////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////////
static void __declspec(naked) asm_mirror_isse_row_2
						(BYTE* srcp,	//4
						int row,		//8
						int height,		//12
						int modulo)		//16
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov ecx,[esp+16+8]			//row
			mov esi,[esp+16+4]			//ptr to src
										//mov ebx,[esp+16+12]			//height
			mov edi,esi					//copy of ptr
			mov ebp,[esp+16+16]			//modulo
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
y_loop_0:	mov ebx,ecx
			sar ecx,6
			jz x_loop_0

			and ebx,63
//...........................................................................................
align 16
ecxis8_0:
			movq mm0,[esi]
			movq mm2,[esi+8]
			movq mm4,[esi+16]
			movq mm6,[esi+24]

			pshufw	mm0,mm0,00011011b
			pshufw	mm2,mm2,00011011b
			pshufw	mm4,mm4,00011011b
			pshufw	mm6,mm6,00011011b

			movq    mm1,mm0
			movq    mm3,mm2
			movq	mm5,mm4
			movq	mm7,mm6

			psllw	mm0,8
			psrlw	mm1,8

			psllw	mm2,8
			psrlw	mm3,8

			psllw	mm4,8
			psrlw	mm5,8

			psllw	mm6,8
			psrlw	mm7,8

			por		mm0,mm1
			por		mm2,mm3
			por		mm4,mm5
			por		mm6,mm7
//...........................................................................................
			movq mm1,[edi-8]
			movq mm3,[edi-16]
			movq mm5,[edi-24]
			movq mm7,[edi-32]

			//prefetchnta [esi+64]

			pshufw	mm1,mm1,00011011b
			pshufw	mm3,mm3,00011011b
			pshufw	mm5,mm5,00011011b
			pshufw	mm7,mm7,00011011b


			movq [edi-8],mm0
			movq [edi-16],mm2
			movq [edi-24],mm4
			movq [edi-32],mm6

			movq	mm0,mm1
			movq	mm2,mm3
			movq	mm4,mm5
			movq	mm6,mm7
			
			psllw	mm1,8
			psrlw	mm0,8

			psllw	mm3,8
			psrlw	mm2,8

			psllw	mm5,8
			psrlw	mm4,8

			psllw	mm7,8
			psrlw	mm6,8
			
			por		mm1,mm0
			por		mm3,mm2
			por		mm5,mm4
			por		mm7,mm6


			sub ecx,1

			movq [esi],mm1
			movq [esi+8],mm3
			movq [esi+16],mm5
			movq [esi+24],mm7

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne  ecxis8_0
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
			mov ecx,ebx

			sar ecx,4
			jz prex_loop_0

			and ebx,15
//...........................................................................................
align 16
loop8:
			movq mm0,[esi]
			movq mm4,[edi-8]

			pshufw	mm0,mm0,00011011b
			pshufw	mm4,mm4,00011011b

			movq    mm1,mm0
			movq	mm5,mm4

			psllw	mm0,8
			psllw	mm4,8

			psrlw	mm1,8 
			psrlw	mm5,8

			por		mm0,mm1
			por		mm4,mm5

			movq [edi-8],mm0
			sub ecx,1
			movq [esi],mm4

			lea edi,[edi-8]
			lea esi,[esi+8]

			jnz loop8
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
prex_loop_0:
			cmp ebx,2
			jl prenextline
//...........................................................................................
align 16
x_loop_0:	
			mov al,[esi]
			mov dl,[edi-1]

			sub ebx,2

			mov [edi-1],al
			mov [esi],dl

			lea edi,[edi-1]
			lea esi,[esi+1]

			cmp ebx,2
			jge x_loop_0
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
prenextline:
			lea esi,[esi+1]				//odd row, just chromas when luma just mod 2
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+8]			//row
			mov eax,ecx					//eax=row
			lea esi,[esi+ebp]			//first ptr+modulo
			sar eax,1					//eax=row/2 just to add with ptrs
			sub dword ptr[esp+16+12],1	//height -1

			lea esi,[esi+eax]			//first ptr+row/2
			mov edi,esi					//copy of ptr
			lea edi,[edi+ecx]			//sec ptr+row

			jne y_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////////
static void __declspec(naked) asm_mirror_sse2_row_2
						(BYTE* srcp,	//4
						int row,		//8
						int height,		//12
						int modulo)		//16
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov ecx,[esp+16+8]			//row
			mov esi,[esp+16+4]			//ptr to src
			mov edi,esi					//copy of ptr
			mov ebp,[esp+16+16]			//modulo
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
testalign:
			mov eax,esi
			or eax,edi

			test eax,15
			jnz y_loop_0
//...........................................................................................
align 16
loopaligned:
			mov ebx,ecx
			sar ecx,6
			jz x_loop_0

			and ebx,63
//...........................................................................................
align 16
loop64ecxal:
			movdqa xmm0,[esi]			//original B15B14B13B12 B11B10B09B08 B07B06B05B04 B03B02B01B00
										//			   DW3			DW2			DW1			DW0
										//         |.......HIGH WORDS.......||.......LOW WORDS.......|
			movdqa xmm2,[esi+16]

			pshufd	xmm0,xmm0,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW3
										//    |.......HIGH WORDS.......||.......LOW WORDS........|
			pshufd	xmm2,xmm2,00011011b// IDEM

										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm0,xmm0,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm0,xmm0,10110001B //GOT		B01B00 B03B02 B05B04 B07B06
										//high words..W7.. ..W6.. ..W5.. ..W4..
			pshuflw	xmm2,xmm2,10110001B
			pshufhw xmm2,xmm2,10110001B

			movdqa    xmm1,xmm0			//copy
			movdqa    xmm3,xmm2			//copy

			psllw	xmm0,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm1,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm0,xmm1			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there are no QWSWAP and DQWSWAP instructions.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm2,8
			psrlw	xmm3,8
			por		xmm2,xmm3
//...........................................................................................
			movdqa xmm4,[edi-16]
			movdqa xmm6,[edi-32]

			pshufd	xmm4,xmm4,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW0
										//    |.......HIGH WORDS.......||.......LOW WORDS........|

			pshufd	xmm6,xmm6,00011011b// IDEM
										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm4,xmm4,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm4,xmm4,10110001B //GOT		B01B00 B03B02 B05B04 B07B06

			pshuflw	xmm6,xmm6,10110001B
			pshufhw xmm6,xmm6,10110001B

			movdqa    xmm5,xmm4			//copy
			movdqa    xmm7,xmm6			//copy

			psllw	xmm4,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm5,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm4,xmm5			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there is no an QWSWAP and DQWSWAP instruction.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm6,8
			psrlw	xmm7,8
			por		xmm6,xmm7
//...........................................................................................
			movdqa [edi-16],xmm0
			movdqa [edi-32],xmm2

			sub ecx,1

			movdqa [esi],xmm4
			movdqa [esi+16],xmm6

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne  loop64ecxal

			jmp testremder
//...........................................................................................
align 16
y_loop_0:	
			test esi,15
			jz   preloop64ecx
align 16
test2align:
			test esi,3
			jz   test4align

			mov al,[esi]
			mov dl,[edi-1]

			sub ecx,2

			mov [edi-1],al
			mov [esi],dl

			lea edi,[edi-1]
			lea esi,[esi+1]

			jmp test2align
align 16
test4align: test esi,15
			jz testbothptrs

			movq mm0,[esi]
			movq mm4,[edi-8]

			pshufw	mm0,mm0,00011011b
			pshufw	mm4,mm4,00011011b

			movq    mm1,mm0
			movq	mm5,mm4

			psllw	mm0,8
			psllw	mm4,8

			psrlw	mm1,8 
			psrlw	mm5,8

			por		mm0,mm1
			por		mm4,mm5

			movd [edi-4],mm0
			sub ecx,8
			movd [esi],mm4

			lea edi,[edi-4]
			lea esi,[esi+4]

			jmp test4align
align 16
testbothptrs:
			test edi,15
			jz	loopaligned
align 16
preloop64ecx:
			mov ebx,ecx
			sar ecx,6
			jz x_loop_0

			and ebx,63
//...........................................................................................
align 16
loop64ecx:
			movdqa xmm0,[esi]			//original B15B14B13B12 B11B10B09B08 B07B06B05B04 B03B02B01B00
										//			   DW3			DW2			DW1			DW0
										//         |.......HIGH WORDS.......||.......LOW WORDS.......|
			movdqa xmm2,[esi+16]

			pshufd	xmm0,xmm0,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW3
										//    |.......HIGH WORDS.......||.......LOW WORDS........|

			pshufd	xmm2,xmm2,00011011b// IDEM
										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm0,xmm0,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm0,xmm0,10110001B //GOT		B01B00 B03B02 B05B04 B07B06
										//high words..W7.. ..W6.. ..W5.. ..W4..
			pshuflw	xmm2,xmm2,10110001B
			pshufhw xmm2,xmm2,10110001B

			movdqa    xmm1,xmm0			//copy
			movdqa    xmm3,xmm2			//copies

			psllw	xmm0,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm1,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm0,xmm1			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there are no QWSWAP and DQWSWAP instructions.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm2,8
			psrlw	xmm3,8
			por		xmm2,xmm3
//...........................................................................................
			movdqu xmm4,[edi-16]
			movdqu xmm6,[edi-32]

			pshufd	xmm4,xmm4,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW0
										//    |.......HIGH WORDS.......||.......LOW WORDS........|

			pshufd	xmm6,xmm6,00011011b// IDEM
										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm4,xmm4,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm4,xmm4,10110001B //GOT		B01B00 B03B02 B05B04 B07B06

			pshuflw	xmm6,xmm6,10110001B
			pshufhw xmm6,xmm6,10110001B

			movdqa    xmm5,xmm4			//copy
			movdqa    xmm7,xmm6			//copy

			psllw	xmm4,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm5,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm4,xmm5			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there is no an QWSWAP and DQWSWAP instruction.
			//(shuffles and shifts)
//...........................................................................................
			psllw	xmm6,8
			psrlw	xmm7,8
			por		xmm6,xmm7
//...........................................................................................
			movdqu [edi-16],xmm0
			movdqu [edi-32],xmm2

			sub ecx,1

			movdqa [esi],xmm4
			movdqa [esi+16],xmm6

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne  loop64ecx
//...........................................................................................
align 16
testremder:
			test ebx,ebx
			jz nextline
//...........................................................................................
			mov ecx,ebx
			sar ecx,4
			jz prex_loop_0

			and ebx,15
//...........................................................................................
align 16
loop8:
			movq mm0,[esi]
			movq mm4,[edi-8]

			pshufw	mm0,mm0,00011011b
			pshufw	mm4,mm4,00011011b

			movq    mm1,mm0
			movq	mm5,mm4

			psllw	mm0,8
			psllw	mm4,8

			psrlw	mm1,8 
			psrlw	mm5,8

			por		mm0,mm1
			por		mm4,mm5

			movq [edi-8],mm0
			movq [esi],mm4

			sub ecx,1

			lea edi,[edi-8]
			lea esi,[esi+8]

			jnz loop8
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
prex_loop_0:
			cmp ebx,2
			jl prenextline
//...........................................................................................
align 16
x_loop_0:	
			mov al,[esi]
			mov dl,[edi-1]

			sub ebx,2

			mov [edi-1],al
			mov [esi],dl

			lea edi,[edi-1]
			lea esi,[esi+1]

			cmp ebx,2
			jge x_loop_0
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
prenextline:
			lea esi,[esi+1]				//odd row, just chromas when luma just mod 2
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+8]			//row
			mov eax,ecx					//eax=row
			lea esi,[esi+ebp]			//first ptr+modulo
			sar eax,1					//eax=row/2 just to add with ptrs
			sub dword ptr[esp+16+12],1	//height -1

			lea esi,[esi+eax]			//first ptr+row/2
			mov edi,esi					//copy of ptr
			lea edi,[edi+ecx]			//sec ptr+row

			jne testalign
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
///////////////////////faster one till now; in pentium4 mobile and ML37//////////////////////
/////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////////
/*static void __declspec(naked) asm_mirror_plain_row_1
						(BYTE* srcp,	//4
						int row,		//8
						int height,		//12
						int modulo)		//16
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov ecx,[esp+16+8]			//row
			mov esi,[esp+16+4]			//ptr to src
										//mov ebx,[esp+16+12]			//height
			mov edi,esi					//copy of ptr
			mov ebp,[esp+16+16]			//modulo
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
y_loop_0:	mov ebx,ecx
			sar ecx,3
			jz x_loop_0					//don't like controls inside the loop

			and ebx,7
//...........................................................................................
align 16
ecxis8_0:
			mov eax,[esi]
			mov edx,[edi-4]

			sub ecx,1

			bswap eax
			bswap edx

			mov [edi-4],eax
			mov [esi],edx

			lea edi,[edi-4]
			lea esi,[esi+4]

			jne  ecxis8_0
//...........................................................................................
			test ebx,ebx
			jz nextline

			cmp ebx,2
			jl prenextline
//...........................................................................................
align 16
x_loop_0:	
			mov al,[esi]
			mov dl,[edi-1]

			sub ebx,2

			mov [edi-1],al
			mov [esi],dl

			lea edi,[edi-1]
			lea esi,[esi+1]

			cmp ebx,2
			jge x_loop_0
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
prenextline:
			lea esi,[esi+1]				//odd row, just chromas when luma just mod 2
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+8]			//row
			mov eax,ecx					//eax=row
			lea esi,[esi+ebp]			//first ptr+modulo
			sar eax,1					//eax=row/2 just to add with ptrs
			sub dword ptr[esp+16+12],1	//height -1

			lea esi,[esi+eax]			//first ptr+row/2
			mov edi,esi					//copy of ptr
			lea edi,[edi+ecx]			//sec ptr+row

			jne y_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}*/
/////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////////
static void __declspec(naked) asm_mirror_plain_row_2
						(BYTE* srcp,	//4
						int row,		//8
						int height,		//12
						int modulo)		//16
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov ecx,[esp+16+8]			//row
			mov esi,[esp+16+4]			//ptr to src
										//mov ebx,[esp+16+12]			//height
			mov edi,esi					//copy of ptr
										//mov ebp,[esp+16+16]			//modulo
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
y_loop_0:	
			cmp ecx,16
			jl x_loop_0					//don't like controls inside the loop
//...........................................................................................
align 16
ecxis8_0:
			mov eax,[esi]
			mov ebx,[esi+4]

			mov edx,[edi-4]
			mov ebp,[edi-8]

			sub ecx,16

			bswap eax
			bswap edx
			bswap ebx
			bswap ebp

			mov [edi-4],eax
			mov [edi-8],ebx

			mov [esi],edx
			mov [esi+4],ebp

			lea edi,[edi-8]
			lea esi,[esi+8]

			cmp ecx,16
			jge  ecxis8_0
//...........................................................................................
			test ecx,ecx
			jz nextline

			cmp ecx,2
			jl prenextline
//...........................................................................................
align 16
x_loop_0:	
			mov al,[esi]
			mov dl,[edi-1]

			sub ecx,2

			mov [edi-1],al
			mov [esi],dl

			lea edi,[edi-1]
			lea esi,[esi+1]

			cmp ecx,2
			jge x_loop_0
//...........................................................................................
			test ecx,ecx
			jz nextline
//...........................................................................................
align 16
prenextline:
			lea esi,[esi+1]				//odd row, just chromas when luma just mod 2
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+8]			//row
			mov eax,ecx					//eax=row
			add esi,[esp+16+16]			//modulo
			sar eax,1					//eax=row/2 just to add with ptrs
			sub dword ptr[esp+16+12],1	//height -1

			lea esi,[esi+eax]			//first ptr+row/2
			mov edi,esi					//copy of ptr
			lea edi,[edi+ecx]			//sec ptr+row

			jne y_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
/////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////////
static void __declspec(naked) asm_mirror_isse_row_2_dst
						(const BYTE* srcp,	//4
						BYTE* dstp,			//8
						int row,			//12
						int height,			//16
						int modulo,			//20
						int modulodst)		//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst// 
			add edi,ecx					//ptr to last byte same row
align 16
y_loop_0:	mov ebx,ecx
			sar ecx,5
			jz pre_loop8

			and ebx,31
//...........................................................................................
align 16
ecxis8_0:
			movq mm0,[esi]
			movq mm2,[esi+8]
			movq mm4,[esi+16]
			movq mm6,[esi+24]

			pshufw	mm0,mm0,00011011b
			pshufw	mm2,mm2,00011011b
			pshufw	mm4,mm4,00011011b
			pshufw	mm6,mm6,00011011b

			movq    mm1,mm0
			movq    mm3,mm2
			movq	mm5,mm4
			movq	mm7,mm6

			psllw	mm0,8
			psrlw	mm1,8

			psllw	mm2,8
			psrlw	mm3,8

			psllw	mm4,8
			psrlw	mm5,8

			psllw	mm6,8
			psrlw	mm7,8

			por		mm0,mm1
			por		mm2,mm3
			por		mm4,mm5
			por		mm6,mm7
//...........................................................................................
			movq [edi-8],mm0
			movq [edi-16],mm2
			movq [edi-24],mm4
			movq [edi-32],mm6

			sub ecx,1

			lea esi,[esi+32]
			lea edi,[edi-32]

			jne  ecxis8_0
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
pre_loop8:
			mov ecx,ebx

			sar ecx,3
			jz x_loop_0

			and ebx,7
//...........................................................................................
align 16
loop8:
			movq mm0,[esi]
			pshufw	mm0,mm0,00011011b

			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1

			sub ecx,1

			movq [edi-8],mm0

			lea esi,[esi+8]
			lea edi,[edi-8]

			jnz loop8
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
x_loop_0:	
			mov al,[esi]

			sub ebx,1

			mov [edi-1],al

			lea edi,[edi-1]
			lea esi,[esi+1]

			jne x_loop_0
//...........................................................................................*/
align 16
nextline:
			mov ecx,[esp+16+12]			//row

			add esi,[esp+16+20]			//first ptr+modulo

			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx					//

			sub dword ptr [esp+16+16],1	//height -1

			jne y_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////////
static void __declspec(naked) asm_mirror_isse_row_2_dst_nta
						(const BYTE* srcp,	//4
						BYTE* dstp,		//8
						int row,		//12
						int height,		//16
						int modulo,		//20
						int modulodst)	//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst// 
			mov ebp,[esp+16+20]
			add edi,ecx					//ptr to last byte same row
			add ebp,ecx
align 16
y_loop_0:
			mov ebx,ecx

			sar ecx,5
			jz pre_loop8

			and ebx,31
//...........................................................................................
align 16
ecxis8_0:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movq mm0,[esi]
			movq mm2,[esi+8]
			movq mm4,[esi+16]
			movq mm6,[esi+24]

			pshufw	mm0,mm0,00011011b
			pshufw	mm2,mm2,00011011b
			pshufw	mm4,mm4,00011011b
			pshufw	mm6,mm6,00011011b

			movq    mm1,mm0
			movq    mm3,mm2
			movq	mm5,mm4
			movq	mm7,mm6

			psllw	mm0,8
			psrlw	mm1,8

			psllw	mm2,8
			psrlw	mm3,8

			psllw	mm4,8
			psrlw	mm5,8

			psllw	mm6,8
			psrlw	mm7,8

			por		mm0,mm1
			por		mm2,mm3
			por		mm4,mm5
			por		mm6,mm7
//...........................................................................................
			movntq [edi-8],mm0
			movntq [edi-16],mm2
			movntq [edi-24],mm4
			movntq [edi-32],mm6


			lea esi,[esi+32]
			sub edi,32

			sub ecx,1

			jne  ecxis8_0
//...........................................................................................
			sfence
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
pre_loop8:
			mov ecx,ebx

			sar ecx,3
			jz x_loop_0

			and ebx,7
//...........................................................................................
align 16
loop8:
			movq mm0,[esi]
			pshufw	mm0,mm0,00011011b

			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1
			movntq [edi-8],mm0

			lea esi,[esi+8]
			sub edi,8

			sub ecx,1
			jnz loop8

			sfence
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
x_loop_0:	
			mov al,[esi]

			sub ebx,1

			mov [edi-1],al

			lea edi,[edi-1]
			lea esi,[esi+1]

			jne x_loop_0
//...........................................................................................*/
align 16
nextline:
			mov ecx,[esp+16+12]			//row

			add esi,[esp+16+20]			//first ptr+modulo

			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx					//
			sub dword ptr [esp+16+16],1	//height -1
			jne y_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////
static void __declspec(naked) asm_mirror_sse2_row_2_dst
						(const BYTE* srcp,	//4
						BYTE* dstp,			//8
						int row,			//12
						int height,			//16
						int modulo,			//20
						int modulodst)		//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst// 
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
testalign:
			mov eax,esi
			or eax,edi

			test eax,15
			jnz y_loop_0
//...........................................................................................
align 16
loopaligned:
			mov ebx,ecx

			sar ecx,5
			jz testremder

			and ebx,31
//...........................................................................................
align 16
loop64ecxal:

			movdqa xmm0,[esi]			//original B15B14B13B12 B11B10B09B08 B07B06B05B04 B03B02B01B00
										//			   DW3			DW2			DW1			DW0
										//         |.......HIGH WORDS.......||.......LOW WORDS.......|
			movdqa xmm2,[esi+16]
			pshufd	xmm0,xmm0,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW3
										//    |.......HIGH WORDS.......||.......LOW WORDS........|
			pshufd	xmm2,xmm2,00011011b	// IDEM

										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm0,xmm0,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm0,xmm0,10110001B //GOT		B01B00 B03B02 B05B04 B07B06
										//high words..W7.. ..W6.. ..W5.. ..W4..
			pshuflw	xmm2,xmm2,10110001B
			pshufhw xmm2,xmm2,10110001B

			movdqa    xmm1,xmm0			//copies
			movdqa    xmm3,xmm2			//copy

			psllw	xmm0,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm1,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm0,xmm1			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there are no QWSWAP and DQWSWAP instructions.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm2,8
			psrlw	xmm3,8
			por		xmm2,xmm3
//...........................................................................................
			movdqa [edi-16],xmm0
			movdqa [edi-32],xmm2

			sub ecx,1

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne  loop64ecxal

			jmp testremder
//...........................................................................................
align 16
y_loop_0:	test esi,15
			jz   preloop64ecx
align 16
test2align:
			test esi,3
			jz   test4align

			mov al,[esi]

			sub ecx,1

			mov [edi-1],al

			lea edi,[edi-1]
			lea esi,[esi+1]

			jmp test2align
align 16
test4align: 
			test esi,15
			jz testbothptrs

			movq mm0,[esi]
			pshufw	mm0,mm0,00011011b
			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1
			movd [edi-4],mm0
			sub ecx,4

			lea esi,[esi+4]
			lea edi,[edi-4]

			jmp test4align
align 16
testbothptrs:
			test edi,15
			jz	loopaligned
align 16
preloop64ecx:
			mov ebx,ecx

			sar ecx,5
			jz testremder

			and ebx,31
//...........................................................................................
align 16
loop64ecx:

			movdqa xmm0,[esi]			//original B15B14B13B12 B11B10B09B08 B07B06B05B04 B03B02B01B00
										//			   DW3			DW2			DW1			DW0
										//         |.......HIGH WORDS.......||.......LOW WORDS.......|
			movdqa xmm2,[esi+16]

			pshufd	xmm0,xmm0,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW3
										//    |.......HIGH WORDS.......||.......LOW WORDS........|

			pshufd	xmm2,xmm2,00011011b// IDEM

										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm0,xmm0,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm0,xmm0,10110001B //GOT		B01B00 B03B02 B05B04 B07B06
										//high words..W7.. ..W6.. ..W5.. ..W4..
			pshuflw	xmm2,xmm2,10110001B
			pshufhw xmm2,xmm2,10110001B

			movdqa    xmm1,xmm0			//copy
			movdqa    xmm3,xmm2			//copy

			psllw	xmm0,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm1,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm0,xmm1			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there are no QWSWAP and DQWSWAP instructions.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm2,8
			psrlw	xmm3,8
			por		xmm2,xmm3
//...........................................................................................
			movdqu [edi-16],xmm0
			movdqu [edi-32],xmm2

			sub ecx,1

			lea esi,[esi+32]
			lea edi,[edi-32]

			jne  loop64ecx
//...........................................................................................
align 16
testremder:
			test ebx,ebx
			jz nextline
//...........................................................................................
			mov ecx,ebx
			sar ecx,3
			jz prex_loop_0

			and ebx,7
//...........................................................................................
align 16
loop8:
			movq mm0,[esi]

			pshufw	mm0,mm0,00011011b

			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1
			movq [edi-8],mm0
			sub ecx,1

			lea esi,[esi+8]
			lea edi,[edi-8]

			jnz loop8
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
prex_loop_0:
//...........................................................................................
			mov ecx,ebx
			sar ecx,2
			jz x_loop_0
//...........................................................................................
			and ebx,3
//...........................................................................................
align 16
loop4:
			movq	mm0,[esi]

			pshufw	mm0,mm0,00011011b

			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1
			movd	[edi-4],mm0

			sub		ecx,1

			lea esi,[esi+4]
			lea edi,[edi-4]

			jnz loop4
//...........................................................................................
			test ebx,ebx
			jz nextline
align 16
x_loop_0:	
			mov al,[esi]
			sub ebx,1
			mov [edi-1],al

			lea esi,[esi+1]
			lea edi,[edi-1]

			jne x_loop_0
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+12]			//row

			add esi,[esp+16+20]			//first ptr+modulo

			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx					//

			sub dword ptr [esp+16+16],1	//height -1

			jne testalign
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////
static void __declspec(naked) asm_mirror_sse2_row_2_dst_nta
						(const BYTE* srcp,	//4
						BYTE* dstp,			//8
						int row,			//12
						int height,			//16
						int modulo,			//20
						int modulodst)		//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst// 
			mov ebp,[esp+16+20]
			add edi,ecx					//ptr to last byte same row
			add ebp,ecx					//copy of row
//...........................................................................................
align 16
testalign:
			mov eax,esi
			or eax,edi

			test eax,15
			jnz y_loop_0
//...........................................................................................
align 16
loopaligned:
			mov ebx,ecx

			sar ecx,5
			//jz x_loop_0					//not needed

			and ebx,31
//...........................................................................................
align 16
loop64ecxal:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movdqa xmm0,[esi]			//original B15B14B13B12 B11B10B09B08 B07B06B05B04 B03B02B01B00
										//			   DW3			DW2			DW1			DW0
										//         |.......HIGH WORDS.......||.......LOW WORDS.......|
			movdqa xmm2,[esi+16]

			pshufd	xmm0,xmm0,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW3
										//    |.......HIGH WORDS.......||.......LOW WORDS........|

			pshufd	xmm2,xmm2,00011011b// IDEM

										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..

			pshuflw	xmm0,xmm0,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm0,xmm0,10110001B //GOT		B01B00 B03B02 B05B04 B07B06

										//high words..W7.. ..W6.. ..W5.. ..W4..
			pshuflw	xmm2,xmm2,10110001B
			pshufhw xmm2,xmm2,10110001B

			movdqa    xmm1,xmm0			//copy
			movdqa    xmm3,xmm2			//copy

			psllw	xmm0,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm1,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm0,xmm1			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there are no QWSWAP and DQWSWAP instructions.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm2,8
			psrlw	xmm3,8
			por		xmm2,xmm3
//...........................................................................................
			movntdq [edi-16],xmm0
			movntdq [edi-32],xmm2

			sub edi,32
			lea esi,[esi+32]

			sub ecx,1

			jne  loop64ecxal
			sfence

			jmp testremder
//...........................................................................................
align 16
y_loop_0:	test edi,15
			jz   preloop64ecx
align 16
test2align:
			test edi,3
			jz   test4align

			mov al,[esi]
			sub ecx,1
			mov [edi-1],al

			sub edi,1
			lea esi,[esi+1]

			jmp test2align
align 16
test4align: 
			test edi,15
			jz testbothptrs

			movq mm0,[esi]
			pshufw	mm0,mm0,00011011b
			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1
			movd [edi-4],mm0

			sub ecx,4

			lea edi,[edi-4]
			lea esi,[esi+4]

			jmp test4align
align 16
testbothptrs:
			test esi,15
			jz	loopaligned
align 16
preloop64ecx:
			mov ebx,ecx
			sar ecx,5
//...........................................................................................
			and ebx,31
//...........................................................................................
align 16
loop64ecx:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movdqu xmm0,[esi]			//original B15B14B13B12 B11B10B09B08 B07B06B05B04 B03B02B01B00
										//			   DW3			DW2			DW1			DW0
										//         |.......HIGH WORDS.......||.......LOW WORDS.......|
			movdqu xmm2,[esi+16]

			pshufd	xmm0,xmm0,00011011b //GOT B03B02B01B00 B07B06B05B04 B11B10B09B08 B15B14B13B12
										//		DW0			   DW1			   DW2			DW3
										//    |.......HIGH WORDS.......||.......LOW WORDS........|

			pshufd	xmm2,xmm2,00011011b// IDEM

										//low words B11B10 B09B08 B15B14 B13B12
										//          ..W3.. ..W2.. ..W1.. ..W0..
			pshuflw	xmm0,xmm0,10110001B //GOT       B09B08 B11B10 B13B12 B15B14
			pshufhw xmm0,xmm0,10110001B //GOT		B01B00 B03B02 B05B04 B07B06
										//high words..W7.. ..W6.. ..W5.. ..W4..
			pshuflw	xmm2,xmm2,10110001B
			pshufhw xmm2,xmm2,10110001B

			movdqa    xmm1,xmm0			//copy
			movdqa    xmm3,xmm2			//copy

			psllw	xmm0,8				//GOT B00BXXB02BXX B04BXXB06BXX B08BXXB10BXX B12BXXB14BXX  
			psrlw	xmm1,8				//GOT BXXB01BXXB03 BXXB05BXXB07 BXXB09BXXB11 BXXB13BXXB15
			por		xmm0,xmm1			//GOT B00B01B02B03 B04B05B06B07 B08B09B10B11 B12B13B14B15

			// all these sh... cause there are no QWSWAP and DQWSWAP instructions.
			//(shuffles and shifts, what have you thought?)
//...........................................................................................
			psllw	xmm2,8
			psrlw	xmm3,8
			por		xmm2,xmm3
//...........................................................................................
			movntdq [edi-16],xmm0
			movntdq [edi-32],xmm2


			lea esi,[esi+32]
			sub edi,32

			sub ecx,1

			jne  loop64ecx
//...........................................................................................
			sfence
align 16
testremder:
			test ebx,ebx
			jz nextline
//...........................................................................................
			mov ecx,ebx
			sar ecx,3
			jz prex_loop_0

			and ebx,7
//...........................................................................................
align 16
loop8:
			movq mm0,[esi]

			pshufw	mm0,mm0,00011011b

			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1
			movntq [edi-8],mm0

			lea esi,[esi+8]
			sub edi,8

			sub ecx,1

			jnz loop8
//...........................................................................................
			sfence

			test ebx,ebx
			jz nextline
//......................add a loop four bytes................................................
align 16
prex_loop_0:
//...........................................................................................
			mov ecx,ebx
			sar ecx,2
			jz x_loop_0
//...........................................................................................
			and ebx,3
//...........................................................................................
align 16
loop4:
			movq mm0,[esi]

			pshufw	mm0,mm0,00011011b

			movq    mm1,mm0
			psllw	mm0,8
			psrlw	mm1,8 
			por		mm0,mm1
			movd [edi-4],mm0
			sub ecx,1

			lea esi,[esi+4]
			lea edi,[edi-4]

			jnz loop4
//...........................................................................................
			test ebx,ebx
			jz nextline
align 16
x_loop_0:	
			mov al,[esi]
			sub ebx,1
			mov [edi-1],al

			lea esi,[esi+1]
			lea edi,[edi-1]

			jne x_loop_0
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+12]			//row

			add esi,[esp+16+20]			//first ptr+modulo

			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx					//

			sub dword ptr [esp+16+16],1	//height -1

			jne testalign
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
///////////////////////faster one till now; in pentium4 mobile and ML37//////////////////////
/////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////////
static void __declspec(naked) asm_mirror_plain_row_1_dst
						(const BYTE* srcp,	//4
						BYTE* dstp,		//8
						int row,		//12
						int height,		//16
						int modulo,		//20
						int modulodst)	//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst// 
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
y_loop_0:	mov ebx,ecx
			sar ecx,2
			jz x_loop_0

			and ebx,3
//...........................................................................................
align 16
ecxis8_0:
			mov eax,[esi]

			bswap eax

			sub ecx,1
			mov [edi-4],eax

			lea esi,[esi+4]
			lea edi,[edi-4]

			jne  ecxis8_0
//...........................................................................................
			test ebx,ebx
			jz nextline
//...........................................................................................
align 16
x_loop_0:	
			mov al,[esi]
			sub ebx,1
			mov [edi-1],al

			lea esi,[esi+1]
			lea edi,[edi-1]

			jne x_loop_0
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+12]			//row

			add esi,[esp+16+20]			//first ptr+modulo

			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx					//

			sub dword ptr [esp+16+16],1	//height -1
			jne y_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
//////////////////Copyright (c) Enrique Ardanaz (ARDA)///////////////////////////////////////
//////////////////probably not used//////////////////////////////////////////////////////////
static void __declspec(naked) asm_mirror_rgb_row_dst
						(const BYTE* srcp,	//4
						BYTE* dstp,		//8
						int row,		//12
						int height,		//16
						int modulo,		//20
						int modulodst)	//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst// 
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
y_loop:
			mov	edx,ecx

			sar ecx,2
			jz	remind2
//...........................................................................................
			and edx,3
//...........................................................................................
align 16
ecxis4_0:
			mov eax,[esi]

			sub ecx,1

			mov [edi-4],eax

			lea esi,[esi+4]
			lea edi,[edi-4]

			jne ecxis4_0				
//...........................................................................................
			test edx,edx
			jz nextline
align 16
remind2:
			lea edi,[edi-2]
			lea esi,[esi+2]
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+12]			//row
			add esi,[esp+16+20]			//first ptr+modulo
			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx					//

			sub dword ptr [esp+16+16],1	//height -1
			jne y_loop
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
//////////////////Copyright (c) Enrique Ardanaz (ARDA)///////////////////////////////////////
static void __declspec(naked) asm_mirror_rgb_row
						(BYTE* srcp,	//4
						int row,		//8
						int height,		//12
						int modulo)		//16
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+8]			//row
			mov ebx,[esp+16+12]			//height
			mov edi,esi					//copy of ptr
			mov ebp,[esp+16+16]			//modulo
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
ecxis8_0:
			mov eax,[esi]
			mov edx,[edi-4]
			sub ecx,8
			mov [edi-4],eax
			mov [esi],edx
			cmp ecx,8
			lea edi,[edi-4]
			lea esi,[esi+4]
			jge ecxis8_0				

			test ecx,ecx
			jz nextline

			lea edi,[edi-2]
			lea esi,[esi+2]
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+8]			//row
										//lea edi,[edi+ebp]			//ptr + modulo
			lea esi,[esi+ebp]			//first ptr+modulo
			mov eax,ecx					//eax=row
			sar eax,1					//eax=row/2 just to add with ptrs
			sub ebx,1					//height -1
			lea esi,[esi+eax]			//first ptr+row/2
										//lea edi,[edi+eax]			//sec ptr+row/2. Finally row+(1 1/2row)??
			mov edi,esi
			lea edi,[edi+ecx]			//sec ptr+row
			jne ecxis8_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
////////////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////
static void __declspec(naked) asm_mirror_rgb24_row
						(BYTE* srcp,	//4
						int row,		//8
						int height,		//12
						int modulo)		//16
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+8]			//row
			mov edi,esi					//copy of ptr
			mov ebp,[esp+16+16]			//modulo
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
ecxis6_0:
			mov ax,[esi]
			mov bl,[esi+2]

			mov dx,[edi-3]
			mov bh,[edi-1]

			sub ecx,6

			mov [edi-3],ax
			mov [edi-1],bl

			mov [esi],dx
			mov [esi+2],bh

			lea edi,[edi-3]
			lea esi,[esi+3]

			cmp ecx,6
			jge ecxis6_0
//...........................................................................................
			mov ecx,[esp+16+8]			//row
										//lea edi,[edi+ebp]			//ptr + modulo
			mov eax,ecx					//eax=row
			mov edx,ecx
			lea esi,[esi+ebp]			//first ptr+modulo
			sar eax,1					//eax=row/2 just to add with ptrs
//...........................................................................................
										///lea edi,[edi+ecx]			//sec ptr+row
			lea esi,[esi+eax]			//first ptr+row/2
			mov edi,esi
			add edi,ecx
										//lea edi,[edi+eax]			//sec ptr+row/2. Finally row+(1 1/2row)??

			and edx,1					//added for width odd in rgb24
			jz noddrow

			sub edi,1					// ??
			add esi,2					// middle pixel we do nothing??
//...........................................................................................
align 16
noddrow:
			sub dword ptr[esp+16+12],1	//height -1
			jne ecxis6_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
////////////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////
static void __declspec(naked) asm_mirror_rgb24_row_dst
						(const BYTE* srcp,	//4
						BYTE* dstp,		//8
						int row,		//12
						int height,		//16
						int modulo,		//20
						int modulodst)	//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov ebp,[esp+16+16]
			mov edi,[esp+16+8]			//ptr to dst// 
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
ecxis6_0:
			mov ax,[esi]
			mov bl,[esi+2]

			sub ecx,3

			mov [edi-3],ax
			mov [edi-1],bl

			lea edi,[edi-3]
			lea esi,[esi+3]

			cmp ecx,3
			jge ecxis6_0
//...........................................................................................
			mov ecx,[esp+16+12]			//row
			add esi,[esp+16+20]			//first ptr+modulo
			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx					//

			sub ebp,1					//dword ptr [esp+16+16],1		//height -1

			jne ecxis6_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
////////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////////
static void __declspec(naked) asm_mirror_rgb_sse2_exp2
						(BYTE* srcp,	//4
						int row,		//8
						int height,		//12
						int modulo)		//16
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+8]			//row
			mov ebx,[esp+16+12]			//height
			mov edi,esi					//copy of ptr
			mov ebp,[esp+16+16]			//modulo
			add edi,ecx					//ptr to last byte same row
align 16
tstptrs:
			mov eax,esi
			or eax,edi

			test eax,15
			jnz prex_loop_0 
//...........................................................................................
align 16
bothaligned:
			mov edx,ecx
			sar ecx,6
			jz  lower32

			and edx,63
align 16
alignloop:
			movdqa xmm2,[esi]
			movdqa xmm5,[esi+16]

			pshufd xmm2,xmm2,00011011b
			pshufd xmm5,xmm5,00011011b

			movdqa xmm3,[edi-16]
			movdqa xmm7,[edi-32]

			pshufd xmm3,xmm3,00011011b
			pshufd xmm7,xmm7,00011011b

			movdqa [edi-16],xmm2
			movdqa [edi-32],xmm5

			sub ecx,1

			movdqa [esi],xmm3
			movdqa [esi+16],xmm7

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne alignloop
//...........................................................................................
			jmp	tstnxtline
//...........................................................................................
align 16
prex_loop_0:
			test esi,15				//here there could be a bug ?? 
			jz   x_loop_0
//...........................................................................................
			mov edx,ecx				//don't like controls in the loop. como evitarlo?? exit if never arrives
			cmp ecx,64				// or arrives with a row smaller than 64 .necessary??happens??
			jl lower32

			mov eax,[esi]
			mov edx,[edi-4]

			sub ecx,8

			mov [edi-4],eax
			mov [esi],edx

			lea edi,[edi-4]
			lea esi,[esi+4]

			jmp	prex_loop_0
align 16
x_loop_0:	test edi,15				//probably uncessary
			jz   bothaligned
//...........................................................................................
			mov edx,ecx
			sar ecx,6
			jz  lower32

			and edx,63
//............................seems useless??................................................
align 16
ecxis8_0:
			movdqa xmm2,[esi]
			movdqa xmm5,[esi+16]

			pshufd xmm2,xmm2,00011011b
			pshufd xmm5,xmm5,00011011b

			movdqu xmm3,[edi-16]
			movdqu xmm7,[edi-32]

			pshufd xmm3,xmm3,00011011b
			pshufd xmm7,xmm7,00011011b

			movdqu [edi-16],xmm2
			movdqu [edi-32],xmm5

			sub ecx,1

			movdqa [esi],xmm3
			movdqa [esi+16],xmm7

			lea edi,[edi-32]
			lea esi,[esi+32]

			jne ecxis8_0
//...........................................................................................
align 16
tstnxtline:
			test edx,edx
			jz nextline
//...........................................................................................
align 16
lower32:
			mov ecx,edx

			cmp edx,8
			jl lower8
//...........................................................................................
align 16
reminder8:
  			mov eax,[esi]
			mov edx,[edi-4]

			sub ecx,8

			mov [edi-4],eax
			mov [esi],edx

			lea edi,[edi-4]
			lea esi,[esi+4]

			cmp ecx,8
			jge reminder8
//...........................................................................................
			test ecx,ecx
			jz  nextline
align 16
lower8:
			lea edi,[edi-2]
			lea esi,[esi+2]
align 16
nextline:
			mov ecx,[esp+16+8]			//row
										//lea edi,[edi+ebp]			//ptr + modulo
			lea esi,[esi+ebp]			//first ptr+modulo
			mov eax,ecx					//eax=row
			sar eax,1					//eax=row/2 just to add with ptrs
			sub ebx,1					//height -1
			lea esi,[esi+eax]			//first ptr+row/2
			mov edi,esi
			lea edi,[edi+ecx]			//sec ptr+row
			jne tstptrs
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
////////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////////
//cambiar busqueda alineamiento, dar prioridad al destino////////////////////////////////////
static void __declspec(naked) asm_mirror_rgb_sse2_exp2_dst
						(const BYTE* srcp,	//4
						BYTE* dstp,			//8
						int row,			//12
						int height,			//16
						int modulo,			//20
						int modulodst)		//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst// 
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
tstptrs:
			mov eax,esi
			or eax,edi

			test eax,15
			jnz prex_loop_0 
//...........................................................................................
align 16
bothaligned:
			mov edx,ecx
			sar ecx,6
			jz  lower32

			and edx,63
align 16
alignloop:
			movdqa xmm0,[esi]
			movdqa xmm1,[esi+16]
			movdqa xmm2,[esi+32]
			movdqa xmm3,[esi+48]

			pshufd xmm0,xmm0,00011011b
			pshufd xmm1,xmm1,00011011b
			pshufd xmm2,xmm2,00011011b
			pshufd xmm3,xmm3,00011011b

			movdqa [edi-16],xmm0
			movdqa [edi-32],xmm1
			movdqa [edi-48],xmm2
			movdqa [edi-64],xmm3

			sub ecx,1

			lea edi,[edi-64]
			lea esi,[esi+64]

			jne alignloop
//...........................................................................................
			jmp	tstnxtline
//...........................................................................................
align 16
prex_loop_0:
			test esi,15
			jz   x_loop_0
//...........................................................................................
			mov edx,ecx
			cmp ecx,32
			jl lower32

			mov eax,[esi]

			sub ecx,4

			mov [edi-4],eax

			lea esi,[esi+4]
			lea edi,[edi-4]

			jmp	prex_loop_0
align 16
x_loop_0:	test edi,15				//probably unecssary??
			jz   bothaligned
//...........................................................................................
			mov edx,ecx
			sar ecx,6
			jz  lower32

			and edx,63
//...........................................................................................
align 16
ecxis8_0:
			movdqa xmm0,[esi]
			movdqa xmm1,[esi+16]
			movdqa xmm2,[esi+32]
			movdqa xmm3,[esi+48]

			pshufd xmm0,xmm0,00011011b
			pshufd xmm1,xmm1,00011011b
			pshufd xmm2,xmm2,00011011b
			pshufd xmm3,xmm3,00011011b

			movdqu [edi-16],xmm0
			movdqu [edi-32],xmm1
			movdqu [edi-48],xmm2
			movdqu [edi-64],xmm3

			sub ecx,1

			lea edi,[edi-64]
			lea esi,[esi+64]

			jne ecxis8_0
//...........................................................................................
align 16
tstnxtline:
			test edx,edx
			jz nextline
//....................should include a 8 bytes loop?.........................................
align 16
lower32:
			mov ecx,edx

			cmp edx,4		//no deberia pasar nunca ??
			jl lower4
//...........................................................................................
align 16
reminder4:
  			mov eax,[esi]
			sub ecx,4
			mov [edi-4],eax

			lea esi,[esi+4]
			lea edi,[edi-4]

			cmp ecx,4
			jge reminder4
//...........................................................................................
			test ecx,ecx
			jz  nextline
//...........................................................................................
align 16
lower4:									//no deberia pasar nunca??
			lea edi,[edi-2]
			lea esi,[esi+2]
align 16
nextline:
			mov ecx,[esp+16+12]			//row
			add esi,[esp+16+20]			//first ptr+modulo
			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx					//

			sub dword ptr [esp+16+16],1	//height -1
			jne tstptrs
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
////////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////////
static void __declspec(naked) asm_mirror_rgb_sse2_exp2_dst_nta
						(const BYTE* srcp,	//4
						BYTE* dstp,			//8
						int row,			//12
						int height,			//16
						int modulo,			//20
						int modulodst)		//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst// 
			mov ebp,[esp+16+20]
			add edi,ecx					//ptr to last byte same row
			add ebp,ecx
//...........................................................................................
align 16
tstptrs:	
			mov eax,esi
			or eax,edi

			test eax,15
			jnz prex_loop_0 
//...........................................................................................
align 16
bothaligned:
			mov edx,ecx
			sar ecx,6
//...........................................................................................
			and edx,63
align 16
alignloop:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movdqa xmm0,[esi]
			movdqa xmm1,[esi+16]
			movdqa xmm2,[esi+32]
			movdqa xmm3,[esi+48]

			pshufd xmm0,xmm0,00011011b
			pshufd xmm1,xmm1,00011011b
			pshufd xmm2,xmm2,00011011b
			pshufd xmm3,xmm3,00011011b

			movntdq [edi-16],xmm0
			movntdq [edi-32],xmm1
			movntdq [edi-48],xmm2
			movntdq [edi-64],xmm3

			lea esi,[esi+64]
			sub edi,64
			sub ecx,1

			jne alignloop
			sfence
//...........................................................................................
			jmp	tstnxtline
//...........................................................................................
align 16
prex_loop_0:
			test edi,15
			jz   x_loop_0
//...........................................................................................
			mov edx,ecx	

			mov eax,[esi]
			sub ecx,4
			mov [edi-4],eax

			lea esi,[esi+4]
			sub edi,4

			jmp	prex_loop_0
align 16
x_loop_0:	test esi,15
			jz   bothaligned
//...........................................................................................
			mov edx,ecx
			sar ecx,6//5
//...........................................................................................
			and edx,63//31
//...........................................................................................
align 16
ecxis8_0:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movdqu xmm0,[esi]
			movdqu xmm1,[esi+16]
			movdqu xmm2,[esi+32]
			movdqu xmm3,[esi+48]

			pshufd xmm0,xmm0,00011011b
			pshufd xmm1,xmm1,00011011b
			pshufd xmm2,xmm2,00011011b
			pshufd xmm3,xmm3,00011011b

			movntdq [edi-16],xmm0
			movntdq [edi-32],xmm1
			movntdq [edi-48],xmm2
			movntdq [edi-64],xmm3

			lea esi,[esi+64]
			sub edi,64

			sub ecx,1
			jne ecxis8_0

			sfence
//...........................................................................................
align 16
tstnxtline:
			test edx,edx
			jz nextline
//....................should include a 8 bytes loop?.........................................
//align 16
//lower32:
			mov ecx,edx

			cmp edx,4		//no deberia pasar nunca??
			jl lower4
//...........................................................................................
align 16
reminder4:					//test this part maybe a bug 9feb 18:32
  			mov eax,[esi]
			sub ecx,4
			mov [edi-4],eax

			lea esi,[esi+4]
			sub edi,4

			cmp ecx,4
			jge reminder4
//...........................................................................................
			test ecx,ecx
			jz  nextline
//...........................................................................................
align 16
lower4:											//no deberia pasar nunca??
			sub edi,2
			lea esi,[esi+2]
align 16
nextline:
			mov ecx,[esp+16+12]					//row
			add esi,[esp+16+20]					//first ptr+modulo
			add edi,[esp+16+24]					//scnd ptr+modulo
			add edi,ecx
			add edi,ecx							//

			sub dword ptr [esp+16+16],1			//height -1
			jne tstptrs
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
////////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////////
static void __declspec(naked) asm_mirror_rgb_mmx_exp_dst
						(const BYTE* srcp,	//4
						BYTE* dstp,		//8
						int row,		//12
						int height,		//16
						int modulo,		//20
						int modulodst)	//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst// 
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
x_loop_0:
			mov edx,ecx
			sar ecx,4
			jz lower16

			and edx,15
//...........................................................................................
align 16
ecxis8_0:
			movd mm0,[esi]
			movd mm2,[esi+4]
			movd mm4,[esi+8]
			movd mm5,[esi+12]

			psllq mm0,32
			psllq mm4,32

			por mm0,mm2
			por mm4,mm5

			movq [edi-8],mm0
			movq [edi-16],mm4

			sub ecx,1

			lea esi,[esi+16]
			lea edi,[edi-16]

			jne ecxis8_0
//...........................................................................................
			test edx,edx
			jz nextline
//...........................................................................................
align 16
lower16:

			mov ecx,edx

			sar	ecx,2
			jz  lower4

			and edx,3

//...........................................................................................
align 16
reminder8:
  			mov eax,[esi]
			sub ecx,1
			mov [edi-4],eax

			lea esi,[esi+4]
			lea edi,[edi-4]

			jne reminder8
//...........................................................................................
			test edx,edx
			jz  nextline
align 16
lower4:									//esto no deberia pasar nunca verificar para quitar??????
			lea edi,[edi-2]
			lea esi,[esi+2]
align 16
nextline:
			mov ecx,[esp+16+12]			//row
			add esi,[esp+16+20]			//first ptr+modulo
			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx					//

			sub dword ptr [esp+16+16],1	//height -1

			jne x_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////
static void __declspec(naked) asm_mirror_rgb_isse_exp_dst
						(const BYTE* srcp,	//4
						BYTE* dstp,			//8
						int row,			//12
						int height,			//16
						int modulo,			//20
						int modulodst)		//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst// 
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
x_loop_0:
			mov edx,ecx
			sar ecx,5
			jz lower16

			and edx,31
//...........................................................................................
align 16
ecxis8_0:
			movq mm0,[esi]
			movq mm1,[esi+8]
			movq mm2,[esi+16]
			movq mm3,[esi+24]

			pshufw mm0,mm0,01001110b
			pshufw mm1,mm1,01001110b
			pshufw mm2,mm2,01001110b
			pshufw mm3,mm3,01001110b

			movq [edi-8],mm0
			movq [edi-16],mm1
			movq [edi-24],mm2
			movq [edi-32],mm3

			sub ecx,1

			lea esi,[esi+32]
			lea edi,[edi-32]

			jne ecxis8_0
//...........................................................................................
			test edx,edx
			jz nextline
//...........................................................................................
align 16
lower16:
			mov ecx,edx

//...........................................................................................
			sar ecx,2
			jz lower4

			and edx,3
align 16
reminder4:
  			mov eax,[esi]
			sub ecx,1
			mov [edi-4],eax

			lea edi,[edi-4]
			lea esi,[esi+4]

			jne reminder4
//...........................................................................................
			test edx,edx
			jz  nextline
//...........................................................................................
align 16
lower4:									///parece imposible????
			lea edi,[edi-2]
			lea esi,[esi+2]
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+12]			//row
			add esi,[esp+16+20]			//first ptr+modulo
			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx					//

			sub dword ptr [esp+16+16],1	//height -1
			jne x_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////
static void __declspec(naked) asm_mirror_rgb_isse_exp_dst_nta
						(const BYTE* srcp,	//4
						BYTE* dstp,		//8
						int row,		//12
						int height,		//16
						int modulo,		//20
						int modulodst)	//24
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row
			mov edi,[esp+16+8]			//ptr to dst// 
			mov ebp,[esp+16+20]
			add edi,ecx					//ptr to last byte same row
			add ebp,ecx
//...........................................................................................
align 16
x_loop_0:
			mov edx,ecx

			sar ecx,5
			//jz lower16					//no deberia pasar nunca

			and edx,31
//...........................................................................................
align 16
ecxis8_0:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movq mm0,[esi]
			movq mm1,[esi+8]
			movq mm2,[esi+16]
			movq mm3,[esi+24]

			pshufw mm0,mm0,01001110b
			pshufw mm1,mm1,01001110b
			pshufw mm2,mm2,01001110b
			pshufw mm3,mm3,01001110b

			movntq [edi-8],mm0
			movntq [edi-16],mm1
			movntq [edi-24],mm2
			movntq [edi-32],mm3

			lea esi,[esi+32]
			sub edi,32
			sub ecx,1
			jne ecxis8_0
//...........................................................................................
			test edx,edx
			jz nextline
//...........................................................................................
//align 16
//lower16:
			mov ecx,edx

//...........................................................................................
			sar ecx,2
			jz lower4

			and edx,3
align 16
reminder4:
  			mov eax,[esi]
			sub ecx,1
			mov [edi-4],eax
			lea esi,[esi+4]
			sub edi,4
			jne reminder4
//...........................................................................................
			test edx,edx
			jz  nextline
//...........................................................................................
align 16
lower4:				///parece imposible????
			sub edi,2
			lea esi,[esi+2]
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+12]			//row
			add esi,[esp+16+20]			//first ptr+modulo
			add edi,[esp+16+24]			//scnd ptr+modulo
			add edi,ecx
			add edi,ecx

			sub dword ptr [esp+16+16],1	//height -1
			jne x_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////
static void __declspec(naked) asm_mirror_rgb_isse_exp
						(BYTE* srcp,	//4
						int row,		//8
						int height,		//12
						int modulo)		//16
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+8]			//row
			mov ebx,[esp+16+12]			//height
			mov edi,esi					//copy of ptr
			mov ebp,[esp+16+16]			//modulo
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
x_loop_0:
			mov edx,ecx
			sar ecx,5
			jz lower32

			and edx,31
//...........................................................................................
align 16
ecxis8_0:
			movq mm2,[esi]
			movq mm5,[esi+8]

			pshufw mm2,mm2,01001110b
			pshufw mm5,mm5,01001110b

			movq mm3,[edi-8]
			movq mm7,[edi-16]

			pshufw mm3,mm3,01001110b
			pshufw mm7,mm7,01001110b

			movq [edi-8],mm2
			movq [edi-16],mm5

			sub ecx,1

			movq [esi],mm3
			movq [esi+8],mm7

			lea edi,[edi-16]
			lea esi,[esi+16]

			jne ecxis8_0
//...........................................................................................
			test edx,edx
			jz nextline
//...........................................................................................
align 16
lower32:
			mov ecx,edx

			cmp edx,8
			jl lower8
//...........................................................................................
align 16
reminder8:
  			mov eax,[esi]
			mov edx,[edi-4]

			sub ecx,8

			mov [edi-4],eax
			mov [esi],edx

			cmp ecx,8

			lea edi,[edi-4]
			lea esi,[esi+4]

			jge reminder8
//...........................................................................................
			test ecx,ecx
			jz  nextline
//...........................................................................................
align 16
lower8:
			lea edi,[edi-2]
			lea esi,[esi+2]
//...........................................................................................
align 16
nextline:
			mov ecx,[esp+16+8]			//row
										//lea edi,[edi+ebp]			//ptr + modulo
			lea esi,[esi+ebp]			//first ptr+modulo

			mov eax,ecx					//eax=row
			sar eax,1					//eax=row/2 just to add with ptrs

			sub ebx,1					//height -1

			lea esi,[esi+eax]			//first ptr+row/2
										//lea edi,[edi+eax]			//sec ptr+row/2. Finally row+(1 1/2row)??
			mov edi,esi
			lea edi,[edi+ecx]			//sec ptr+row


			jne x_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
////////////////////Copyright (c) Enrique Ardanaz (ARDA)/////////////////////////////////////
static void __declspec(naked) asm_mirror_rgb_mmx_exp
						(BYTE* srcp,	//4
						int row,		//8
						int height,		//12
						int modulo)		//16
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+8]			//row
			mov ebx,[esp+16+12]			//height
			mov edi,esi					//copy of ptr
			mov ebp,[esp+16+16]			//modulo
			add edi,ecx					//ptr to last byte same row
//...........................................................................................
align 16
x_loop_0:
			mov edx,ecx
			sar ecx,5
			jz lower32

			and edx,31
//...........................................................................................
align 16
ecxis8_0:
			movd mm0,[esi]
			movd mm2,[esi+4]
			movd mm4,[esi+8]
			movd mm5,[esi+12]

			psllq mm0,32
			psllq mm4,32

			movd mm1,[edi-8]
			movd mm3,[edi-4]
			movd mm6,[edi-16]
			movd mm7,[edi-12]

			psllq mm1,32
			psllq mm6,32

			por mm0,mm2
			por mm4,mm5
			por mm1,mm3
			por mm6,mm7

			movq [edi-8],mm0
			movq [edi-16],mm4

			sub ecx,1

			movq [esi],mm1
			movq [esi+8],mm6

			lea edi,[edi-16]
			lea esi,[esi+16]

			jne ecxis8_0
//...........................................................................................
			test edx,edx
			jz nextline
//...........................................................................................
align 16
lower32:

			mov ecx,edx

			cmp edx,8
			jl lower8
//...........................................................................................
align 16
reminder8:
  			mov eax,[esi]
			mov edx,[edi-4]

			sub ecx,8

			mov [edi-4],eax
			mov [esi],edx


			lea edi,[edi-4]
			lea esi,[esi+4]

			cmp ecx,8
			jge reminder8
//...........................................................................................
			test ecx,ecx
			jz  nextline
align 16
lower8:
			lea edi,[edi-2]
			lea esi,[esi+2]
align 16
nextline:
			mov ecx,[esp+16+8]			//row
										//lea edi,[edi+ebp]			//ptr + modulo
			lea esi,[esi+ebp]			//first ptr+modulo

			mov eax,ecx					//eax=row
			sar eax,1					//eax=row/2 just to add with ptrs

			sub ebx,1					//height -1

			lea esi,[esi+eax]			//first ptr+row/2
										//lea edi,[edi+eax]			//sec ptr+row/2. Finally row+(1 1/2row)??
			mov edi,esi
			lea edi,[edi+ecx]			//sec ptr+row


			jne x_loop_0
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
/////////////////////Copyright (c) Enrique Ardanaz (ARDA)////////////////////////////////////
static void __declspec(naked) asm_updown_plain_row
							(BYTE* srcp,		//4
							int row,			//8
							int height,			//12
							int modulo,			//16
							int sizefr)			//20
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ecx,[esp+16+8]		//row
			mov ebp,[esp+16+16]		//modulo
			mov eax,[esp+16+20]		//size frame
//...........................................................................................
			mov edi,esi				//copy of ptr
			add edi,eax
			sub edi,ebp				//ptr to last byte of src
			sub edi,ecx				//ptr to first byte of last row
//...........................................................................................
align 16
y_loop1:
			mov ebx,ecx
			sar ecx,2
//...........................................................................................
			and ebx,3
align 16
x_loop:
				mov eax,[esi]
				mov edx,[edi]

				mov [edi],eax
				mov [esi],edx

				add edi,4
				add esi,4

				dec ecx
				jnz x_loop
//...........................................................................................
				test ebx,ebx
				jz nextline
align 16
lower4:
				mov al,[esi]
				mov dl,[edi]

				mov [edi],al
				mov [esi],dl

				add edi,1
				add esi,1

				sub ebx,1
				jnz lower4
align 16
nextline:
			add esi,ebp
			mov ecx,[esp+16+8]		//row
			sub edi,ebp
			sub edi,ecx
			sub edi,ecx

			sub [esp+16+12],1		//height-1
			jnz y_loop1
//...........................................................................................
		mov ebx,[esp]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		add esp,16
		ret
		};
}
//////////////////Copyright (c) Enrique Ardanaz (ARDA)///////////////////////////////////////
static void __declspec(naked) mmx_updown_row
					(BYTE* srcp,		//4
					int row,			//8
					int height,			//12
					int modulo,			//16
					int sizefr)			//20
{
__asm {
		push ebp
		push edi
		push esi
		push ebx
//...........................................................................................
			mov esi,[esp+16+4]		//ptr to src
			mov ecx,[esp+16+8]		//row
			mov ebx,[esp+16+12]		//height
			mov ebp,[esp+16+16]		//modulo
			mov eax,[esp+16+20]		//size frame
//...........................................................................................
			mov edi,esi				//copy of ptr
			sar ebx,1				//(heigh/2)
			add edi,eax
			sub edi,ebp				//ptr to last byte of src
			sub edi,ecx				//ptr to first byte last row
//...........................................................................................
align 16
y_loop1:
			mov edx,ecx
			sar ecx,5
			jz	tailower32

			and edx,31
//...........................................................................................
align 16
x_loop:
				movq mm0,[esi]
				movq mm1,[esi+8]
				movq mm4,[esi+16]
				movq mm5,[esi+24]

				movq mm2,[edi]
				movq mm3,[edi+8]
				movq mm6,[edi+16]
				movq mm7,[edi+24]

				movq [edi],mm0
				movq [edi+8],mm1
				movq [edi+16],mm4
				movq [edi+24],mm5

				movq [esi],mm2
				movq [esi+8],mm3
				movq [esi+16],mm6
				movq [esi+24],mm7

				add edi,32
				add esi,32

				dec ecx
				jnz x_loop
//...........................................................................................
				test edx,edx
				jz endx
//...........................................................................................
align 16
tailower32:
				mov	ecx,edx				
				sar edx,2
				jz rest2_1

				and	ecx,3
//...........................................................................................
align 16
rest4:
				movd mm0,[esi]
				movd mm2,[edi]

				movd [edi],mm0
				movd [esi],mm2

				add edi,4
				add esi,4
				dec edx
				jnz rest4
//...........................................................................................
				test ecx,ecx
				jz	endx
align 16
rest2_1:
			mov eax,[esi]
			mov edx,[edi]

			sub ecx,1

			mov [esi],dl
			mov [edi],al

			lea esi,[esi+1]
			lea edi,[edi+1]

			jz endx
//...........................................................................................
			sar eax,8
			sar edx,8

			mov [edi],al
			mov [esi],dl

			sub ecx,1

			lea edi,[edi+1]
			lea esi,[esi+1]

			jz endx
//...........................................................................................
			sar eax,8
			sar edx,8

			mov [edi],al
			mov [esi],dl

			//sub ecx,1

			lea edi,[edi+1]
			lea esi,[esi+1]
//...........................................................................................
align 16
endx:
			add esi,ebp				//modulo
			mov ecx,[esp+16+8]		//row
			sub edi,ebp				//modulo
			sub edi,ecx
			sub edi,ecx
			dec ebx
			jnz y_loop1
//...........................................................................................
		mov ebx,[esp+0]
		mov esi,[esp+4]
		mov edi,[esp+8]
		mov ebp,[esp+12]
		emms
		add esp,16
		ret
		};
}
///////////////////Copyright (c) Enrique Ardanaz (ARDA)//////////////////////////////////////
static void __declspec(naked) sse2_updown_row
						(BYTE* srcp,		//4
						int row,			//8
						int height,			//12
						int modulo,			//16
						int sizefr)			//20
{
__asm {
			push ebp
			push edi
			push esi
			push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+8]			//row

			mov ebp,[esp+16+16]			//modulo
			mov eax,[esp+16+20]			//size frame
//...........................................................................................
			mov edi,esi					//copy of ptr

			add edi,eax
			sub edi,ebp					//ptr to last byte of src
			sub edi,ecx					//ptr to first byte of last row
//...........................................................................................
align 16
y_loop:
			mov edx,ecx
			cmp ecx,16
			jl lower64unali


			test esi,15
			jz x_loop1
//...........................................................................................
			test esi,1

			mov eax,[esi]
			mov edx,[edi]

			jz testalign4
//...........................................................................................
			mov [esi],dl
			mov [edi],al

			sub ecx,1

			lea esi,[esi+1]
			lea edi,[edi+1]

			sar eax,8
			sar edx,8
//...........................................................................................
align 16
testalign4:
			test esi,3
			jz testalign8
//...........................................................................................
			mov [esi],dx
			mov [edi],ax

			sub ecx,2

			lea esi,[esi+2]
			lea edi,[edi+2]

			mov eax,[esi]
			mov edx,[edi]

			jmp testalign4
//...........................................................................................
align 16
testalign8:
			test esi,15
			jz x_loop1
//...........................................................................................
			sub ecx,4

			mov [edi],eax
			mov [esi],edx

			lea edi,[edi+4]
			lea esi,[esi+4]

			mov edx,[edi]
			mov eax,[esi]

			jmp testalign8
//...........................................................................................
align 16
x_loop1:
//...........................................................................................
			test edi,15
			jnz pre_x_loop

			mov edx,ecx
			sar ecx,6
			jz lower64ali

			and edx,63
//...........................................................................................
align 16
x_loopal:
			//prefetcht0[esi+640]

			sub ecx,1

			movdqa xmm0,[esi]
			movdqa xmm1,[esi+16]
			movdqa xmm4,[esi+32]
			movdqa xmm5,[esi+48]

			//prefetcht0[edi-640]

			movdqa xmm2,[edi]
			movdqa xmm3,[edi+16]
			movdqa xmm6,[edi+32]
			movdqa xmm7,[edi+48]

			movdqa [edi],xmm0
			movdqa [edi+16],xmm1
			movdqa [edi+32],xmm4
			movdqa [edi+48],xmm5

			movdqa [esi],xmm2
			movdqa [esi+16],xmm3
			movdqa [esi+32],xmm6
			movdqa [esi+48],xmm7

			lea edi,[edi+64]
			lea esi,[esi+64]
			jne x_loopal
//...........................................................................................
			test edx,edx
			jz endx
//...........................................................................................
align 16
lower64ali:
			mov ecx,edx
			sar ecx,4
			jz	testend

			and edx,15
align 16
loop16dstal:
			movdqa xmm0,[esi]
			movdqa xmm2,[edi]

			sub ecx,1

			movdqa [edi],xmm0
			movdqa [esi],xmm2

			lea edi,[edi+16]
			lea esi,[esi+16]

			jnz loop16dstal

			jmp testend
//...........................................................................................
align 16
pre_x_loop:
			mov edx,ecx
			sar ecx,6
			jz lower64unali

			and edx,63
//...........................................................................................
align 16
x_loop:					//probable que nunca pase, estudiar. (tal vez si el pitch no es multiplo de 16)
			//prefetcht0[esi+640]

			sub ecx,1

			movdqa xmm0,[esi]
			movdqa xmm1,[esi+16]
			movdqa xmm4,[esi+32]
			movdqa xmm5,[esi+48]

			//prefetcht0[edi-640]

			movdqu xmm2,[edi]
			movdqu xmm3,[edi+16]
			movdqu xmm6,[edi+32]
			movdqu xmm7,[edi+48]

			movdqu [edi],xmm0
			movdqu [edi+16],xmm1
			movdqu [edi+32],xmm4
			movdqu [edi+48],xmm5

			movdqa [esi],xmm2
			movdqa [esi+16],xmm3
			movdqa [esi+32],xmm6
			movdqa [esi+48],xmm7

			lea edi,[edi+64]
			lea esi,[esi+64]

			jne x_loop
//...........................................................................................
			test edx,edx
			jz endx
//...........................................................................................
align 16
lower64unali:
			mov ecx,edx
			sar ecx,4
			jz	testend

			and edx,15
align 16
loop16dstunal:
			movdqa xmm0,[esi]
			movdqu xmm2,[edi]

			sub ecx,1

			movdqu [edi],xmm0
			movdqa [esi],xmm2

			lea edi,[edi+16]
			lea esi,[esi+16]

			jnz loop16dstunal
//...........................................................................................
align 16
testend:
			test edx,edx
			jz endx
//...........................................................................................
			mov ecx,edx
			sar edx,2
			jz	rest2_1

			and ecx,3
//...........................................................................................
align 16
rest4:		
			mov eax,[esi]
			mov ebx,[edi]

			sub edx,1

			mov [edi],eax
			mov [esi],ebx

			lea edi,[edi+4]
			lea esi,[esi+4]

			jne rest4
//...........................................................................................
			test ecx,ecx
			jz endx
//...........................................................................................
align 16
rest2_1:
			mov eax,[esi]
			mov edx,[edi]

			sub ecx,1

			mov [esi],dl
			mov [edi],al

			lea esi,[esi+1]
			lea edi,[edi+1]

			jz endx
//...........................................................................................
			sar eax,8
			sar edx,8

			mov [edi],al
			mov [esi],dl

			sub ecx,1

			lea edi,[edi+1]
			lea esi,[esi+1]

			jz endx
//...........................................................................................
			sar eax,8
			sar edx,8

			mov [edi],al
			mov [esi],dl

			//sub ecx,1

			lea edi,[edi+1]
			lea esi,[esi+1]
//...........................................................................................
align 16
endx:
			mov ecx,[esp+16+8]				//row
			sub edi,ebp						//modulosrc
			sub edi,ecx
			lea esi,[esi+ebp]
			sub edi,ecx

			sub dword ptr[esp+16+12],1		//height -1
			jne y_loop
//...........................................................................................
			mov ebx,[esp+0]
			mov esi,[esp+4]
			mov edi,[esp+8]
			mov ebp,[esp+12]
			add esp,16
			ret
		};
}
///////////////////Copyright (c) Enrique Ardanaz (ARDA)//////////////////////////////////////
//dar prioridad al alineamiento de destino ?/////////////////////////////////////////////////
static void __declspec(naked) sse2_updown_row_dst
						(const BYTE* srcp,	//4
						 BYTE* dstp,		//8
						int row,			//12
						int height,			//16
						int modulo,			//20
						int dstrow,			//24
						int dstmodulo,		//28
						int dstsizefr		//32
						)
{
__asm {
			push ebp
			push edi
			push esi
			push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row src

//...........................................................................................
			mov edi,[esp+16+8]			//ptr to dst

			add edi,[esp+16+32]			//+size frame
			sub edi,[esp+16+28]			//ptr to last byte of src
			sub edi,ecx					//[esp+16+24]			//ptr to first byte of last row
//...........................................................................................
align 16
y_loop:
			mov edx,ecx
			cmp ecx,16
			jl lower64unali
//...........................................................................................
			test esi,15
			jz x_loop1
//...........................................................................................
			test esi,1
			mov eax,[esi]
			jz testalign4
//...........................................................................................
			mov [edi],al
			sub ecx,1
			lea esi,[esi+1]
			lea edi,[edi+1]

			sar eax,8
//..........................warning maybe a bug......2]2]07..................................
align 16
testalign4:
			test esi,3
			jz testalign8
//...........................................................................................
			mov [edi],ax
			sub ecx,2
			lea esi,[esi+2]
			lea edi,[edi+2]

			mov eax,[esi]
			jmp testalign4
//...........................................................................................
align 16
testalign8:
			test esi,15
			jz x_loop1
//...........................................................................................
			sub ecx,4
			mov [edi],eax
			lea esi,[esi+4]
			lea edi,[edi+4]
			mov eax,[esi]

			jmp testalign8
//...........................................................................................
align 16
x_loop1:
//...........................................................................................
			test edi,15
			jnz pre_x_loop

			mov edx,ecx
			sar ecx,7
			jz lower64ali

			and edx,127
//...........................................................................................
align 16
x_loopal:
			sub ecx,1

			movdqa xmm0,[esi]
			movdqa xmm1,[esi+16]
			movdqa xmm4,[esi+32]
			movdqa xmm5,[esi+48]

			movdqa xmm2,[esi+64]
			movdqa xmm3,[esi+80]
			movdqa xmm6,[esi+96]
			movdqa xmm7,[esi+112]

			movdqa [edi],xmm0
			movdqa [edi+16],xmm1
			movdqa [edi+32],xmm4
			movdqa [edi+48],xmm5

			movdqa [edi+64],xmm2
			movdqa [edi+80],xmm3
			movdqa [edi+96],xmm6
			movdqa [edi+112],xmm7

			lea edi,[edi+128]
			lea esi,[esi+128]

			jne x_loopal
//...........................................................................................
			test edx,edx
			jz endx
//...........................................................................................
align 16
lower64ali:
			mov ecx,edx
			sar ecx,4
			jz	testend

			and edx,15
align 16
loop16dstal:
			movdqa xmm0,[esi]
			sub ecx,1
			movdqa [edi],xmm0

			lea esi,[esi+16]
			lea edi,[edi+16]

			jnz loop16dstal

			jmp testend
//...........................................................................................
align 16
pre_x_loop:
			mov edx,ecx
			sar ecx,7
			jz lower64unali

			and edx,127
//...........................................................................................
align 16
x_loop:			//probable que nunca pase, estudiar. (tal vez si el pitch no es multiplo de 16)
			//prefetcht0[esi+640]

			sub ecx,1

			movdqa xmm0,[esi]
			movdqa xmm1,[esi+16]
			movdqa xmm4,[esi+32]
			movdqa xmm5,[esi+48]

			movdqa xmm2,[esi+64]
			movdqa xmm3,[esi+80]
			movdqa xmm6,[esi+96]
			movdqa xmm7,[esi+112]

			movdqu [edi],xmm0
			movdqu [edi+16],xmm1
			movdqu [edi+32],xmm4
			movdqu [edi+48],xmm5

			movdqu [edi+64],xmm2
			movdqu [edi+80],xmm3
			movdqu [edi+96],xmm6
			movdqu [edi+112],xmm7

			lea esi,[esi+128]
			lea edi,[edi+128]

			jne x_loop
//...........................................................................................
			test edx,edx
			jz endx
//...........................................................................................
align 16
lower64unali:
			mov ecx,edx
			sar ecx,4
			jz	testend

			and edx,15
align 16
loop16dstunal:
			movdqa xmm0,[esi]

			sub ecx,1

			movdqu [edi],xmm0
			lea esi,[esi+16]
			lea edi,[edi+16]

			jnz loop16dstunal
//...........................................................................................
align 16
testend:	
			test edx,edx
			jz endx
//...........................................................................................
			mov ecx,edx
			sar edx,2
			jz	rest2_1

			and ecx,3
//...........................................................................................
align 16
rest4:		
			mov eax,[esi]
			sub edx,1
			mov [edi],eax

			lea edi,[edi+4]
			lea esi,[esi+4]

			jne rest4
//...........................................................................................
			test ecx,ecx
			jz endx
//...........................................................................................
align 16
rest2_1:
			mov eax,[esi]
			sub ecx,1
			mov [edi],al

			lea esi,[esi+1]
			lea edi,[edi+1]

			jz endx
//...........................................................................................
			sar eax,8
			sub ecx,1

			mov [edi],al

			lea esi,[esi+1]
			lea edi,[edi+1]

			jz endx
//...........................................................................................
			sar eax,8
			lea esi,[esi+1]

			mov [edi],al
			lea edi,[edi+1]
//...........................................................................................
align 16
endx:
			mov ecx,[esp+16+12]				//row src

			sub edi,[esp+16+28]				//dstmodulo,		//36
			sub edi,ecx						//row src should be equal to row dst
			sub edi,ecx						//row src

			add esi,[esp+16+20]				//srcmodulo

			sub dword ptr[esp+16+16],1		//height -1// compiler needs dword ptr
			jne y_loop
//...........................................................................................
			mov ebx,[esp+0]
			mov esi,[esp+4]
			mov edi,[esp+8]
			mov ebp,[esp+12]
			add esp,16
			ret
		};
}
///////////////////Copyright (c) Enrique Ardanaz (ARDA)//////////////////////////////////////
/*static void __declspec(naked) sse2_updown_row_dst_nta
						(const BYTE* srcp,	//4
						 BYTE* dstp,		//8
						int row,			//12
						int height,			//16
						int modulo,			//20
						int dstrow,			//24
						int dstmodulo,		//28
						int dstsizefr		//32
						)
{
__asm {
			push ebp
			push edi
			push esi
			push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row src
//...........................................................................................
			mov edi,[esp+16+8]			//ptr to dst
			mov ebp,[esp+16+20]			//modulo

			add edi,[esp+16+32]			//+size frame
			sub edi,[esp+16+28]			//ptr to last byte of src
			sub edi,ecx					//ptr to first byte of last row
			add ebp,ecx					//pitch
//...........................................................................................
align 16
y_loop:
			mov edx,ecx
//...........................................................................................
			test edi,15
			jz x_loop1
//...........................................................................................
			test edi,1
			mov eax,[esi]
			jz testalign4
//...........................................................................................
			mov [edi],al
			sub ecx,1
			lea esi,[esi+1]
			lea edi,[edi+1]

			sar eax,8
//..........................warning could be a bug......2]2]07...............................
align 16
testalign4:
			test edi,3
			jz testalign8
//...........................................................................................
			mov [edi],ax
			sub ecx,2
			lea esi,[esi+2]
			lea edi,[edi+2]

			mov eax,[esi]
			jmp testalign4
//...........................................................................................
align 16
testalign8:
			test edi,15
			jz x_loop1
//...........................................................................................
			sub ecx,4
			mov [edi],eax
			lea esi,[esi+4]
			lea edi,[edi+4]
			mov eax,[esi]

			jmp testalign8
//...........................................................................................
align 16
x_loop1:
//...........................................................................................
			//clflush [edi]
			test esi,15
			jnz pre_x_loop

			mov edx,ecx
			sar ecx,7

			and edx,127
//...........................................................................................
align 16
x_loopal:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movdqa xmm0,[esi]
			movdqa xmm1,[esi+16]
			movdqa xmm4,[esi+32]
			movdqa xmm5,[esi+48]

			movntdq [edi],xmm0
			movntdq [edi+16],xmm1
			movntdq [edi+32],xmm4
			movntdq [edi+48],xmm5

			movdqa xmm2,[esi+64]
			movdqa xmm3,[esi+80]
			movdqa xmm6,[esi+96]
			movdqa xmm7,[esi+112]

			movntdq [edi+64],xmm2
			movntdq [edi+80],xmm3
			movntdq [edi+96],xmm6
			movntdq [edi+112],xmm7

			lea esi,[esi+128]
			add edi,128

			sub ecx,1
			jne x_loopal

			sfence
//...........................................................................................
			test edx,edx
			jz endx
//...........................................................................................
			mov ecx,edx
			sar ecx,4
			jz	testend					//don't like controls in the loop

			and edx,15
align 16
loop16dstal:
			movdqa xmm0,[esi]
			movntdq [edi],xmm0

			lea esi,[esi+16]
			add edi,16

			sub ecx,1
			jnz loop16dstal
			sfence

			jmp testend
//...........................................................................................
align 16
pre_x_loop:
			mov edx,ecx
			sar ecx,7

			and edx,127
//...........................................................................................
align 16
x_loop:			//probable que nunca pase, estudiar. (tal vez si el pitch no es multiplo de 16)
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movdqu xmm0,[esi]
			movdqu xmm1,[esi+16]
			movdqu xmm4,[esi+32]
			movdqu xmm5,[esi+48]

			movdqu xmm2,[esi+64]
			movdqu xmm3,[esi+80]
			movdqu xmm6,[esi+96]
			movdqu xmm7,[esi+112]

			movntdq [edi],xmm0
			movntdq [edi+16],xmm1
			movntdq [edi+32],xmm4
			movntdq [edi+48],xmm5

			movntdq [edi+64],xmm2
			movntdq [edi+80],xmm3
			movntdq [edi+96],xmm6
			movntdq [edi+112],xmm7

			lea esi,[esi+128]
			add edi,128

			sub ecx,1
			jne x_loop

			sfence
//...........................................................................................
			test edx,edx
			jz endx
//...........................................................................................
			mov ecx,edx
			sar ecx,4
			jz	testend			//don't like controls in the loop

			and edx,15
align 16
loop16dstunal:
			movdqu xmm0,[esi]
			movntdq [edi],xmm0
			lea esi,[esi+16]
			add edi,16
			sub ecx,1
			jnz loop16dstunal
			sfence
//...........................................................................................
align 16
testend:	
			test edx,edx
			jz endx
//...........................................................................................
			mov ecx,edx
			sar edx,2
			jz	rest2_1			//don't like controls in the loop

			and ecx,3
//...........................................................................................
align 16
rest4:		
			mov eax,[esi]
			sub edx,1
			mov [edi],eax
			lea edi,[edi+4]
			lea esi,[esi+4]
			jne rest4
//...........................................................................................
			test ecx,ecx
			jz endx
//...........................................................................................
align 16
rest2_1:
			mov eax,[esi]
			sub ecx,1
			mov [edi],al

			lea esi,[esi+1]
			lea edi,[edi+1]

			jz endx
//...........................................................................................
			sar eax,8
			sub ecx,1

			mov [edi],al

			lea esi,[esi+1]
			lea edi,[edi+1]

			jz endx
//...........................................................................................
			sar eax,8
			lea esi,[esi+1]

			mov [edi],al
			lea edi,[edi+1]
//...........................................................................................
align 16
endx:
			mov ecx,[esp+16+12]				//row src

			sub edi,[esp+16+28]				//dstmodulo,		//36
			sub edi,ecx						//row src should be equal to row dst
			sub edi,ecx						//row src

			add esi,[esp+16+20]				//srcmodulo

			sub dword ptr[esp+16+16],1		//height -1// compiler needs dword ptr
			jne y_loop
//...........................................................................................
			mov ebx,[esp+0]
			mov esi,[esp+4]
			mov edi,[esp+8]
			mov ebp,[esp+12]
			add esp,16
			ret
		};
}*/
///////////////////Copyright (c) Enrique Ardanaz (ARDA)//////////////////////////////////////
static void __declspec(naked) mmx_updown_row_dst
						(const BYTE* srcp,	//4
						 BYTE* dstp,		//8
						int row,			//12
						int height,			//16
						int modulo,			//20
						int dstrow,			//24
						int dstmodulo,		//28
						int dstsizefr		//32
						)
{
__asm {
			push ebp
			push edi
			push esi
			push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row src

//...........................................................................................
			mov edi,[esp+16+8]			//ptr to dst

			add edi,[esp+16+32]			//+size frame
			sub edi,[esp+16+28]			//ptr to last byte of src
			sub edi,ecx					//ptr to first byte of last row
//...........................................................................................
align 16
y_loop:
			mov edx,ecx

			mov edx,ecx
			sar ecx,6
			jz lower32ali

			and edx,63
//...........................................................................................
align 16
x_loopal:
			sub ecx,1

			movq mm0,[esi]
			movq mm1,[esi+8]
			movq mm4,[esi+16]
			movq mm5,[esi+24]

			movq mm2,[esi+32]
			movq mm3,[esi+40]
			movq mm6,[esi+48]
			movq mm7,[esi+56]

			movq [edi],mm0
			movq [edi+8],mm1
			movq [edi+16],mm4
			movq [edi+24],mm5

			movq [edi+32],mm2
			movq [edi+40],mm3
			movq [edi+48],mm6
			movq [edi+56],mm7

			lea esi,[esi+64]
			lea edi,[edi+64]

			jne x_loopal
//...........................................................................................
			test edx,edx
			jz endx
//...........................................................................................
align 16
lower32ali:
			mov ecx,edx
			sar ecx,3
			jz	testend

			and edx,7
align 16
loop16dstal:
			movq mm0,[esi]

			sub ecx,1

			movq [edi],mm0

			lea esi,[esi+8]
			lea edi,[edi+8]

			jnz loop16dstal
//...........................................................................................
align 16
testend:
			test edx,edx
			jz endx
//...........................................................................................
			mov ecx,edx
			sar edx,2
			jz	rest2_1

			and ecx,3
//...........................................................................................
align 16
rest4:		
			mov eax,[esi]

			sub edx,1

			mov [edi],eax

			lea edi,[edi+4]
			lea esi,[esi+4]

			jne rest4
//...........................................................................................
			test ecx,ecx
			jz endx
//...........................................................................................
align 16
rest2_1:
			mov eax,[esi]
			sub ecx,1
			mov [edi],al

			lea esi,[esi+1]
			lea edi,[edi+1]

			jz endx
//...........................................................................................
			sar eax,8

			mov [edi],al
			sub ecx,1

			lea edi,[edi+1]
			lea esi,[esi+1]

			jz endx
//...........................................................................................
			sar eax,8

			mov [edi],al

			lea esi,[esi+1]
			lea edi,[edi+1]
//...........................................................................................
align 16
endx:
			mov ecx,[esp+16+12]				//row src

			sub edi,[esp+16+28]				//dstmodulo
			sub edi,ecx						//row src should be equal to row dst
			sub edi,ecx						//row src

			add esi,[esp+16+20]				//srcmodulo

			sub dword ptr[esp+16+16],1		//height -1// compiler needs dword ptr
			jne y_loop
//...........................................................................................
			mov ebx,[esp+0]
			mov esi,[esp+4]
			mov edi,[esp+8]
			mov ebp,[esp+12]
			emms
			add esp,16
			ret
		};
}
///////////////////Copyright (c) Enrique Ardanaz (ARDA)//////////////////////////////////////
/*static void __declspec(naked) isse_updown_row_dst_nta
						(const BYTE* srcp,	//4
						 BYTE* dstp,		//8
						int row,			//12
						int height,			//16
						int modulo,			//20
						int dstrow,			//24
						int dstmodulo,		//28
						int dstsizefr		//32
						)			//20
{
__asm {
			push ebp
			push edi
			push esi
			push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row src

//...........................................................................................
			mov edi,[esp+16+8]			//ptr to dst
			mov ebp,[esp+16+20]			//row src

			add edi,[esp+16+32]			//+size frame
			sub edi,[esp+16+28]			//ptr to last byte of src
			add ebp,ecx
			sub edi,ecx					//[esp+16+24]			//ptr to first byte of last row
//...........................................................................................
align 16
y_loop:
			mov edx,ecx

			mov edx,ecx
			sar ecx,6//5

			and edx,63//31
//...........................................................................................
align 16
x_loopal:
			prefetchnta	[esi+ebp+128]
			prefetchnta	[esi+ebp+256]
			prefetchnta	[esi+ebp+512]
			prefetchnta	[esi+ebp*2]

			movq mm0,[esi]
			movq mm1,[esi+8]
			movq mm4,[esi+16]
			movq mm5,[esi+24]

			movq mm2,[esi+32]
			movq mm3,[esi+40]
			movq mm6,[esi+48]
			movq mm7,[esi+56]

			movntq [edi],mm0
			movntq [edi+8],mm1
			movntq [edi+16],mm4
			movntq [edi+24],mm5

			movntq [edi+32],mm2
			movntq [edi+40],mm3
			movntq [edi+48],mm6
			movntq [edi+56],mm7

			lea esi,[esi+64]
			add edi,64

			sub ecx,1
			jne x_loopal

			sfence
//...........................................................................................
			test edx,edx				// tal vez innecesario
			jz endx
//...........................................................................................
			mov ecx,edx
			sar ecx,3
			jz	testend

			and edx,7
align 16
loop16dstal:
			movq mm0,[esi]
			movntq [edi],mm0

			lea esi,[esi+8]
			add edi,8

			sub ecx,1
			jnz loop16dstal

			sfence
//...........................................................................................
align 16
testend:
			test edx,edx
			jz endx
//...........................................................................................
			mov ecx,edx
			sar edx,2
			jz	rest2_1

			and ecx,3
//...........................................................................................
align 16
rest4:		
			mov eax,[esi]
			sub edx,1
			mov [edi],eax

			lea esi,[esi+4]
			lea edi,[edi+4]

			jne rest4
//...........................................................................................
			test ecx,ecx
			jz endx
//...........................................................................................
align 16
rest2_1:
			mov eax,[esi]
			sub ecx,1
			mov [edi],al

			lea esi,[esi+1]
			lea edi,[edi+1]

			jz endx
//...........................................................................................
			sar eax,8
			sub ecx,1

			mov [edi],al
			lea esi,[esi+1]
			lea edi,[edi+1]

			jz endx
//...........................................................................................
			sar eax,8
			lea esi,[esi+1]

			mov [edi],al
			lea edi,[edi+1]
//...........................................................................................
align 16
endx:
			mov ecx,[esp+16+12]				//row src

			sub edi,[esp+16+28]				//dstmodulo,		//36
			sub edi,ecx						//row src should be equal to row dst
			sub edi,ecx						//row src

			add esi,[esp+16+20]				//srcmodulo

			sub dword ptr[esp+16+16],1		//height -1// compiler needs dword ptr
			jne y_loop
//...........................................................................................
			mov ebx,[esp+0]
			mov esi,[esp+4]
			mov edi,[esp+8]
			mov ebp,[esp+12]
			emms
			add esp,16
			ret
		};
}*/
///////////////////Copyright (c) Enrique Ardanaz (ARDA)//////////////////////////////////////
static void __declspec(naked) asm_updown_row_dst
						(const BYTE* srcp,	//4
						 BYTE* dstp,		//8
						int row,			//12
						int height,			//16
						int modulo,			//20
						int dstrow,			//24//creo no se necesita
						int dstmodulo,		//28
						int dstsizefr		//32
						)
{
__asm {
			push ebp
			push edi
			push esi
			push ebx
//...........................................................................................
			mov esi,[esp+16+4]			//ptr to src
			mov ecx,[esp+16+12]			//row src
//...........................................................................................
			mov edi,[esp+16+8]			//ptr to dst

			add edi,[esp+16+32]			//+size frame
			sub edi,[esp+16+28]			//ptr to last byte of src
			sub edi,ecx					//[esp+16+24]			//ptr to first byte of last row
//...........................................................................................
align 16
y_loop:
			mov edx,ecx
			sar ecx,3
			jz lower8

			and edx,7
//...........................................................................................
align 16
x_loopal:
			//prefetcht0[esi+640]

			sub ecx,1

			mov eax,[esi]
			mov ebx,[esi+4]

			mov [edi],eax
			mov [edi+4],ebx

			lea edi,[edi+8]
			lea esi,[esi+8]
			jne x_loopal
//...........................................................................................
			test edx,edx
			jz endx
//...........................................................................................
align 16
lower8:
			mov ecx,edx
			sar edx,2
			jz	rest2_1
//...........................................................................................
			and ecx,3
//...........................................................................................
align 16
rest4:		
			mov eax,[esi]
			sub edx,1
			mov [edi],eax

			lea esi,[esi+4]
			lea edi,[edi+4]

			jne rest4
//...........................................................................................
			test ecx,ecx
			jz endx
//...........................................................................................
align 16
rest2_1:
			mov eax,[esi]
			sub ecx,1
			mov [edi],al

			lea esi,[esi+1]
			lea edi,[edi+1]

			jz endx
//...........................................................................................
			sar eax,8
			sub ecx,1

			mov [edi],al

			lea edi,[edi+1]
			lea esi,[esi+1]

			jz endx
//...........................................................................................
			sar eax,8

			mov [edi],al
			lea edi,[edi+1]
			lea esi,[esi+1]
//...........................................................................................
align 16
endx:
			mov ecx,[esp+16+12]				//row src
//...........................................................................................
			sub edi,[esp+16+28]				//dstmodulo
			sub edi,ecx						//row src should be equal to row dst
			sub edi,ecx						//row src

			add esi,[esp+16+20]				//srcmodulo

			sub dword ptr[esp+16+16],1		//height -1// compiler needs dword ptr
			jne y_loop
//...........................................................................................
			mov ebx,[esp+0]
			mov esi,[esp+4]
			mov edi,[esp+8]
			mov ebp,[esp+12]
			add esp,16
			ret
		};
}
/////////////////////////////////////////////////////////////////////////////////////////////
PVideoFrame __stdcall Rotate180::GetFrame(int n,IScriptEnvironment* env)
{
		PVideoFrame src = child->GetFrame(n, env);

	if (!src->IsWritable())DoInPlace=false;

	if (DoInPlace)
	{
		env->MakeWritable(&src);
//...........................................................................................
		if (vi.IsYUY2())
		{
			int src_pitch = src->GetPitch();
			int height = src->GetHeight();
			int row_size = src->GetRowSize();
			int framesize = (src_pitch*(height-1))+row_size;
			int modulo = src_pitch-row_size;

			if(use_SSE2)
			{
				asm_sse2_yuy2_180
				( src->GetWritePtr(), row_size, height, modulo, framesize );
			}
			else if(use_ISSE)
			{
				asm_isse_yuy2_180_3
				( src->GetWritePtr(), row_size, height, modulo, framesize );
			}
			else
			{
				asm_MMX2_yuy2_180_3
				( src->GetWritePtr(), row_size, height, modulo, framesize );
			}
		}
//...........................................................................................
// TODO: ISSE and SSE2 for YV12
		else if (vi.IsYV12())			//change to planar when 2.6 and test Y8//if (vi.IsPlanar())
		{
		const int src_pitch = src->GetPitch(PLANAR_Y);
		const int height	= src->GetHeight(PLANAR_Y);
		const int row_size	= src->GetRowSize(PLANAR_Y);
		const int framesize = (src_pitch*(height-1))+row_size;
		const int modulo	= src_pitch-row_size;

		const int src_pitchU = src->GetPitch(PLANAR_U);
		const int heightU	 = src->GetHeight(PLANAR_U);
		const int row_sizeU  = src->GetRowSize(PLANAR_U);
		const int framesizeU = (src_pitchU*(heightU-1))+row_sizeU;
		const int moduloU	 = src_pitchU-row_sizeU;

			if(use_SSE2)
			{
			asm_rotate180_sse2_row_2
			(src->GetWritePtr(PLANAR_Y), row_size, height, modulo, framesize);
		//if (vi.!IsY8())

			asm_rotate180_sse2_row_2
			(src->GetWritePtr(PLANAR_U), row_sizeU, heightU, moduloU, framesizeU);

			asm_rotate180_sse2_row_2
			(src->GetWritePtr(PLANAR_V), row_sizeU, heightU, moduloU, framesizeU);
			}
			else if(use_ISSE)
			{
			asm_rotate180_isse_row_2
			(src->GetWritePtr(PLANAR_Y), row_size, height, modulo, framesize);

		//if (vi.!IsY8())

			asm_rotate180_isse_row_2
			(src->GetWritePtr(PLANAR_U), row_sizeU, heightU, moduloU, framesizeU);

			asm_rotate180_isse_row_2
			(src->GetWritePtr(PLANAR_V), row_sizeU, heightU, moduloU, framesizeU);
			}
			else
			{
			asm_rotate180_plain_row_1
			(src->GetWritePtr(PLANAR_Y), row_size, height, modulo, framesize);

		//if (vi.!IsY8())
			asm_rotate180_plain_row_1
			(src->GetWritePtr(PLANAR_U), row_sizeU, heightU, moduloU, framesizeU);

			asm_rotate180_plain_row_1
			(src->GetWritePtr(PLANAR_V), row_sizeU, heightU, moduloU, framesizeU);
			}
		}
//...........................................................................................
		else if (vi.IsRGB32())
		{
		const int src_pitch	= src->GetPitch();
		const int height	= src->GetHeight();
		const int row_size	= src->GetRowSize();
		const int framesize	= (src_pitch*(height-1))+row_size;
		const int modulo	= src_pitch-row_size;

			if(use_SSE2)
			{
			SSE2_rotate180_RGB32_al
			( src->GetWritePtr(), row_size, height, modulo, framesize );
			}
			else if(use_ISSE)
			{
			ISSE_rotate180_RGB32_1
			( src->GetWritePtr(), row_size, height, modulo, framesize );
			}
			else if(use_MMX)
			{
			MMX_rotate180_RGB32_1
			( src->GetWritePtr(), row_size, height, modulo, framesize );
			}
			else
			{
			asm_rotate180_RGB32//not needed ??
			( src->GetWritePtr(), row_size, height, modulo, framesize );
			}
		}
		else
		{
			asm_rotate180_RGB24_2
			( src->GetWritePtr(), src->GetRowSize(), src->GetHeight(),
			src->GetPitch()-src->GetRowSize(), (src->GetPitch()*(src->GetHeight()-1))+src->GetRowSize());

		}
//...........................................................................................
return src;
	}
/////////////////////////////////////////////////////////////////////////////////////////////
	else
	{
			PVideoFrame dst = env->NewVideoFrame(vi,32);//??1632
			env->MakeWritable(&dst);

			chsz2=chsz2-(chsz2>>2);		//75% ???????


			bool DoNta=( (dst->GetPitch()*dst->GetHeight()) >(chsz2) )!=0;

			//use_SSE2=false;
//...........................................................................................
		if (vi.IsYUY2())
		{
			const int src_pitch	= src->GetPitch();
			const int height	= src->GetHeight();
			const int row_size	= src->GetRowSize();
			const int framesize	= (src_pitch*(height-1))+row_size;
			const int modulo	= src_pitch-row_size;

			const int dst_pitch		= dst->GetPitch();
			const int dstrow_size	= dst->GetRowSize();
			const int dstframesize	= (dst_pitch*(height-1))+dstrow_size;
			const int dstmodulo		= dst_pitch-dstrow_size;

			if(use_SSE2)
			{
				if (DoNta)
				{
				asm_sse2_yuy2_180_dst_nta
				( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
				dstmodulo, dstframesize );
				}
				else
				{
				asm_sse2_yuy2_180_dst
				( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
				dstmodulo, dstframesize );
				}
			}
			else if(use_ISSE)
			{
				if (DoNta)
				{
				asm_isse_yuy2_180_3_dst_nta
				( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
				dstmodulo, dstframesize );
				}
				else
				{
				asm_isse_yuy2_180_3_dst
				( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
				dstmodulo, dstframesize );
				}
			}
			else
			{
				asm_MMX2_yuy2_180_3_dst
				( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
				dstmodulo, dstframesize );
			}
		}
//...........................................................................................
		else if (vi.IsYV12())			//change to planar when 2.6 and test Y8//if (vi.IsPlanar())
		{
			const int src_pitch = src->GetPitch(PLANAR_Y);
			const int height	= src->GetHeight(PLANAR_Y);
			const int row_size	= src->GetRowSize(PLANAR_Y);
			const int framesize = (src_pitch*(height-1))+row_size;
			const int modulo	= src_pitch-row_size;

			const int dst_pitch		= dst->GetPitch(PLANAR_Y);
			const int dstrow_size	= dst->GetRowSize(PLANAR_Y);
			const int dstframesize	= (dst_pitch*(height-1))+dstrow_size;
			const int dstmodulo		= dst_pitch-dstrow_size;

		//if (vi.!IsY8())
			const int src_pitchU = src->GetPitch(PLANAR_U);
			const int heightU	 = src->GetHeight(PLANAR_U);
			const int row_sizeU  = src->GetRowSize(PLANAR_U);
			const int framesizeU = (src_pitchU*(heightU-1))+row_sizeU;
			const int moduloU	 = src_pitchU-row_sizeU;

			const int dst_pitchU	= dst->GetPitch(PLANAR_U);
			const int dstrow_sizeU	= dst->GetRowSize(PLANAR_U);
			const int dstframesizeU = (dst_pitchU*(heightU-1))+dstrow_sizeU;
			const int dstmoduloU	= dst_pitchU-dstrow_sizeU;

			if(use_SSE2)
			{
				if (DoNta)
				{
					asm_rotate180_sse2_row_2_dst_nta
					( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
					dstmodulo, dstframesize );
				}
				else
				{
					asm_rotate180_sse2_row_2_dst
					( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
					dstmodulo, dstframesize );
				}
//...........................................................................................
				if ( ((src_pitchU*heightU)) >(chsz2) )
				{
					asm_rotate180_sse2_row_2_dst_nta
					( src->GetReadPtr(PLANAR_U), row_sizeU, heightU, moduloU, framesizeU,
					dst->GetWritePtr(PLANAR_U),	dstmoduloU, dstframesizeU );

					asm_rotate180_sse2_row_2_dst_nta
					( src->GetReadPtr(PLANAR_V), row_sizeU, heightU, moduloU, framesizeU,
					dst->GetWritePtr(PLANAR_V),	dstmoduloU, dstframesizeU );
				}
				else
				{
					asm_rotate180_sse2_row_2_dst
					( src->GetReadPtr(PLANAR_U), row_sizeU, heightU, moduloU, framesizeU,
					dst->GetWritePtr(PLANAR_U),	dstmoduloU, dstframesizeU );

					asm_rotate180_sse2_row_2_dst
					( src->GetReadPtr(PLANAR_V), row_sizeU, heightU, moduloU, framesizeU,
					dst->GetWritePtr(PLANAR_V),	dstmoduloU, dstframesizeU );
				}
			}
//...........................................................................................
			else if(use_ISSE)
			{
				if (DoNta)
				{
					asm_rotate180_isse_row_2_dst_nta
					( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
					dstmodulo, dstframesize );
				}
				else
				{
					asm_rotate180_isse_row_2_dst
					( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
					dstmodulo, dstframesize );
				}
//...........................................................................................
				if ( ((src_pitchU*heightU) + (dst_pitchU*heightU)) >(chsz2) )
				{
					asm_rotate180_isse_row_2_dst_nta
					( src->GetReadPtr(PLANAR_U), row_sizeU, heightU, moduloU, framesizeU,
					dst->GetWritePtr(PLANAR_U),	dstmoduloU, dstframesizeU );

					asm_rotate180_isse_row_2_dst_nta
					( src->GetReadPtr(PLANAR_V), row_sizeU, heightU, moduloU, framesizeU,
					dst->GetWritePtr(PLANAR_V),	dstmoduloU, dstframesizeU );
				}
				else
				{
					asm_rotate180_isse_row_2_dst
					( src->GetReadPtr(PLANAR_U), row_sizeU, heightU, moduloU, framesizeU,
					dst->GetWritePtr(PLANAR_U),	dstmoduloU, dstframesizeU );

					asm_rotate180_isse_row_2_dst
					( src->GetReadPtr(PLANAR_V), row_sizeU, heightU, moduloU, framesizeU,
					dst->GetWritePtr(PLANAR_V),	dstmoduloU, dstframesizeU );
				}
			}
			else
			{
			asm_rotate180_plain_row_1_dst
			( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
			dstmodulo, dstframesize );

			asm_rotate180_plain_row_1_dst
			( src->GetReadPtr(PLANAR_U), row_sizeU, heightU, moduloU, framesizeU,
			dst->GetWritePtr(PLANAR_U),	dstmoduloU, dstframesizeU );

			asm_rotate180_plain_row_1_dst
			( src->GetReadPtr(PLANAR_V), row_sizeU, heightU, moduloU, framesizeU,
			dst->GetWritePtr(PLANAR_V),	dstmoduloU, dstframesizeU );
			}
		}
//...........................................................................................
		else if (vi.IsRGB32())
		{
			const int src_pitch	= src->GetPitch();
			const int height	= src->GetHeight();
			const int row_size	= src->GetRowSize();
			const int framesize	= (src_pitch*(height-1))+row_size;
			const int modulo	= src_pitch-row_size;

			const int dst_pitch	= dst->GetPitch();
			const int dstrow_size	= dst->GetRowSize();
			const int dstframesize	= (dst_pitch*(height-1))+dstrow_size;
			const int dstmodulo	= dst_pitch-dstrow_size;

			if(use_SSE2)
			{					
				if (DoNta)
				{
				SSE2_rotate180_RGB32_al_dst_nta
				( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
				dstmodulo, dstframesize );
				}
				else
				{
				SSE2_rotate180_RGB32_al_dst
				( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
				dstmodulo, dstframesize );
				}
			}
			else if(use_ISSE)
			{
				if (DoNta)
				{
				ISSE_rotate180_RGB32_1_dst_nta
				( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
				dstmodulo, dstframesize );
				}
				else
				{
				ISSE_rotate180_RGB32_1_dst
				( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
				dstmodulo, dstframesize );
				}
			}
			else if(use_MMX)
			{
			MMX_rotate180_RGB32_1_dst
				( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
				dstmodulo, dstframesize );
			}
			else
			{
			asm_rotate180_RGB32_dst//not needed 
				( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
				dstmodulo, dstframesize );
			}
		}
//...........................................................................................
		else
		{
			const int src_pitch	= src->GetPitch();
			const int height	= src->GetHeight();
			const int row_size	= src->GetRowSize();
			const int framesize	= (src_pitch*(height-1))+row_size;
			const int modulo	= src_pitch-row_size;

			const int dst_pitch	= dst->GetPitch();
			const int dstrow_size	= dst->GetRowSize();
			const int dstframesize	= (dst_pitch*(height-1))+dstrow_size;
			const int dstmodulo	= dst_pitch-dstrow_size;

			asm_rotate180_RGB24_2_dst
			( src->GetReadPtr(), row_size, height, modulo, framesize, dst->GetWritePtr(),
			dstmodulo, dstframesize );
		}
			return dst;
//...........................................................................................
	}
};
/////////////////////////////////////////////////////////////////////////////////////////////
PVideoFrame __stdcall FVertical::GetFrame(int n,IScriptEnvironment* env)
{
		PVideoFrame src = child->GetFrame(n, env);

		//DoInPlace=( (src->GetPitch())*(src->GetHeight()) < ((chsz2<<2)+(chsz2<<1)) )!=0;
		// need to be tuned in several machines , is it worth while?
		if (!src->IsWritable())DoInPlace=false;

	if (DoInPlace)
	{
		env->MakeWritable(&src);
//...........................................................................................
		if (vi.IsYV12())	//change to planar when 2.6 and test Y8//if (vi.IsPlanar())
		{
			const int src_pitch = src->GetPitch(PLANAR_Y);
			const int height	= src->GetHeight(PLANAR_Y);
			const int row_size	= src->GetRowSize(PLANAR_Y_ALIGNED);
			const int framesize = src_pitch*height;
			const int modulo	= src_pitch-row_size;

			const int src_pitchU = src->GetPitch(PLANAR_U);
			const int heightU	 = src->GetHeight(PLANAR_U);
			const int row_sizeU	 = src->GetRowSize(PLANAR_U_ALIGNED);
			const int framesizeU = src_pitchU*heightU;
			const int moduloU	 = src_pitchU-row_sizeU;

//...........................................................................................
			if(use_SSE2)
			{
				sse2_updown_row
				(src->GetWritePtr(PLANAR_Y), row_size,height>>1, modulo, framesize);
		//if (!vi.IsY8())

				sse2_updown_row
				(src->GetWritePtr(PLANAR_U), row_sizeU,heightU>>1, moduloU, framesizeU);

				sse2_updown_row
				(src->GetWritePtr(PLANAR_V), row_sizeU,heightU>>1, moduloU, framesizeU);
			}
//...........................................................................................
			else if (use_MMX)
			{
				mmx_updown_row
				(src->GetWritePtr(PLANAR_Y), row_size,height, modulo, framesize);
		//if (!vi.IsY8())

				mmx_updown_row
				(src->GetWritePtr(PLANAR_U), row_sizeU,heightU, moduloU, framesizeU);

				mmx_updown_row
				(src->GetWritePtr(PLANAR_V), row_sizeU,heightU, moduloU, framesizeU);
			}
			else
//...........................................................................................
			{
				//plain is still here cause was the beginning of this idea
				asm_updown_plain_row
				(src->GetWritePtr(PLANAR_Y), row_size,height>>1, modulo, framesize);
		//if (!vi.IsY8())

				asm_updown_plain_row
				(src->GetWritePtr(PLANAR_U), row_sizeU,heightU>>1, moduloU, framesizeU);

				asm_updown_plain_row
				(src->GetWritePtr(PLANAR_V), row_sizeU,heightU>>1, moduloU, framesizeU);
			}
		}
//...........................................................................................
	else{		//this part should work for any RGB (32 & 24) and yuy2
			const int src_pitch = src->GetPitch();
			const int height	= src->GetHeight();
			const int row_size	= src->GetRowSize();
			const int framesize = src_pitch*height;
			const int modulo	= src_pitch-row_size;
//...........................................................................................
			if(use_SSE2)
			{
				sse2_updown_row
				(src->GetWritePtr(), row_size, height>>1, modulo, framesize);
			}
			else if (use_MMX)
			{
				mmx_updown_row
				(src->GetWritePtr(), row_size, height, modulo, framesize);
			}
			else
			{	//plain is still here cause was the beginning of this idea
				asm_updown_plain_row
				(src->GetWritePtr(), row_size, height, modulo, framesize);
			}
	}
	return src;
	}
//...........................................................................................
	else
	{
			PVideoFrame dst = env->NewVideoFrame(vi,32);//??16
			env->MakeWritable(&dst);

		chsz2=(chsz2-(chsz2>>2));//75% still need to be tuned

			//use_SSE2=false;
			///use_MMX=false;
		bool DoNta=( (dst->GetPitch()*dst->GetHeight()) >(chsz2) )!=0;

		if (vi.IsYV12())	//change to planar when 2.6 and test Y8//if (vi.IsPlanar())
		{
			const int src_pitch = src->GetPitch(PLANAR_Y);
			const int height	= src->GetHeight(PLANAR_Y);
			const int row_size	= src->GetRowSize(PLANAR_Y);
			const int modulo	= src_pitch-row_size;

			const int src_pitchU = src->GetPitch(PLANAR_U);
			const int heightU	 = src->GetHeight(PLANAR_U);
			const int row_sizeU	 = src->GetRowSize(PLANAR_U);
			const int moduloU	 = src_pitchU-row_sizeU;
//...........................................................................................

			const int dst_pitch		= dst->GetPitch(PLANAR_Y);
			const int dstheight		= dst->GetHeight(PLANAR_Y);
			const int dstrow_size	= dst->GetRowSize(PLANAR_Y);
			const int dstframesize	= dst_pitch*dstheight;
			const int dstmodulo		= dst_pitch-dstrow_size;

			const int dst_pitchU	= dst->GetPitch(PLANAR_U);
			const int dstheightU	= dst->GetHeight(PLANAR_U);
			const int dstrow_sizeU	= dst->GetRowSize(PLANAR_U);
			const int dstframesizeU = dst_pitchU*dstheightU;
			const int dstmoduloU	= dst_pitchU-dstrow_sizeU;
//...........................................................................................
			if(use_SSE2)
			{
				if (DoNta)
				{
				  env->BitBlt(dst->GetWritePtr(), dst_pitch, src->GetReadPtr() + (height-1) * src_pitch,
					  -src_pitch, row_size, height);
				}
				else
				{
				sse2_updown_row_dst
				(src->GetReadPtr(PLANAR_Y), dst->GetWritePtr(PLANAR_Y), row_size, height, 
				modulo, dstrow_size, dstmodulo, dstframesize );
				}
		//if (!vi.IsY8())
				if ((src_pitchU*heightU) >(chsz2))
				{
				  env->BitBlt(dst->GetWritePtr(PLANAR_U), dst_pitchU, src->GetReadPtr(PLANAR_U) + (heightU-1) * src_pitchU,
					  -src_pitchU, row_sizeU, heightU);

				  env->BitBlt(dst->GetWritePtr(PLANAR_V), dst_pitchU, src->GetReadPtr(PLANAR_V) + (heightU-1) * src_pitchU,
					  -src_pitchU, row_sizeU, heightU);
				}
				else
				{
				sse2_updown_row_dst
				(src->GetReadPtr(PLANAR_U), dst->GetWritePtr(PLANAR_U), row_sizeU, heightU, 
				moduloU, dstrow_sizeU, dstmoduloU, dstframesizeU);

				sse2_updown_row_dst
				(src->GetReadPtr(PLANAR_V), dst->GetWritePtr(PLANAR_V), row_sizeU, heightU, 
				moduloU, dstrow_sizeU, dstmoduloU, dstframesizeU );
				}
			}
//...........................................................................................
			else if (use_ISSE)
			{
				if (DoNta)
				{
				  env->BitBlt(dst->GetWritePtr(), dst_pitch, src->GetReadPtr() + (height-1) * src_pitch,
					  -src_pitch, row_size, height);
				}
				else
				{
				mmx_updown_row_dst
				(src->GetReadPtr(PLANAR_Y), dst->GetWritePtr(PLANAR_Y), row_size, height, 
				modulo, dstrow_size, dstmodulo, dstframesize );
				}
		//if (!vi.IsY8())
				if ((src_pitchU*heightU) >(chsz2))
				{
				  env->BitBlt(dst->GetWritePtr(PLANAR_U), dst_pitchU, src->GetReadPtr(PLANAR_U) + (heightU-1) * src_pitchU,
					  -src_pitchU, row_sizeU, heightU);

				  env->BitBlt(dst->GetWritePtr(PLANAR_V), dst_pitchU, src->GetReadPtr(PLANAR_V) + (heightU-1) * src_pitchU,
					  -src_pitchU, row_sizeU, heightU);

				}
				else
				{
				mmx_updown_row_dst
				(src->GetReadPtr(PLANAR_U), dst->GetWritePtr(PLANAR_U), row_sizeU, heightU, 
				moduloU, dstrow_sizeU, dstmoduloU, dstframesizeU);

				mmx_updown_row_dst
				(src->GetReadPtr(PLANAR_V), dst->GetWritePtr(PLANAR_V), row_sizeU, heightU, 
				moduloU, dstrow_sizeU, dstmoduloU, dstframesizeU );
				}
			}

			else if (use_MMX)
			{
			mmx_updown_row_dst
			(src->GetReadPtr(PLANAR_Y), dst->GetWritePtr(PLANAR_Y), row_size, height, 
			modulo, dstrow_size, dstmodulo, dstframesize );

		//if (!vi.IsY8())

			mmx_updown_row_dst
			(src->GetReadPtr(PLANAR_U), dst->GetWritePtr(PLANAR_U), row_sizeU, heightU, 
			moduloU, dstrow_sizeU, dstmoduloU, dstframesizeU);

			mmx_updown_row_dst
			(src->GetReadPtr(PLANAR_V), dst->GetWritePtr(PLANAR_V), row_sizeU, heightU, 
			moduloU, dstrow_sizeU, dstmoduloU, dstframesizeU );
			}
			else
//...........................................................................................
			{
				//plain is still here cause was the beginning of this idea
			asm_updown_row_dst
			(src->GetReadPtr(PLANAR_Y), dst->GetWritePtr(PLANAR_Y), row_size, height, 
			modulo, dstrow_size, dstmodulo, dstframesize );
		//if (!vi.IsY8())

			asm_updown_row_dst
			(src->GetReadPtr(PLANAR_U), dst->GetWritePtr(PLANAR_U), row_sizeU, heightU, 
			moduloU, dstrow_sizeU, dstmoduloU, dstframesizeU);

			asm_updown_row_dst
			(src->GetReadPtr(PLANAR_V), dst->GetWritePtr(PLANAR_V), row_sizeU, heightU, 
			moduloU, dstrow_sizeU, dstmoduloU, dstframesizeU );
			}
		}
//...........................................................................................
		else
		{		//this part should work for RGB (32 & 24) and yuy2
			const int src_pitch = src->GetPitch();
			const int height	= src->GetHeight();
			const int row_size	= src->GetRowSize();
			const int modulo	= src_pitch-row_size;

			const int dst_pitch		= dst->GetPitch();
			const int dstheight		= dst->GetHeight();
			const int dstrow_size	= dst->GetRowSize();
			const int dstframesize	= dst_pitch*dstheight;
			const int dstmodulo		= dst_pitch-dstrow_size;
//...........................................................................................
			if(use_SSE2)
			{
				if (DoNta)
				{
				  env->BitBlt(dst->GetWritePtr(), dst_pitch, src->GetReadPtr() + (height-1) * src_pitch,
					  -src_pitch, row_size, height);
				}
				else
				{
				sse2_updown_row_dst
				(src->GetReadPtr(),	dst->GetWritePtr(), row_size, height, modulo, dstrow_size, dstmodulo, dstframesize);
				}
			}
			else if (use_ISSE)
			{
				if (DoNta)
				{
				  env->BitBlt(dst->GetWritePtr(), dst_pitch, src->GetReadPtr() + (height-1) * src_pitch,
				  -src_pitch, row_size, height);
				}
				else
				{
				mmx_updown_row_dst
				(src->GetReadPtr(),	dst->GetWritePtr(), row_size, height, modulo, dstrow_size, dstmodulo, dstframesize );
				}
			}
			else if (use_MMX)
			{
			mmx_updown_row_dst
			(src->GetReadPtr(),	dst->GetWritePtr(), row_size, height, modulo, dstrow_size, dstmodulo, dstframesize );
			}
			else
			{	//plain is still here cause was the beginning of this idea
			asm_updown_row_dst
			(src->GetReadPtr(), dst->GetWritePtr(), row_size, height, modulo, dstrow_size, dstmodulo, dstframesize );
			}
		}
			return dst;
	}
//...........................................................................................
};
/////////////////////////////////////////////////////////////////////////////////////////////
PVideoFrame __stdcall FHorizontal::GetFrame(int n,IScriptEnvironment* env)
{
		PVideoFrame src = child->GetFrame(n, env);

		if (!src->IsWritable())DoInPlace=false;

	if (DoInPlace)
	{
		env->MakeWritable(&src);
//...........................................................................................
		if (vi.IsYUY2())
		{
			if(use_SSE2)
			{
				asm_sse2_yuy2_mirror
				(src->GetWritePtr(), src->GetRowSize(), src->GetHeight(), src->GetPitch()-src->GetRowSize());
			}
			else if(use_ISSE)
			{
				asm_isse_yuy2_mirror2
				(src->GetWritePtr(), src->GetRowSize(), src->GetHeight(), src->GetPitch()-src->GetRowSize());
			}
			else
			{
				asm_MMX_yuy2_mirror2
				(src->GetWritePtr(), src->GetRowSize(), src->GetHeight(), src->GetPitch()-src->GetRowSize());
			}
		}
//...........................................................................................
		else if (vi.IsYV12())		//change to planar when 2.6 and test !Y8//if (vi.IsPlanar())
		{
//...........................................................................................
				const int height	= src->GetHeight(PLANAR_Y);
				const int row_size  = src->GetRowSize(PLANAR_Y);
				const int modulo	= src->GetPitch(PLANAR_Y)-row_size;

				const int heightU	= src->GetHeight(PLANAR_U);
				const int row_sizeU = src->GetRowSize(PLANAR_U);
				const int moduloU	= src->GetPitch(PLANAR_U)-row_sizeU;
//...........................................................................................
			if(use_SSE2)
			{
				asm_mirror_sse2_row_2	
				(src->GetWritePtr(PLANAR_Y), row_size, height, modulo);

		//if (!vi.IsY8())
				asm_mirror_sse2_row_2
				(src->GetWritePtr(PLANAR_U), row_sizeU, heightU, moduloU);

				asm_mirror_sse2_row_2
				(src->GetWritePtr(PLANAR_V), row_sizeU, heightU, moduloU);
			}
//...........................................................................................
			else if(use_ISSE)
			{
				asm_mirror_isse_row_2
				(src->GetWritePtr(PLANAR_Y), row_size, height, modulo);
		//if (!vi.IsY8())
				asm_mirror_isse_row_2
				(src->GetWritePtr(PLANAR_U), row_sizeU, heightU, moduloU);

				asm_mirror_isse_row_2
				(src->GetWritePtr(PLANAR_V), row_sizeU, heightU, moduloU);
			}
//...........................................................................................
			else
			{
				asm_mirror_plain_row_2
				(src->GetWritePtr(PLANAR_Y), row_size, height, modulo);
		//if (!vi.IsY8())
				asm_mirror_plain_row_2
				(src->GetWritePtr(PLANAR_U), row_sizeU, heightU, moduloU);

				asm_mirror_plain_row_2
				(src->GetWritePtr(PLANAR_V), row_sizeU, heightU, moduloU);
			}
		}
//...........................................................................................
		else if (vi.IsRGB32())
		{
			if(use_SSE2)
			{
			asm_mirror_rgb_sse2_exp2
			( src->GetWritePtr(), src->GetRowSize(), src->GetHeight(), src->GetPitch()-src->GetRowSize() );
			}
			else if(use_ISSE)
			{
			asm_mirror_rgb_isse_exp
			( src->GetWritePtr(), src->GetRowSize(), src->GetHeight(), src->GetPitch()-src->GetRowSize() );
			}
			else if(use_MMX)
			{
			asm_mirror_rgb_mmx_exp
			( src->GetWritePtr(), src->GetRowSize(), src->GetHeight(), src->GetPitch()-src->GetRowSize() );
			}
			else
			{
			asm_mirror_rgb_row
			( src->GetWritePtr(), src->GetRowSize(), src->GetHeight(), src->GetPitch()-src->GetRowSize() );
			}
		}
//...........................................................................................
		else
		{
			asm_mirror_rgb24_row
			( src->GetWritePtr(), src->GetRowSize(), src->GetHeight(), src->GetPitch()-src->GetRowSize() );
		}
		return src;
//...........................................................................................
		}
/////////////////////////////////////////////////////////////////////////////////////////////
		else
		{
			PVideoFrame dst = env->NewVideoFrame(vi,32);//64??16
			env->MakeWritable(&dst);

		chsz2=chsz2-(chsz2>>2);	//75 % need to be tuned

		bool DoNta=( (dst->GetPitch()*dst->GetHeight()) >(chsz2) )!=0;
//...........................................................................................
	//use_SSE2=false;
	//use_ISSE=false;
	//use_MMX=false;
		if (vi.IsYUY2())
		{
//...........................................................................................
			if(use_SSE2)
			{
				if(DoNta)
				{
				asm_sse2_yuy2_mirror_dst_nta
				(src->GetReadPtr(), dst->GetWritePtr(),src->GetRowSize(), src->GetHeight(),
				src->GetPitch()-src->GetRowSize(), dst->GetPitch()-dst->GetRowSize() );
				}
				else
				{
				asm_sse2_yuy2_mirror_dst
				(src->GetReadPtr(), dst->GetWritePtr(),src->GetRowSize(), src->GetHeight(),
				src->GetPitch()-src->GetRowSize(), dst->GetPitch()-dst->GetRowSize() );
				}
			}
//...........................................................................................
			else if(use_ISSE)
			{
				if(DoNta)
				{
				asm_isse_yuy2_mirror2_dst_nta
				(src->GetReadPtr(), dst->GetWritePtr(),src->GetRowSize(), src->GetHeight(),
				src->GetPitch()-src->GetRowSize(), dst->GetPitch()-dst->GetRowSize() );
				}
				else
				{
				asm_isse_yuy2_mirror2_dst
				(src->GetReadPtr(), dst->GetWritePtr(),src->GetRowSize(), src->GetHeight(),
				src->GetPitch()-src->GetRowSize(), dst->GetPitch()-dst->GetRowSize() );
				}
			}
//...........................................................................................
			else
			{
				asm_MMX_yuy2_mirror2_dst
				(src->GetReadPtr(), dst->GetWritePtr(),src->GetRowSize(), src->GetHeight(),
				src->GetPitch()-src->GetRowSize(), dst->GetPitch()-dst->GetRowSize() );
			}
		}
//...........................................................................................
		else if (vi.IsYV12())		//change to planar when 2.6 and test Y8//if (!vi.IsPlanar())
		{
		//use_SSE2=false;
		//use_ISSE=false;
//...........................................................................................
			const int height	= src->GetHeight(PLANAR_Y);
			const int row_size  = src->GetRowSize(PLANAR_Y);
			const int modulo	= src->GetPitch(PLANAR_Y)-row_size;
//...........................................................................................
			const int dstrow_size	= dst->GetRowSize(PLANAR_Y);
			const int modulodst		= dst->GetPitch(PLANAR_Y)-dstrow_size;

			const int heightU	= src->GetHeight(PLANAR_U);
			const int row_sizeU = src->GetRowSize(PLANAR_U);
			const int moduloU	= src->GetPitch(PLANAR_U)-row_sizeU;
//...........................................................................................
			const int dstrow_sizeU	= dst->GetRowSize(PLANAR_U);
			const int dstmoduloU	= dst->GetPitch(PLANAR_U)-dstrow_sizeU;
//...........................................................................................
			if(use_SSE2)
			{
				if(DoNta)
				{
				asm_mirror_sse2_row_2_dst_nta
				(src->GetReadPtr(PLANAR_Y), dst->GetWritePtr(PLANAR_Y),row_size,
				height,	modulo, modulodst);
				}
				else
				{
				asm_mirror_sse2_row_2_dst
				(src->GetReadPtr(PLANAR_Y), dst->GetWritePtr(PLANAR_Y),row_size,
				height,	modulo, modulodst);
				}

		//if (!vi.IsY8())
				if ( ((src->GetPitch(PLANAR_U)*heightU) ) >(chsz2))
				{
				asm_mirror_sse2_row_2_dst_nta
				(src->GetReadPtr(PLANAR_U), dst->GetWritePtr(PLANAR_U),row_sizeU,
				heightU,moduloU, dstmoduloU);

				asm_mirror_sse2_row_2_dst_nta
				(src->GetReadPtr(PLANAR_V), dst->GetWritePtr(PLANAR_V),row_sizeU,
				heightU,moduloU, dstmoduloU);
				}
				else
				{
				asm_mirror_sse2_row_2_dst
				(src->GetReadPtr(PLANAR_U), dst->GetWritePtr(PLANAR_U),row_sizeU,
				heightU, moduloU, dstmoduloU);

				asm_mirror_sse2_row_2_dst
				(src->GetReadPtr(PLANAR_V), dst->GetWritePtr(PLANAR_V),row_sizeU,
				heightU, moduloU, dstmoduloU);
				}
			}
//...........................................................................................
			else if(use_ISSE)
			{
				if(DoNta)
				{
				asm_mirror_isse_row_2_dst_nta
				(src->GetReadPtr(PLANAR_Y), dst->GetWritePtr(PLANAR_Y),row_size,
				height,	modulo, modulodst);
				}
				else
				{
				asm_mirror_isse_row_2_dst
				(src->GetReadPtr(PLANAR_Y), dst->GetWritePtr(PLANAR_Y),row_size,
				height,	modulo, modulodst);
				}

		//if (!vi.IsY8())
				if ( ((src->GetPitch(PLANAR_U)*heightU) + (dst->GetPitch(PLANAR_U)*heightU)) >(chsz2))
				{
				asm_mirror_isse_row_2_dst_nta
				(src->GetReadPtr(PLANAR_U), dst->GetWritePtr(PLANAR_U),row_sizeU,
				heightU,moduloU, dstmoduloU);

				asm_mirror_isse_row_2_dst_nta
				(src->GetReadPtr(PLANAR_V), dst->GetWritePtr(PLANAR_V),row_sizeU,
				heightU,moduloU, dstmoduloU);
				}
				else
				{
				asm_mirror_isse_row_2_dst
				(src->GetReadPtr(PLANAR_U), dst->GetWritePtr(PLANAR_U),row_sizeU,
				heightU,moduloU, dstmoduloU);

				asm_mirror_isse_row_2_dst
				(src->GetReadPtr(PLANAR_V), dst->GetWritePtr(PLANAR_V),row_sizeU,
				heightU,moduloU, dstmoduloU);
				}
			}
//...........................................................................................
			else
			{
				asm_mirror_plain_row_1_dst
				(src->GetReadPtr(PLANAR_Y), dst->GetWritePtr(PLANAR_Y),row_size,
				height,	modulo, modulodst);

		//if (!vi.IsY8())
				asm_mirror_plain_row_1_dst
				(src->GetReadPtr(PLANAR_U), dst->GetWritePtr(PLANAR_U),row_sizeU,
				heightU,moduloU, dstmoduloU);

				asm_mirror_plain_row_1_dst
				(src->GetReadPtr(PLANAR_V), dst->GetWritePtr(PLANAR_V),row_sizeU,
				heightU,moduloU, dstmoduloU);
			}
		}
//...........................................................................................
		else if (vi.IsRGB32())
		{
//...........................................................................................
			if(use_SSE2)
			{
			  if(DoNta) 
			  {
				asm_mirror_rgb_sse2_exp2_dst_nta
				(src->GetReadPtr(), dst->GetWritePtr(),src->GetRowSize(), src->GetHeight(),
				src->GetPitch()-src->GetRowSize(), dst->GetPitch()-dst->GetRowSize() );
			  }
			  else
			  {
				asm_mirror_rgb_sse2_exp2_dst
				(src->GetReadPtr(), dst->GetWritePtr(),src->GetRowSize(), src->GetHeight(),
				src->GetPitch()-src->GetRowSize(), dst->GetPitch()-dst->GetRowSize() );
			  }
			}
//...........................................................................................
			else if(use_ISSE)
			{
			  if(DoNta) 
			  {
				asm_mirror_rgb_isse_exp_dst_nta
				(src->GetReadPtr(), dst->GetWritePtr(),src->GetRowSize(), src->GetHeight(),
				src->GetPitch()-src->GetRowSize(), dst->GetPitch()-dst->GetRowSize() );
			  }
			  else
			  {
				asm_mirror_rgb_isse_exp_dst
				(src->GetReadPtr(), dst->GetWritePtr(),src->GetRowSize(), src->GetHeight(),
				src->GetPitch()-src->GetRowSize(), dst->GetPitch()-dst->GetRowSize() );
			  }
			}
//...........................................................................................
			else if(use_MMX)
			{
				asm_mirror_rgb_mmx_exp_dst
				(src->GetReadPtr(), dst->GetWritePtr(),src->GetRowSize(), src->GetHeight(),
				src->GetPitch()-src->GetRowSize(), dst->GetPitch()-dst->GetRowSize() );
			}
			else
			{
				asm_mirror_rgb_row_dst
				(src->GetReadPtr(), dst->GetWritePtr(),src->GetRowSize(), src->GetHeight(),
				src->GetPitch()-src->GetRowSize(), dst->GetPitch()-dst->GetRowSize() );
			}
		}
//...........................................................................................
		else
		{
			asm_mirror_rgb24_row_dst
			(src->GetReadPtr(), dst->GetWritePtr(),src->GetRowSize(), src->GetHeight(),
			src->GetPitch()-src->GetRowSize(), dst->GetPitch()-dst->GetRowSize() );
		}
		return dst;
		}
};
/////////////////////////////////////////////////////////////////////////////////////////////
AVSValue __cdecl Create_FHorizontal
(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	return new FHorizontal
				(args[0].AsClip(),
				env);
}
/////////////////////////////////////////////////////////////////////////////////////////////
AVSValue __cdecl Create_FVertical
(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	return new FVertical
				(args[0].AsClip(),
				env);
}
/////////////////////////////////////////////////////////////////////////////////////////////
AVSValue __cdecl Create_Rotate180
(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	return new Rotate180
				(args[0].AsClip(),
				env);
}
/////////////////////////////////////////////////////////////////////////////////////////////
extern "C" __declspec(dllexport) const char*__stdcall
 AvisynthPluginInit2(IScriptEnvironment* env)
{
	env->AddFunction("Rotate180", "c", Create_Rotate180, 0);
	env->AddFunction("FVertical", "c", Create_FVertical, 0);
	env->AddFunction("FHorizontal", "c", Create_FHorizontal, 0);
	return 0;
}
//////////////////////////////////////////END////////////////////////////////////////////////
